searchData={"items":[{"type":"module","title":"AshCommanded","doc":"AshCommanded is an extension for the Ash Framework that provides Command Query Responsibility Segregation (CQRS)\nand Event Sourcing (ES) patterns using the Commanded library.\n\nThis extension enables declarative definition of commands, events, and projections within Ash resources.","ref":"AshCommanded.html"},{"type":"module","title":"AshCommanded.Commanded.Command","doc":"Represents a command in the Commanded DSL.\n\nCommands define actions that can be performed on a resource, resulting in events.","ref":"AshCommanded.Commanded.Command.html"},{"type":"type","title":"AshCommanded.Commanded.Command.t/0","doc":"","ref":"AshCommanded.Commanded.Command.html#t:t/0"},{"type":"module","title":"AshCommanded.Commanded.CommandActionMapper","doc":"Provides utilities for mapping between Commanded commands and Ash actions.\n\nThis module enhances the integration between commands and actions, providing:\n\n1. Advanced parameter transformation between commands and actions\n2. Support for different action types (create, update, destroy, etc.)\n3. Helper functions for command handling\n4. Utilities for applying actions with proper context\n5. Standardized error handling and reporting\n6. Transaction support for atomic command execution\n\nThese utilities are used by generated code (command handlers, aggregates, etc.)\nbut can also be used directly in custom implementations.","ref":"AshCommanded.Commanded.CommandActionMapper.html"},{"type":"function","title":"AshCommanded.Commanded.CommandActionMapper.infer_action_type/1","doc":"Infers the action type based on the action name.","ref":"AshCommanded.Commanded.CommandActionMapper.html#infer_action_type/1"},{"type":"function","title":"Examples - AshCommanded.Commanded.CommandActionMapper.infer_action_type/1","doc":"iex> infer_action_type(:create)\n    :create\n    \n    iex> infer_action_type(:update_email)\n    :update\n    \n    iex> infer_action_type(:destroy_account)\n    :destroy\n    \n    iex> infer_action_type(:custom_operation)\n    :custom","ref":"AshCommanded.Commanded.CommandActionMapper.html#infer_action_type/1-examples"},{"type":"function","title":"AshCommanded.Commanded.CommandActionMapper.map_to_action/4","doc":"Maps a command to an Ash action and executes it.","ref":"AshCommanded.Commanded.CommandActionMapper.html#map_to_action/4"},{"type":"function","title":"Parameters - AshCommanded.Commanded.CommandActionMapper.map_to_action/4","doc":"* `command` - The command struct to execute\n* `resource` - The Ash resource module\n* `action_name` - The name of the Ash action to call\n* `opts` - Additional options for the action mapping","ref":"AshCommanded.Commanded.CommandActionMapper.html#map_to_action/4-parameters"},{"type":"function","title":"Options - AshCommanded.Commanded.CommandActionMapper.map_to_action/4","doc":"* `:action_type` - The type of action (:create, :update, :destroy, :read, or :custom)\n* `:identity_field` - The field used to identify the record (default: :id)\n* `:param_mapping` - A map or function for transforming command fields to action params\n* `:context` - Context to pass to the Ash action\n* `:before_action` - Function to call before executing the action\n* `:after_action` - Function to call after executing the action\n* `:transforms` - List of parameter transformations to apply\n* `:validations` - List of parameter validations to apply\n* `:in_transaction?` - Whether to execute the command in a transaction (default: false)\n* `:repo` - Repository to use for transaction (required if in_transaction? is true)\n* `:transaction_opts` - Options for the transaction (timeout, isolation_level)","ref":"AshCommanded.Commanded.CommandActionMapper.html#map_to_action/4-options"},{"type":"function","title":"Return Values - AshCommanded.Commanded.CommandActionMapper.map_to_action/4","doc":"* `{:ok, record}` - The action was executed successfully, returning the record\n* `{:error, error}` - An error occurred. The error will be a standardized AshCommanded.Commanded.Error or list of errors","ref":"AshCommanded.Commanded.CommandActionMapper.html#map_to_action/4-return-values"},{"type":"function","title":"Examples - AshCommanded.Commanded.CommandActionMapper.map_to_action/4","doc":"Basic usage with default mappings:\n\n    map_to_action(%MyApp.Commands.RegisterUser{}, MyApp.User, :create)\n    \nWith custom parameter mapping:\n\n    map_to_action(\n      %MyApp.Commands.UpdateEmail{id: \"123\", email: \"new@example.com\"},\n      MyApp.User,\n      :update_email,\n      action_type: :update,\n      param_mapping: %{email: :new_email},\n      context: %{actor: current_user}\n    )\n\nWith transformation functions:\n\n    map_to_action(\n      %MyApp.Commands.CreateProduct{},\n      MyApp.Product,\n      :create,\n      param_mapping: fn cmd -> \n        Map.put(cmd, :created_at, DateTime.utc_now())\n      end\n    )\n    \nWith transaction support:\n\n    map_to_action(\n      %MyApp.Commands.RegisterUser{},\n      MyApp.User,\n      :create,\n      in_transaction?: true,\n      repo: MyApp.Repo,\n      transaction_opts: [timeout: 30_000]\n    )","ref":"AshCommanded.Commanded.CommandActionMapper.html#map_to_action/4-examples"},{"type":"function","title":"AshCommanded.Commanded.CommandActionMapper.transactional_map_to_action/3","doc":"Maps and executes multiple commands in a single transaction.","ref":"AshCommanded.Commanded.CommandActionMapper.html#transactional_map_to_action/3"},{"type":"function","title":"Parameters - AshCommanded.Commanded.CommandActionMapper.transactional_map_to_action/3","doc":"* `commands` - List of command specifications\n* `repo` - Repository to use for the transaction\n* `opts` - Transaction options","ref":"AshCommanded.Commanded.CommandActionMapper.html#transactional_map_to_action/3-parameters"},{"type":"function","title":"Command Specifications - AshCommanded.Commanded.CommandActionMapper.transactional_map_to_action/3","doc":"Each command specification is a map with the following keys:\n* `:command` - The command struct to execute\n* `:resource` - The Ash resource module\n* `:action` - The action name\n* `:opts` - (Optional) Command-specific options","ref":"AshCommanded.Commanded.CommandActionMapper.html#transactional_map_to_action/3-command-specifications"},{"type":"function","title":"Return Values - AshCommanded.Commanded.CommandActionMapper.transactional_map_to_action/3","doc":"* `{:ok, results}` - All commands executed successfully, with a map of results\n* `{:error, failed_operation, error, results_so_far}` - Transaction failed","ref":"AshCommanded.Commanded.CommandActionMapper.html#transactional_map_to_action/3-return-values"},{"type":"function","title":"Examples - AshCommanded.Commanded.CommandActionMapper.transactional_map_to_action/3","doc":"```elixir\nAshCommanded.Commanded.CommandActionMapper.transactional_map_to_action([\n  %{\n    command: %MyApp.Commands.RegisterUser{name: \"John\", email: \"john@example.com\"},\n    resource: MyApp.User,\n    action: :create\n  },\n  %{\n    command: %MyApp.Commands.CreateProfile{user_id: \"123\", bio: \"Developer\"},\n    resource: MyApp.Profile,\n    action: :create,\n    opts: [param_mapping: %{user_id: :owner_id}]\n  }\n], MyApp.Repo)\n```","ref":"AshCommanded.Commanded.CommandActionMapper.html#transactional_map_to_action/3-examples"},{"type":"module","title":"AshCommanded.Commanded.Error","doc":"Standardized error handling for AshCommanded.\n\nThis module provides a consistent approach to error handling across the AshCommanded extension.\nIt defines error types, formatting, and helper functions for error creation and handling.","ref":"AshCommanded.Commanded.Error.html"},{"type":"function","title":"AshCommanded.Commanded.Error.action_error/2","doc":"Creates an action error.","ref":"AshCommanded.Commanded.Error.html#action_error/2"},{"type":"function","title":"Parameters - AshCommanded.Commanded.Error.action_error/2","doc":"* `message` - The action error message\n  * `opts` - Additional options for the error","ref":"AshCommanded.Commanded.Error.html#action_error/2-parameters"},{"type":"function","title":"Examples - AshCommanded.Commanded.Error.action_error/2","doc":"iex> AshCommanded.Commanded.Error.action_error(\"Failed to execute action\", context: %{action: :create_user})\n    %AshCommanded.Commanded.Error{\n      type: :action_error,\n      message: \"Failed to execute action\",\n      path: [],\n      field: nil,\n      value: nil,\n      context: %{action: :create_user}\n    }","ref":"AshCommanded.Commanded.Error.html#action_error/2-examples"},{"type":"function","title":"AshCommanded.Commanded.Error.aggregate_error/2","doc":"Creates an aggregate error.","ref":"AshCommanded.Commanded.Error.html#aggregate_error/2"},{"type":"function","title":"Parameters - AshCommanded.Commanded.Error.aggregate_error/2","doc":"* `message` - The aggregate error message\n  * `opts` - Additional options for the error","ref":"AshCommanded.Commanded.Error.html#aggregate_error/2-parameters"},{"type":"function","title":"Examples - AshCommanded.Commanded.Error.aggregate_error/2","doc":"iex> AshCommanded.Commanded.Error.aggregate_error(\"Aggregate not found\", context: %{aggregate_id: \"123\"})\n    %AshCommanded.Commanded.Error{\n      type: :aggregate_error,\n      message: \"Aggregate not found\",\n      path: [],\n      field: nil,\n      value: nil,\n      context: %{aggregate_id: \"123\"}\n    }","ref":"AshCommanded.Commanded.Error.html#aggregate_error/2-examples"},{"type":"function","title":"AshCommanded.Commanded.Error.command_error/2","doc":"Creates a command error.","ref":"AshCommanded.Commanded.Error.html#command_error/2"},{"type":"function","title":"Parameters - AshCommanded.Commanded.Error.command_error/2","doc":"* `message` - The command error message\n  * `opts` - Additional options for the error","ref":"AshCommanded.Commanded.Error.html#command_error/2-parameters"},{"type":"function","title":"Examples - AshCommanded.Commanded.Error.command_error/2","doc":"iex> AshCommanded.Commanded.Error.command_error(\"Command is missing required field\", field: :id)\n    %AshCommanded.Commanded.Error{\n      type: :command_error,\n      message: \"Command is missing required field\",\n      path: [],\n      field: :id,\n      value: nil,\n      context: %{}\n    }","ref":"AshCommanded.Commanded.Error.html#command_error/2-examples"},{"type":"function","title":"AshCommanded.Commanded.Error.dispatch_error/2","doc":"Creates a dispatch error.","ref":"AshCommanded.Commanded.Error.html#dispatch_error/2"},{"type":"function","title":"Parameters - AshCommanded.Commanded.Error.dispatch_error/2","doc":"* `message` - The dispatch error message\n  * `opts` - Additional options for the error","ref":"AshCommanded.Commanded.Error.html#dispatch_error/2-parameters"},{"type":"function","title":"Examples - AshCommanded.Commanded.Error.dispatch_error/2","doc":"iex> AshCommanded.Commanded.Error.dispatch_error(\"Failed to dispatch command\", context: %{command: \"RegisterUser\"})\n    %AshCommanded.Commanded.Error{\n      type: :dispatch_error,\n      message: \"Failed to dispatch command\",\n      path: [],\n      field: nil,\n      value: nil,\n      context: %{command: \"RegisterUser\"}\n    }","ref":"AshCommanded.Commanded.Error.html#dispatch_error/2-examples"},{"type":"type","title":"AshCommanded.Commanded.Error.error_type/0","doc":"Types of errors that can occur in AshCommanded","ref":"AshCommanded.Commanded.Error.html#t:error_type/0"},{"type":"function","title":"AshCommanded.Commanded.Error.errors_from_result/1","doc":"Returns the errors from a result.","ref":"AshCommanded.Commanded.Error.html#errors_from_result/1"},{"type":"function","title":"Parameters - AshCommanded.Commanded.Error.errors_from_result/1","doc":"* `result` - The result to extract errors from","ref":"AshCommanded.Commanded.Error.html#errors_from_result/1-parameters"},{"type":"function","title":"Examples - AshCommanded.Commanded.Error.errors_from_result/1","doc":"iex> error = %AshCommanded.Commanded.Error{type: :validation_error, message: \"Invalid\"}\n    iex> AshCommanded.Commanded.Error.errors_from_result({:error, error})\n    [%AshCommanded.Commanded.Error{type: :validation_error, message: \"Invalid\"}]\n\n    iex> AshCommanded.Commanded.Error.errors_from_result({:error, [%AshCommanded.Commanded.Error{type: :validation_error, message: \"Invalid\"}]})\n    [%AshCommanded.Commanded.Error{type: :validation_error, message: \"Invalid\"}]\n\n    iex> AshCommanded.Commanded.Error.errors_from_result({:ok, \"Success\"})\n    []","ref":"AshCommanded.Commanded.Error.html#errors_from_result/1-examples"},{"type":"function","title":"AshCommanded.Commanded.Error.format/1","doc":"Formats an error for display.","ref":"AshCommanded.Commanded.Error.html#format/1"},{"type":"function","title":"Parameters - AshCommanded.Commanded.Error.format/1","doc":"* `error` - The error to format","ref":"AshCommanded.Commanded.Error.html#format/1-parameters"},{"type":"function","title":"Examples - AshCommanded.Commanded.Error.format/1","doc":"iex> error = AshCommanded.Commanded.Error.validation_error(\"Value must be positive\", field: :age, value: -5)\n    iex> AshCommanded.Commanded.Error.format(error)\n    \"Validation error: Value must be positive (field: age, value: -5)\"","ref":"AshCommanded.Commanded.Error.html#format/1-examples"},{"type":"function","title":"AshCommanded.Commanded.Error.from_ash_error/1","doc":"Converts an Ash error to an AshCommanded error.","ref":"AshCommanded.Commanded.Error.html#from_ash_error/1"},{"type":"function","title":"Parameters - AshCommanded.Commanded.Error.from_ash_error/1","doc":"* `ash_error` - The Ash error to convert","ref":"AshCommanded.Commanded.Error.html#from_ash_error/1-parameters"},{"type":"function","title":"Examples - AshCommanded.Commanded.Error.from_ash_error/1","doc":"iex> AshCommanded.Commanded.Error.from_ash_error(%Ash.Error.Invalid{errors: [%Ash.Error.Changes.InvalidAttribute{field: :name, message: \"can't be blank\"}]})\n    %AshCommanded.Commanded.Error{\n      type: :validation_error,\n      message: \"can't be blank\",\n      path: [],\n      field: :name,\n      value: nil,\n      context: %{source: :ash}\n    }","ref":"AshCommanded.Commanded.Error.html#from_ash_error/1-examples"},{"type":"function","title":"AshCommanded.Commanded.Error.from_commanded_error/1","doc":"Converts a Commanded error to an AshCommanded error.","ref":"AshCommanded.Commanded.Error.html#from_commanded_error/1"},{"type":"function","title":"Parameters - AshCommanded.Commanded.Error.from_commanded_error/1","doc":"* `commanded_error` - The Commanded error to convert","ref":"AshCommanded.Commanded.Error.html#from_commanded_error/1-parameters"},{"type":"function","title":"Examples - AshCommanded.Commanded.Error.from_commanded_error/1","doc":"iex> AshCommanded.Commanded.Error.from_commanded_error(%Commanded.Aggregates.ExecutionError{message: \"Failed to execute command\"})\n    %AshCommanded.Commanded.Error{\n      type: :aggregate_error,\n      message: \"Failed to execute command\",\n      path: [],\n      field: nil,\n      value: nil,\n      context: %{source: :commanded}\n    }","ref":"AshCommanded.Commanded.Error.html#from_commanded_error/1-examples"},{"type":"function","title":"AshCommanded.Commanded.Error.is_error?/1","doc":"Returns whether a value is an AshCommanded error.","ref":"AshCommanded.Commanded.Error.html#is_error?/1"},{"type":"function","title":"Parameters - AshCommanded.Commanded.Error.is_error?/1","doc":"* `value` - The value to check","ref":"AshCommanded.Commanded.Error.html#is_error?/1-parameters"},{"type":"function","title":"Examples - AshCommanded.Commanded.Error.is_error?/1","doc":"iex> AshCommanded.Commanded.Error.is_error?(%AshCommanded.Commanded.Error{type: :validation_error, message: \"Invalid\"})\n    true\n\n    iex> AshCommanded.Commanded.Error.is_error?(\"Not an error\")\n    false","ref":"AshCommanded.Commanded.Error.html#is_error?/1-examples"},{"type":"function","title":"AshCommanded.Commanded.Error.is_error_result?/1","doc":"Returns whether a result is an error result.","ref":"AshCommanded.Commanded.Error.html#is_error_result?/1"},{"type":"function","title":"Parameters - AshCommanded.Commanded.Error.is_error_result?/1","doc":"* `result` - The result to check","ref":"AshCommanded.Commanded.Error.html#is_error_result?/1-parameters"},{"type":"function","title":"Examples - AshCommanded.Commanded.Error.is_error_result?/1","doc":"iex> AshCommanded.Commanded.Error.is_error_result?({:error, %AshCommanded.Commanded.Error{}})\n    true\n\n    iex> AshCommanded.Commanded.Error.is_error_result?({:ok, \"Success\"})\n    false","ref":"AshCommanded.Commanded.Error.html#is_error_result?/1-examples"},{"type":"function","title":"AshCommanded.Commanded.Error.new/3","doc":"Creates a new error with the specified type and message.","ref":"AshCommanded.Commanded.Error.html#new/3"},{"type":"function","title":"Parameters - AshCommanded.Commanded.Error.new/3","doc":"* `type` - The type of error\n  * `message` - The error message\n  * `opts` - Additional options for the error","ref":"AshCommanded.Commanded.Error.html#new/3-parameters"},{"type":"function","title":"Options - AshCommanded.Commanded.Error.new/3","doc":"* `:path` - The path to the error in the data structure\n  * `:field` - The specific field that caused the error\n  * `:value` - The value that caused the error\n  * `:context` - Additional context about the error","ref":"AshCommanded.Commanded.Error.html#new/3-options"},{"type":"function","title":"Examples - AshCommanded.Commanded.Error.new/3","doc":"iex> AshCommanded.Commanded.Error.new(:validation_error, \"Field must be a string\")\n    %AshCommanded.Commanded.Error{\n      type: :validation_error,\n      message: \"Field must be a string\",\n      path: [],\n      field: nil,\n      value: nil,\n      context: %{}\n    }\n\n    iex> AshCommanded.Commanded.Error.new(:validation_error, \"Field must be a string\", field: :name, value: 123)\n    %AshCommanded.Commanded.Error{\n      type: :validation_error, \n      message: \"Field must be a string\",\n      path: [],\n      field: :name,\n      value: 123,\n      context: %{}\n    }","ref":"AshCommanded.Commanded.Error.html#new/3-examples"},{"type":"function","title":"AshCommanded.Commanded.Error.normalize_error/1","doc":"Converts a single error to a standardized format.","ref":"AshCommanded.Commanded.Error.html#normalize_error/1"},{"type":"function","title":"Parameters - AshCommanded.Commanded.Error.normalize_error/1","doc":"* `error` - The error to normalize","ref":"AshCommanded.Commanded.Error.html#normalize_error/1-parameters"},{"type":"function","title":"Examples - AshCommanded.Commanded.Error.normalize_error/1","doc":"iex> AshCommanded.Commanded.Error.normalize_error(%Ash.Error.Changes.InvalidAttribute{field: :name, message: \"can't be blank\"})\n    [%AshCommanded.Commanded.Error{type: :validation_error, message: \"can't be blank\", field: :name, context: %{source: :ash}}]","ref":"AshCommanded.Commanded.Error.html#normalize_error/1-examples"},{"type":"function","title":"AshCommanded.Commanded.Error.normalize_errors/1","doc":"Converts a list of errors to a standardized format.","ref":"AshCommanded.Commanded.Error.html#normalize_errors/1"},{"type":"function","title":"Parameters - AshCommanded.Commanded.Error.normalize_errors/1","doc":"* `errors` - The list of errors to normalize","ref":"AshCommanded.Commanded.Error.html#normalize_errors/1-parameters"},{"type":"function","title":"Examples - AshCommanded.Commanded.Error.normalize_errors/1","doc":"iex> errors = [\n    ...>   %Ash.Error.Changes.InvalidAttribute{field: :name, message: \"can't be blank\"},\n    ...>   %Commanded.Aggregates.ExecutionError{message: \"Failed to execute command\"}\n    ...> ]\n    iex> AshCommanded.Commanded.Error.normalize_errors(errors)\n    [\n      %AshCommanded.Commanded.Error{type: :validation_error, message: \"can't be blank\", field: :name, context: %{source: :ash}},\n      %AshCommanded.Commanded.Error{type: :aggregate_error, message: \"Failed to execute command\", context: %{source: :commanded}}\n    ]","ref":"AshCommanded.Commanded.Error.html#normalize_errors/1-examples"},{"type":"function","title":"AshCommanded.Commanded.Error.projection_error/2","doc":"Creates a projection error.","ref":"AshCommanded.Commanded.Error.html#projection_error/2"},{"type":"function","title":"Parameters - AshCommanded.Commanded.Error.projection_error/2","doc":"* `message` - The projection error message\n  * `opts` - Additional options for the error","ref":"AshCommanded.Commanded.Error.html#projection_error/2-parameters"},{"type":"function","title":"Examples - AshCommanded.Commanded.Error.projection_error/2","doc":"iex> AshCommanded.Commanded.Error.projection_error(\"Failed to apply projection\", context: %{event: \"UserRegistered\"})\n    %AshCommanded.Commanded.Error{\n      type: :projection_error,\n      message: \"Failed to apply projection\",\n      path: [],\n      field: nil,\n      value: nil,\n      context: %{event: \"UserRegistered\"}\n    }","ref":"AshCommanded.Commanded.Error.html#projection_error/2-examples"},{"type":"type","title":"AshCommanded.Commanded.Error.t/0","doc":"Standard error structure for AshCommanded","ref":"AshCommanded.Commanded.Error.html#t:t/0"},{"type":"function","title":"AshCommanded.Commanded.Error.transformation_error/2","doc":"Creates a transformation error.","ref":"AshCommanded.Commanded.Error.html#transformation_error/2"},{"type":"function","title":"Parameters - AshCommanded.Commanded.Error.transformation_error/2","doc":"* `message` - The transformation error message\n  * `opts` - Additional options for the error","ref":"AshCommanded.Commanded.Error.html#transformation_error/2-parameters"},{"type":"function","title":"Examples - AshCommanded.Commanded.Error.transformation_error/2","doc":"iex> AshCommanded.Commanded.Error.transformation_error(\"Failed to cast value to integer\", field: :age, value: \"abc\")\n    %AshCommanded.Commanded.Error{\n      type: :transformation_error,\n      message: \"Failed to cast value to integer\",\n      path: [],\n      field: :age,\n      value: \"abc\",\n      context: %{}\n    }","ref":"AshCommanded.Commanded.Error.html#transformation_error/2-examples"},{"type":"function","title":"AshCommanded.Commanded.Error.validation_error/2","doc":"Creates a validation error.","ref":"AshCommanded.Commanded.Error.html#validation_error/2"},{"type":"function","title":"Parameters - AshCommanded.Commanded.Error.validation_error/2","doc":"* `message` - The validation error message\n  * `opts` - Additional options for the error","ref":"AshCommanded.Commanded.Error.html#validation_error/2-parameters"},{"type":"function","title":"Examples - AshCommanded.Commanded.Error.validation_error/2","doc":"iex> AshCommanded.Commanded.Error.validation_error(\"Value must be positive\", field: :age, value: -5)\n    %AshCommanded.Commanded.Error{\n      type: :validation_error,\n      message: \"Value must be positive\",\n      path: [],\n      field: :age,\n      value: -5,\n      context: %{}\n    }","ref":"AshCommanded.Commanded.Error.html#validation_error/2-examples"},{"type":"module","title":"AshCommanded.Commanded.Event","doc":"Represents an event in the Commanded DSL.\n\nEvents represent facts that have occurred in the system and are emitted by commands.","ref":"AshCommanded.Commanded.Event.html"},{"type":"type","title":"AshCommanded.Commanded.Event.t/0","doc":"","ref":"AshCommanded.Commanded.Event.html#t:t/0"},{"type":"module","title":"AshCommanded.Commanded.EventHandler","doc":"Represents a general purpose event handler in the Commanded DSL.\n\nEvent handlers respond to specific events with custom logic that can\nperform side effects, interact with external systems, or trigger\nother actions that don't necessarily update the resource state.","ref":"AshCommanded.Commanded.EventHandler.html"},{"type":"type","title":"AshCommanded.Commanded.EventHandler.t/0","doc":"","ref":"AshCommanded.Commanded.EventHandler.html#t:t/0"},{"type":"module","title":"AshCommanded.Commanded.Middleware.BaseMiddleware","doc":"Base module for implementing command middleware in AshCommanded.\n\nThis module provides default implementations and helper functions\nfor common middleware patterns. By using this module, you can implement\nonly the callbacks you need while inheriting sensible defaults.","ref":"AshCommanded.Commanded.Middleware.BaseMiddleware.html"},{"type":"module","title":"Examples - AshCommanded.Commanded.Middleware.BaseMiddleware","doc":"```elixir\ndefmodule MyApp.LoggingMiddleware do\n  use AshCommanded.Commanded.Middleware.BaseMiddleware\n  \n  @impl true\n  def before_dispatch(command, context, next) do\n    # Log command being dispatched\n    IO.inspect(command, label: \"Dispatching command\")\n    next.(command, context)\n  end\n  \n  @impl true\n  def after_dispatch({:ok, result} = success, command, _context) do\n    # Log successful execution\n    IO.inspect(result, label: \"Command succeeded\")\n    success\n  end\n  \n  def after_dispatch({:error, reason} = error, command, _context) do\n    # Log failed execution\n    IO.inspect(reason, label: \"Command failed\")\n    error\n  end\nend\n```","ref":"AshCommanded.Commanded.Middleware.BaseMiddleware.html#module-examples"},{"type":"module","title":"Default Implementations - AshCommanded.Commanded.Middleware.BaseMiddleware","doc":"By default, this module:\n- Passes the command and context unchanged to the next middleware\n- Returns the result from the command handler unchanged","ref":"AshCommanded.Commanded.Middleware.BaseMiddleware.html#module-default-implementations"},{"type":"macro","title":"AshCommanded.Commanded.Middleware.BaseMiddleware.__using__/1","doc":"Using this module will implement the CommandMiddleware behavior.","ref":"AshCommanded.Commanded.Middleware.BaseMiddleware.html#__using__/1"},{"type":"function","title":"AshCommanded.Commanded.Middleware.BaseMiddleware.apply_middleware/4","doc":"Helper to apply a list of middleware to a command.","ref":"AshCommanded.Commanded.Middleware.BaseMiddleware.html#apply_middleware/4"},{"type":"function","title":"Parameters - AshCommanded.Commanded.Middleware.BaseMiddleware.apply_middleware/4","doc":"* `middleware` - List of middleware modules to apply\n* `command` - The command to process\n* `context` - The context for the command\n* `final_handler` - Function to call after all middleware has been applied","ref":"AshCommanded.Commanded.Middleware.BaseMiddleware.html#apply_middleware/4-parameters"},{"type":"function","title":"Examples - AshCommanded.Commanded.Middleware.BaseMiddleware.apply_middleware/4","doc":"```elixir\napply_middleware(\n  [LoggingMiddleware, ValidationMiddleware],\n  %MyCommand{},\n  %{},\n  fn cmd, ctx -> {:ok, dispatch_command(cmd)} end\n)\n```","ref":"AshCommanded.Commanded.Middleware.BaseMiddleware.html#apply_middleware/4-examples"},{"type":"behaviour","title":"AshCommanded.Commanded.Middleware.CommandMiddleware","doc":"Protocol defining the behavior for command middleware in AshCommanded.\n\nCommand middleware allow you to intercept, modify, or enhance commands\nbefore they're processed. This is useful for cross-cutting concerns like:\n\n- Validation\n- Logging\n- Authorization\n- Rate limiting\n- Auditing\n- Parameter transformation\n- Error handling\n\nMiddleware is applied in the order it's defined, with each middleware able to:\n1. Modify the command\n2. Pass it to the next middleware\n3. Short-circuit the chain by returning an error","ref":"AshCommanded.Commanded.Middleware.CommandMiddleware.html"},{"type":"behaviour","title":"Implementing Middleware - AshCommanded.Commanded.Middleware.CommandMiddleware","doc":"To create custom middleware, implement this protocol:\n\n```elixir\ndefmodule MyApp.MyMiddleware do\n  @behaviour AshCommanded.Commanded.Middleware.CommandMiddleware\n  \n  @impl true\n  def before_dispatch(%{} = command, context, next) do\n    # Modify command or context here\n    modified_command = update_in(command.metadata, &Map.put(&1, :traced, true))\n    modified_context = Map.put(context, :middleware_timestamp, DateTime.utc_now())\n    \n    # Call the next middleware in the chain\n    next.(modified_command, modified_context)\n  end\n  \n  @impl true\n  def after_dispatch({:ok, result} = success, _command, _context) do\n    # Handle successful result\n    success\n  end\n  \n  def after_dispatch({:error, _reason} = error, _command, _context) do\n    # Handle errors\n    error\n  end\nend\n```","ref":"AshCommanded.Commanded.Middleware.CommandMiddleware.html#module-implementing-middleware"},{"type":"behaviour","title":"Using Middleware - AshCommanded.Commanded.Middleware.CommandMiddleware","doc":"Middleware can be specified globally for all commands, per resource, or per command:\n\n```elixir\n# Global middleware in application config\nconfig :ash_commanded, :global_middleware, [\n  MyApp.LoggingMiddleware,\n  MyApp.AuditingMiddleware\n]\n\n# Resource-level middleware\ndefmodule MyApp.User do\n  use Ash.Resource,\n    extensions: [AshCommanded.Commanded.Dsl]\n    \n  commanded do\n    middleware [\n      MyApp.AuthorizationMiddleware,\n      MyApp.ValidationMiddleware\n    ]\n    \n    commands do\n      # Command-specific middleware\n      command :register_user do\n        fields [:id, :email, :name]\n        middleware [MyApp.RegistrationMiddleware]\n      end\n    end\n  end\nend\n```","ref":"AshCommanded.Commanded.Middleware.CommandMiddleware.html#module-using-middleware"},{"type":"callback","title":"AshCommanded.Commanded.Middleware.CommandMiddleware.after_dispatch/3","doc":"Called after a command is dispatched.","ref":"AshCommanded.Commanded.Middleware.CommandMiddleware.html#c:after_dispatch/3"},{"type":"callback","title":"Parameters - AshCommanded.Commanded.Middleware.CommandMiddleware.after_dispatch/3","doc":"* `result` - The result of the command handler\n* `command` - The original command that was dispatched\n* `context` - The context for this command dispatch","ref":"AshCommanded.Commanded.Middleware.CommandMiddleware.html#c:after_dispatch/3-parameters"},{"type":"callback","title":"Returns - AshCommanded.Commanded.Middleware.CommandMiddleware.after_dispatch/3","doc":"The potentially modified result.","ref":"AshCommanded.Commanded.Middleware.CommandMiddleware.html#c:after_dispatch/3-returns"},{"type":"callback","title":"AshCommanded.Commanded.Middleware.CommandMiddleware.before_dispatch/3","doc":"Called before a command is dispatched.","ref":"AshCommanded.Commanded.Middleware.CommandMiddleware.html#c:before_dispatch/3"},{"type":"callback","title":"Parameters - AshCommanded.Commanded.Middleware.CommandMiddleware.before_dispatch/3","doc":"* `command` - The command to be dispatched\n* `context` - The context for this command dispatch\n* `next` - Function to call the next middleware in the chain","ref":"AshCommanded.Commanded.Middleware.CommandMiddleware.html#c:before_dispatch/3-parameters"},{"type":"callback","title":"Returns - AshCommanded.Commanded.Middleware.CommandMiddleware.before_dispatch/3","doc":"The result of the last middleware or command handler in the chain.","ref":"AshCommanded.Commanded.Middleware.CommandMiddleware.html#c:before_dispatch/3-returns"},{"type":"module","title":"AshCommanded.Commanded.Middleware.CommandMiddlewareProcessor","doc":"Processes command middleware for AshCommanded.\n\nThis module is responsible for:\n1. Collecting middleware from configuration, resources, and commands\n2. Building the middleware chain\n3. Applying middleware to commands\n\nIt's used internally by command handlers to ensure consistent\nmiddleware application across the system.","ref":"AshCommanded.Commanded.Middleware.CommandMiddlewareProcessor.html"},{"type":"function","title":"AshCommanded.Commanded.Middleware.CommandMiddlewareProcessor.apply_middleware/4","doc":"Apply all applicable middleware to a command.","ref":"AshCommanded.Commanded.Middleware.CommandMiddlewareProcessor.html#apply_middleware/4"},{"type":"function","title":"Parameters - AshCommanded.Commanded.Middleware.CommandMiddlewareProcessor.apply_middleware/4","doc":"* `command` - The command to process\n* `resource` - The resource the command belongs to\n* `context` - The context for the command dispatch\n* `final_handler` - Function to call after all middleware has been applied","ref":"AshCommanded.Commanded.Middleware.CommandMiddlewareProcessor.html#apply_middleware/4-parameters"},{"type":"function","title":"Examples - AshCommanded.Commanded.Middleware.CommandMiddlewareProcessor.apply_middleware/4","doc":"```elixir\napply_middleware(\n  %MyApp.Commands.RegisterUser{},\n  MyApp.User,\n  %{},\n  fn cmd, ctx -> {:ok, process_command(cmd)} end\n)\n```","ref":"AshCommanded.Commanded.Middleware.CommandMiddlewareProcessor.html#apply_middleware/4-examples"},{"type":"function","title":"AshCommanded.Commanded.Middleware.CommandMiddlewareProcessor.get_middleware_chain/2","doc":"Get the complete middleware chain for a command.\n\nThis collects middleware from:\n1. Global application config\n2. Resource-level middleware\n3. Command-specific middleware","ref":"AshCommanded.Commanded.Middleware.CommandMiddlewareProcessor.html#get_middleware_chain/2"},{"type":"function","title":"Parameters - AshCommanded.Commanded.Middleware.CommandMiddlewareProcessor.get_middleware_chain/2","doc":"* `command` - The command struct\n* `resource` - The resource module","ref":"AshCommanded.Commanded.Middleware.CommandMiddlewareProcessor.html#get_middleware_chain/2-parameters"},{"type":"function","title":"Returns - AshCommanded.Commanded.Middleware.CommandMiddlewareProcessor.get_middleware_chain/2","doc":"A list of middleware modules with their configuration.","ref":"AshCommanded.Commanded.Middleware.CommandMiddlewareProcessor.html#get_middleware_chain/2-returns"},{"type":"module","title":"AshCommanded.Commanded.Middleware.LoggingMiddleware","doc":"Middleware that logs command dispatch and results.\n\nThis middleware logs information about commands being dispatched\nand their results. It's useful for debugging and auditing.","ref":"AshCommanded.Commanded.Middleware.LoggingMiddleware.html"},{"type":"module","title":"Configuration - AshCommanded.Commanded.Middleware.LoggingMiddleware","doc":"The logging level can be configured with the `:level` option:\n\n```elixir\nmiddleware AshCommanded.Commanded.Middleware.LoggingMiddleware, level: :debug\n```\n\nAvailable levels: `:debug`, `:info`, `:warn`, `:error`","ref":"AshCommanded.Commanded.Middleware.LoggingMiddleware.html#module-configuration"},{"type":"module","title":"Log Format - AshCommanded.Commanded.Middleware.LoggingMiddleware","doc":"The middleware logs the following information:\n\n* Before dispatch: Command type, command ID, and fields\n* After successful dispatch: Command type, command ID, and result\n* After failed dispatch: Command type, command ID, and error reason","ref":"AshCommanded.Commanded.Middleware.LoggingMiddleware.html#module-log-format"},{"type":"module","title":"AshCommanded.Commanded.Middleware.ValidationMiddleware","doc":"Middleware that validates commands before processing.\n\nThis middleware applies validation functions to commands\nbefore they are dispatched. If validation fails, the command\nis rejected with an error.","ref":"AshCommanded.Commanded.Middleware.ValidationMiddleware.html"},{"type":"module","title":"Configuration - AshCommanded.Commanded.Middleware.ValidationMiddleware","doc":"Validation can be configured in several ways:\n\n```elixir\n# Simple validation of required fields\nmiddleware AshCommanded.Commanded.Middleware.ValidationMiddleware,\n  required: [:id, :name, :email]\n  \n# Custom validation function\nmiddleware AshCommanded.Commanded.Middleware.ValidationMiddleware,\n  validate: fn command ->\n    # Return :ok or {:error, reason}\n    if String.contains?(command.email, \"@\") do\n      :ok\n    else\n      {:error, \"Invalid email format\"}\n    end\n  end\n  \n# Multiple validations\nmiddleware AshCommanded.Commanded.Middleware.ValidationMiddleware,\n  validations: [\n    required: [:id, :name, :email],\n    format: [email: ~r/@/],\n    custom: fn command ->\n      # Custom validation logic\n      :ok\n    end\n  ]\n```","ref":"AshCommanded.Commanded.Middleware.ValidationMiddleware.html#module-configuration"},{"type":"module","title":"AshCommanded.Commanded.ParameterTransformer","doc":"Advanced parameter transformation for mapping commands to Ash actions.\n\nThis module provides a more sophisticated approach to parameter transformation\nthan the basic mapping in CommandActionMapper. It allows for:\n\n1. Type conversion and validation\n2. Default values\n3. Computed fields\n4. Nested transformations\n5. Collection handling\n6. Custom transformation functions\n\nThese transformations can be defined in the DSL and are used when mapping\ncommands to Ash actions.","ref":"AshCommanded.Commanded.ParameterTransformer.html"},{"type":"module","title":"Example - AshCommanded.Commanded.ParameterTransformer","doc":"```elixir\ndefmodule MyApp.User do\n  use Ash.Resource,\n    extensions: [AshCommanded.Commanded.Dsl]\n    \n  commanded do\n    commands do\n      command :register_user do\n        fields [:id, :name, :email, :birthdate, :roles]\n        \n        transform_params do\n          # Simple field rename\n          map :name, to: :full_name\n          \n          # Type conversion\n          cast :birthdate, :date\n          \n          # Computed field\n          compute :age, fn params ->\n            date = params.birthdate\n            now = Date.utc_today()\n            years = now.year - date.year\n            if Date.compare(\n              %{date | year: now.year}, \n              %{now | month: date.month, day: date.day}\n            ) == :gt, do: years - 1, else: years\n          end\n          \n          # Nested transformation for collections\n          transform :roles, fn roles ->\n            roles |> Enum.map(&String.to_atom/1)\n          end\n          \n          # Add default values\n          default :status, \"active\"\n          default :registered_at, &DateTime.utc_now/0\n          \n          # Custom transformation function\n          custom fn params ->\n            Map.put(params, :normalized_email, String.downcase(params.email))\n          end\n        end\n      end\n    end\n  end\nend\n```","ref":"AshCommanded.Commanded.ParameterTransformer.html#module-example"},{"type":"function","title":"AshCommanded.Commanded.ParameterTransformer.build_transforms/1","doc":"Builds a transformation specification from a DSL block.\n\nThis function is used by the DSL to convert transformation declarations\ninto a list of transformation specifications that can be applied to\ncommand parameters.","ref":"AshCommanded.Commanded.ParameterTransformer.html#build_transforms/1"},{"type":"function","title":"Parameters - AshCommanded.Commanded.ParameterTransformer.build_transforms/1","doc":"* `transform_block` - A keyword list of transformation declarations","ref":"AshCommanded.Commanded.ParameterTransformer.html#build_transforms/1-parameters"},{"type":"function","title":"Returns - AshCommanded.Commanded.ParameterTransformer.build_transforms/1","doc":"A list of transformation specifications.","ref":"AshCommanded.Commanded.ParameterTransformer.html#build_transforms/1-returns"},{"type":"function","title":"Example - AshCommanded.Commanded.ParameterTransformer.build_transforms/1","doc":"```elixir\nbuild_transforms([\n  map: [:name, to: :full_name],\n  cast: [:birthdate, :date],\n  compute: [:age, fn params -> calculate_age(params.birthdate) end],\n  default: [:status, \"active\"]\n])\n```","ref":"AshCommanded.Commanded.ParameterTransformer.html#build_transforms/1-example"},{"type":"function","title":"AshCommanded.Commanded.ParameterTransformer.transform_params/2","doc":"Apply a transformation specification to command parameters.\n\nTakes a map of parameters from a command and a transformation spec,\nand returns a new map with the transformations applied.","ref":"AshCommanded.Commanded.ParameterTransformer.html#transform_params/2"},{"type":"function","title":"Parameters - AshCommanded.Commanded.ParameterTransformer.transform_params/2","doc":"* `params` - Map of parameters from the command\n* `transforms` - List of transformation specifications","ref":"AshCommanded.Commanded.ParameterTransformer.html#transform_params/2-parameters"},{"type":"function","title":"Returns - AshCommanded.Commanded.ParameterTransformer.transform_params/2","doc":"A new map with all transformations applied.","ref":"AshCommanded.Commanded.ParameterTransformer.html#transform_params/2-returns"},{"type":"function","title":"Example - AshCommanded.Commanded.ParameterTransformer.transform_params/2","doc":"```\ntransform_params(\n  %{name: \"John Doe\", email: \"john@example.com\"},\n  [\n    {:map, :name, [to: :full_name]},\n    {:compute, :display_name, [using: fn p -> \"Display name\" end]},\n    {:default, :created_at, [value: DateTime.utc_now()]}\n  ]\n)\n```","ref":"AshCommanded.Commanded.ParameterTransformer.html#transform_params/2-example"},{"type":"module","title":"AshCommanded.Commanded.ParameterValidator","doc":"Advanced parameter validation for commands.\n\nThis module provides comprehensive validation capabilities for command parameters,\nensuring they meet specific criteria before being processed by actions.\n\nValidation features include:\n\n1. Type checking - Ensure values match expected types\n2. Format validation - Validate strings with regular expressions\n3. Range validation - Check numeric values against ranges\n4. Domain validation - Validate values against lists of allowed values\n5. Custom validation functions - Apply arbitrary validation logic\n6. Nested validation - Validate nested maps and collections\n\nThese validations complement the more basic validations in ValidationMiddleware.","ref":"AshCommanded.Commanded.ParameterValidator.html"},{"type":"module","title":"Example - AshCommanded.Commanded.ParameterValidator","doc":"```elixir\ndefmodule MyApp.User do\n  use Ash.Resource,\n    extensions: [AshCommanded.Commanded.Dsl]\n    \n  commanded do\n    commands do\n      command :register_user do\n        fields [:id, :name, :email, :age, :roles]\n        \n        validate_params do\n          # Type validation\n          validate :id, type: :string\n          validate :name, type: :string\n          validate :email, type: :string\n          validate :age, type: :integer\n          validate :roles, type: :list\n          \n          # Format validation\n          validate :email, format: ~r/^[^ ]+@[^ ]+.[^ ]+$/\n          \n          # Range validation\n          validate :age, min: 18, max: 120\n          \n          # Domain validation\n          validate :roles, subset_of: [:user, :admin, :moderator]\n          \n          # Custom validation\n          validate :name, fn value ->\n            if String.trim(value) == \"\" do\n              {:error, \"Name cannot be blank\"}\n            else\n              :ok\n            end\n          end\n          \n          # Multiple validations on one field\n          validate :password do\n            min_length 8\n            format ~r/[A-Z]/\n            format ~r/[a-z]/\n            format ~r/[0-9]/\n            format ~r/[^A-Za-z0-9]/\n          end\n        end\n      end\n    end\n  end\nend\n```","ref":"AshCommanded.Commanded.ParameterValidator.html#module-example"},{"type":"function","title":"AshCommanded.Commanded.ParameterValidator.build_validations/1","doc":"Builds a validation specification from a DSL block.\n\nThis function is used by the DSL to convert validation declarations\ninto a list of validation specifications that can be applied to\ncommand parameters.","ref":"AshCommanded.Commanded.ParameterValidator.html#build_validations/1"},{"type":"function","title":"Parameters - AshCommanded.Commanded.ParameterValidator.build_validations/1","doc":"* `validation_block` - A keyword list of validation declarations","ref":"AshCommanded.Commanded.ParameterValidator.html#build_validations/1-parameters"},{"type":"function","title":"Returns - AshCommanded.Commanded.ParameterValidator.build_validations/1","doc":"A list of validation specifications.","ref":"AshCommanded.Commanded.ParameterValidator.html#build_validations/1-returns"},{"type":"function","title":"Example - AshCommanded.Commanded.ParameterValidator.build_validations/1","doc":"```elixir\nbuild_validations([\n  validate: [:name, [type: :string, min_length: 2]],\n  validate: [:email, [type: :string, format: ~r/@.*./]],\n  validate: [:age, [type: :integer, min: 18]]\n])\n```","ref":"AshCommanded.Commanded.ParameterValidator.html#build_validations/1-example"},{"type":"function","title":"AshCommanded.Commanded.ParameterValidator.validate_params/2","doc":"Validates parameters against a validation specification.\n\nTakes a map of parameters and a list of validation rules,\nand returns `:ok` or an error tuple with validation failures.","ref":"AshCommanded.Commanded.ParameterValidator.html#validate_params/2"},{"type":"function","title":"Parameters - AshCommanded.Commanded.ParameterValidator.validate_params/2","doc":"* `params` - Map of parameters to validate\n* `validations` - List of validation specifications","ref":"AshCommanded.Commanded.ParameterValidator.html#validate_params/2-parameters"},{"type":"function","title":"Returns - AshCommanded.Commanded.ParameterValidator.validate_params/2","doc":"* `:ok` - If all validations pass\n* `{:error, errors}` - If any validations fail, with errors being a list of validation error structs","ref":"AshCommanded.Commanded.ParameterValidator.html#validate_params/2-returns"},{"type":"function","title":"Example - AshCommanded.Commanded.ParameterValidator.validate_params/2","doc":"```elixir\nvalidate_params(\n  %{name: \"John\", email: \"john@example\", age: 15},\n  [\n    {:validate, :name, [type: :string, min_length: 2]},\n    {:validate, :email, [type: :string, format: ~r/@.*./]},\n    {:validate, :age, [type: :integer, min: 18]}\n  ]\n)\n# => {:error, [%Error{type: :validation_error, message: \"does not match required format\", field: :email}, \n#              %Error{type: :validation_error, message: \"must be at least 18\", field: :age}]}\n```","ref":"AshCommanded.Commanded.ParameterValidator.html#validate_params/2-example"},{"type":"module","title":"AshCommanded.Commanded.Projection","doc":"Represents a projection in the Commanded DSL.\n\nProjections define how events affect the resource state, transforming events into resource updates.","ref":"AshCommanded.Commanded.Projection.html"},{"type":"type","title":"AshCommanded.Commanded.Projection.t/0","doc":"","ref":"AshCommanded.Commanded.Projection.html#t:t/0"},{"type":"module","title":"AshCommanded.Commanded.Snapshot","doc":"Represents a snapshot of an aggregate's state at a specific point in time.\n\nSnapshots are used as a performance optimization to avoid replaying all events\nfrom the beginning of the aggregate's history. Instead, a snapshot represents\nthe aggregate state as of a specific version, allowing the system to load the\nsnapshot and only replay events that occurred after the snapshot was taken.","ref":"AshCommanded.Commanded.Snapshot.html"},{"type":"function","title":"AshCommanded.Commanded.Snapshot.new/4","doc":"Creates a new snapshot from an aggregate state.","ref":"AshCommanded.Commanded.Snapshot.html#new/4"},{"type":"function","title":"Parameters - AshCommanded.Commanded.Snapshot.new/4","doc":"* `aggregate` - The aggregate state\n* `source_type` - The aggregate module\n* `version` - The aggregate version (event number)\n* `source_version` - The snapshot schema version","ref":"AshCommanded.Commanded.Snapshot.html#new/4-parameters"},{"type":"function","title":"Returns - AshCommanded.Commanded.Snapshot.new/4","doc":"A new snapshot structure","ref":"AshCommanded.Commanded.Snapshot.html#new/4-returns"},{"type":"function","title":"Examples - AshCommanded.Commanded.Snapshot.new/4","doc":"iex> aggregate = %MyApp.UserAggregate{id: \"123\", name: \"John\"}\n    iex> AshCommanded.Commanded.Snapshot.new(aggregate, MyApp.UserAggregate, 5, 1)\n    %AshCommanded.Commanded.Snapshot{\n      source_uuid: \"123\",\n      source_type: MyApp.UserAggregate,\n      source_version: 1,\n      state: %MyApp.UserAggregate{id: \"123\", name: \"John\"},\n      version: 5,\n      created_at: ~U[2023-01-01 00:00:00Z]\n    }","ref":"AshCommanded.Commanded.Snapshot.html#new/4-examples"},{"type":"function","title":"AshCommanded.Commanded.Snapshot.source_uuid/1","doc":"Gets the source UUID (aggregate ID) from a snapshot.","ref":"AshCommanded.Commanded.Snapshot.html#source_uuid/1"},{"type":"function","title":"Parameters - AshCommanded.Commanded.Snapshot.source_uuid/1","doc":"* `snapshot` - The snapshot to get the UUID from","ref":"AshCommanded.Commanded.Snapshot.html#source_uuid/1-parameters"},{"type":"function","title":"Returns - AshCommanded.Commanded.Snapshot.source_uuid/1","doc":"The source UUID","ref":"AshCommanded.Commanded.Snapshot.html#source_uuid/1-returns"},{"type":"function","title":"Examples - AshCommanded.Commanded.Snapshot.source_uuid/1","doc":"iex> snapshot = %AshCommanded.Commanded.Snapshot{source_uuid: \"123\"}\n    iex> AshCommanded.Commanded.Snapshot.source_uuid(snapshot)\n    \"123\"","ref":"AshCommanded.Commanded.Snapshot.html#source_uuid/1-examples"},{"type":"function","title":"AshCommanded.Commanded.Snapshot.state/1","doc":"Extracts the aggregate state from a snapshot.","ref":"AshCommanded.Commanded.Snapshot.html#state/1"},{"type":"function","title":"Parameters - AshCommanded.Commanded.Snapshot.state/1","doc":"* `snapshot` - The snapshot to extract state from","ref":"AshCommanded.Commanded.Snapshot.html#state/1-parameters"},{"type":"function","title":"Returns - AshCommanded.Commanded.Snapshot.state/1","doc":"The aggregate state","ref":"AshCommanded.Commanded.Snapshot.html#state/1-returns"},{"type":"function","title":"Examples - AshCommanded.Commanded.Snapshot.state/1","doc":"iex> snapshot = %AshCommanded.Commanded.Snapshot{state: %{id: \"123\", name: \"John\"}}\n    iex> AshCommanded.Commanded.Snapshot.state(snapshot)\n    %{id: \"123\", name: \"John\"}","ref":"AshCommanded.Commanded.Snapshot.html#state/1-examples"},{"type":"type","title":"AshCommanded.Commanded.Snapshot.t/0","doc":"","ref":"AshCommanded.Commanded.Snapshot.html#t:t/0"},{"type":"function","title":"AshCommanded.Commanded.Snapshot.version/1","doc":"Gets the aggregate version from a snapshot.","ref":"AshCommanded.Commanded.Snapshot.html#version/1"},{"type":"function","title":"Parameters - AshCommanded.Commanded.Snapshot.version/1","doc":"* `snapshot` - The snapshot to get the version from","ref":"AshCommanded.Commanded.Snapshot.html#version/1-parameters"},{"type":"function","title":"Returns - AshCommanded.Commanded.Snapshot.version/1","doc":"The aggregate version","ref":"AshCommanded.Commanded.Snapshot.html#version/1-returns"},{"type":"function","title":"Examples - AshCommanded.Commanded.Snapshot.version/1","doc":"iex> snapshot = %AshCommanded.Commanded.Snapshot{version: 5}\n    iex> AshCommanded.Commanded.Snapshot.version(snapshot)\n    5","ref":"AshCommanded.Commanded.Snapshot.html#version/1-examples"},{"type":"module","title":"AshCommanded.Commanded.SnapshotAdapter","doc":"Adapter to integrate AshCommanded's snapshot functionality with Commanded.\n\nThis module implements the necessary callbacks to allow Commanded to \nuse AshCommanded's snapshot functionality during aggregate initialization\nand event application.\n\nIt implements the `Commanded.Aggregates.Aggregate.Snapshotter` behaviour\nto integrate with Commanded's built-in snapshot support when it's available.\nDuring tests without Commanded, it falls back to simpler implementation.","ref":"AshCommanded.Commanded.SnapshotAdapter.html"},{"type":"function","title":"AshCommanded.Commanded.SnapshotAdapter.load_snapshot/2","doc":"Loads a snapshot for an aggregate.","ref":"AshCommanded.Commanded.SnapshotAdapter.html#load_snapshot/2"},{"type":"function","title":"Parameters - AshCommanded.Commanded.SnapshotAdapter.load_snapshot/2","doc":"* `aggregate_module` - The aggregate module\n* `aggregate_uuid` - The unique identifier of the aggregate","ref":"AshCommanded.Commanded.SnapshotAdapter.html#load_snapshot/2-parameters"},{"type":"function","title":"Returns - AshCommanded.Commanded.SnapshotAdapter.load_snapshot/2","doc":"* `{:ok, version, state}` - If a snapshot was found\n* `{:error, reason}` - If no snapshot was found or could not be loaded","ref":"AshCommanded.Commanded.SnapshotAdapter.html#load_snapshot/2-returns"},{"type":"function","title":"AshCommanded.Commanded.SnapshotAdapter.snapshot_threshold/1","doc":"Returns the snapshot threshold for an aggregate module.","ref":"AshCommanded.Commanded.SnapshotAdapter.html#snapshot_threshold/1"},{"type":"function","title":"Parameters - AshCommanded.Commanded.SnapshotAdapter.snapshot_threshold/1","doc":"* `aggregate_module` - The aggregate module","ref":"AshCommanded.Commanded.SnapshotAdapter.html#snapshot_threshold/1-parameters"},{"type":"function","title":"Returns - AshCommanded.Commanded.SnapshotAdapter.snapshot_threshold/1","doc":"The number of events to process before taking a snapshot, or nil if \nsnapshotting is disabled","ref":"AshCommanded.Commanded.SnapshotAdapter.html#snapshot_threshold/1-returns"},{"type":"function","title":"AshCommanded.Commanded.SnapshotAdapter.snapshot_version/1","doc":"Returns the snapshot schema version for an aggregate module.","ref":"AshCommanded.Commanded.SnapshotAdapter.html#snapshot_version/1"},{"type":"function","title":"Parameters - AshCommanded.Commanded.SnapshotAdapter.snapshot_version/1","doc":"* `aggregate_module` - The aggregate module","ref":"AshCommanded.Commanded.SnapshotAdapter.html#snapshot_version/1-parameters"},{"type":"function","title":"Returns - AshCommanded.Commanded.SnapshotAdapter.snapshot_version/1","doc":"The snapshot schema version","ref":"AshCommanded.Commanded.SnapshotAdapter.html#snapshot_version/1-returns"},{"type":"function","title":"AshCommanded.Commanded.SnapshotAdapter.take_snapshot/2","doc":"Takes a snapshot of an aggregate's state.","ref":"AshCommanded.Commanded.SnapshotAdapter.html#take_snapshot/2"},{"type":"function","title":"Parameters - AshCommanded.Commanded.SnapshotAdapter.take_snapshot/2","doc":"* `aggregate_module` - The aggregate module\n* `aggregate_state` - The current state of the aggregate","ref":"AshCommanded.Commanded.SnapshotAdapter.html#take_snapshot/2-parameters"},{"type":"function","title":"Returns - AshCommanded.Commanded.SnapshotAdapter.take_snapshot/2","doc":"* `{:ok, snapshot}` - If the snapshot was taken successfully\n* `{:error, reason}` - If the snapshot could not be taken","ref":"AshCommanded.Commanded.SnapshotAdapter.html#take_snapshot/2-returns"},{"type":"module","title":"AshCommanded.Commanded.SnapshotConfiguration","doc":"Provides configuration options for Commanded to use AshCommanded's snapshot functionality.\n\nThis module standardizes snapshot configuration options for Commanded applications.","ref":"AshCommanded.Commanded.SnapshotConfiguration.html"},{"type":"function","title":"AshCommanded.Commanded.SnapshotConfiguration.dispatch_options/0","doc":"Returns the snapshot options to use when dispatching a command.","ref":"AshCommanded.Commanded.SnapshotConfiguration.html#dispatch_options/0"},{"type":"function","title":"Returns - AshCommanded.Commanded.SnapshotConfiguration.dispatch_options/0","doc":"A keyword list of snapshot options for command dispatch","ref":"AshCommanded.Commanded.SnapshotConfiguration.html#dispatch_options/0-returns"},{"type":"function","title":"Examples - AshCommanded.Commanded.SnapshotConfiguration.dispatch_options/0","doc":"iex> AshCommanded.Commanded.SnapshotConfiguration.dispatch_options()\n    [snapshot_module: AshCommanded.Commanded.SnapshotAdapter]","ref":"AshCommanded.Commanded.SnapshotConfiguration.html#dispatch_options/0-examples"},{"type":"function","title":"AshCommanded.Commanded.SnapshotConfiguration.snapshot_options/1","doc":"Returns the snapshot options to use when configuring a Commanded application.","ref":"AshCommanded.Commanded.SnapshotConfiguration.html#snapshot_options/1"},{"type":"function","title":"Parameters - AshCommanded.Commanded.SnapshotConfiguration.snapshot_options/1","doc":"* `threshold` - The number of events to process before taking a snapshot (default: 100)","ref":"AshCommanded.Commanded.SnapshotConfiguration.html#snapshot_options/1-parameters"},{"type":"function","title":"Returns - AshCommanded.Commanded.SnapshotConfiguration.snapshot_options/1","doc":"A keyword list of snapshot options for Commanded","ref":"AshCommanded.Commanded.SnapshotConfiguration.html#snapshot_options/1-returns"},{"type":"function","title":"Examples - AshCommanded.Commanded.SnapshotConfiguration.snapshot_options/1","doc":"iex> AshCommanded.Commanded.SnapshotConfiguration.snapshot_options(500)\n    [snapshot_every: 500, snapshot_module: AshCommanded.Commanded.SnapshotAdapter]","ref":"AshCommanded.Commanded.SnapshotConfiguration.html#snapshot_options/1-examples"},{"type":"behaviour","title":"AshCommanded.Commanded.SnapshotStore","doc":"Defines the behavior and default implementation for aggregate snapshot storage.\n\nThe snapshot store is responsible for storing and retrieving snapshots of aggregates,\nproviding a performance optimization by allowing the system to load the latest snapshot\nrather than replaying all events from the beginning.\n\nThe default implementation uses ETS tables for storage. Alternative implementations\ncan be provided by specifying a custom snapshot store module in the application config.","ref":"AshCommanded.Commanded.SnapshotStore.html"},{"type":"function","title":"AshCommanded.Commanded.SnapshotStore.delete_snapshots/2","doc":"Deletes all snapshots for an aggregate.","ref":"AshCommanded.Commanded.SnapshotStore.html#delete_snapshots/2"},{"type":"function","title":"Parameters - AshCommanded.Commanded.SnapshotStore.delete_snapshots/2","doc":"* `source_uuid` - The unique identifier of the aggregate\n* `source_type` - The aggregate module","ref":"AshCommanded.Commanded.SnapshotStore.html#delete_snapshots/2-parameters"},{"type":"function","title":"Returns - AshCommanded.Commanded.SnapshotStore.delete_snapshots/2","doc":"* `:ok` - If the snapshots were deleted successfully\n* `{:error, reason}` - If the snapshots could not be deleted","ref":"AshCommanded.Commanded.SnapshotStore.html#delete_snapshots/2-returns"},{"type":"callback","title":"AshCommanded.Commanded.SnapshotStore.delete_snapshots/2","doc":"Deletes all snapshots for an aggregate.","ref":"AshCommanded.Commanded.SnapshotStore.html#c:delete_snapshots/2"},{"type":"callback","title":"Parameters - AshCommanded.Commanded.SnapshotStore.delete_snapshots/2","doc":"* `source_uuid` - The unique identifier of the aggregate\n* `source_type` - The aggregate module","ref":"AshCommanded.Commanded.SnapshotStore.html#c:delete_snapshots/2-parameters"},{"type":"callback","title":"Returns - AshCommanded.Commanded.SnapshotStore.delete_snapshots/2","doc":"* `:ok` - If the snapshots were deleted successfully\n* `{:error, reason}` - If the snapshots could not be deleted","ref":"AshCommanded.Commanded.SnapshotStore.html#c:delete_snapshots/2-returns"},{"type":"function","title":"AshCommanded.Commanded.SnapshotStore.get_snapshot/2","doc":"Gets the latest snapshot for an aggregate by its UUID.","ref":"AshCommanded.Commanded.SnapshotStore.html#get_snapshot/2"},{"type":"function","title":"Parameters - AshCommanded.Commanded.SnapshotStore.get_snapshot/2","doc":"* `source_uuid` - The unique identifier of the aggregate\n* `source_type` - The aggregate module","ref":"AshCommanded.Commanded.SnapshotStore.html#get_snapshot/2-parameters"},{"type":"function","title":"Returns - AshCommanded.Commanded.SnapshotStore.get_snapshot/2","doc":"* `{:ok, snapshot}` - If a snapshot was found\n* `:error` - If no snapshot was found","ref":"AshCommanded.Commanded.SnapshotStore.html#get_snapshot/2-returns"},{"type":"callback","title":"AshCommanded.Commanded.SnapshotStore.get_snapshot/2","doc":"Gets the latest snapshot for an aggregate by its UUID.","ref":"AshCommanded.Commanded.SnapshotStore.html#c:get_snapshot/2"},{"type":"callback","title":"Parameters - AshCommanded.Commanded.SnapshotStore.get_snapshot/2","doc":"* `source_uuid` - The unique identifier of the aggregate\n* `source_type` - The aggregate module","ref":"AshCommanded.Commanded.SnapshotStore.html#c:get_snapshot/2-parameters"},{"type":"callback","title":"Returns - AshCommanded.Commanded.SnapshotStore.get_snapshot/2","doc":"* `{:ok, snapshot}` - If a snapshot was found\n* `:error` - If no snapshot was found","ref":"AshCommanded.Commanded.SnapshotStore.html#c:get_snapshot/2-returns"},{"type":"function","title":"AshCommanded.Commanded.SnapshotStore.init/1","doc":"Initializes the snapshot store.","ref":"AshCommanded.Commanded.SnapshotStore.html#init/1"},{"type":"function","title":"Parameters - AshCommanded.Commanded.SnapshotStore.init/1","doc":"* `config` - Configuration options for the snapshot store (unused for ETS implementation)","ref":"AshCommanded.Commanded.SnapshotStore.html#init/1-parameters"},{"type":"function","title":"Returns - AshCommanded.Commanded.SnapshotStore.init/1","doc":"* `:ok` - If the snapshot store was initialized successfully","ref":"AshCommanded.Commanded.SnapshotStore.html#init/1-returns"},{"type":"callback","title":"AshCommanded.Commanded.SnapshotStore.init/1","doc":"Initializes the snapshot store.","ref":"AshCommanded.Commanded.SnapshotStore.html#c:init/1"},{"type":"callback","title":"Parameters - AshCommanded.Commanded.SnapshotStore.init/1","doc":"* `config` - Configuration options for the snapshot store","ref":"AshCommanded.Commanded.SnapshotStore.html#c:init/1-parameters"},{"type":"callback","title":"Returns - AshCommanded.Commanded.SnapshotStore.init/1","doc":"* `:ok` - If the snapshot store was initialized successfully\n* `{:error, reason}` - If the snapshot store could not be initialized","ref":"AshCommanded.Commanded.SnapshotStore.html#c:init/1-returns"},{"type":"function","title":"AshCommanded.Commanded.SnapshotStore.save_snapshot/1","doc":"Saves a snapshot of an aggregate.","ref":"AshCommanded.Commanded.SnapshotStore.html#save_snapshot/1"},{"type":"function","title":"Parameters - AshCommanded.Commanded.SnapshotStore.save_snapshot/1","doc":"* `snapshot` - The snapshot to save","ref":"AshCommanded.Commanded.SnapshotStore.html#save_snapshot/1-parameters"},{"type":"function","title":"Returns - AshCommanded.Commanded.SnapshotStore.save_snapshot/1","doc":"* `:ok` - If the snapshot was saved successfully\n* `{:error, reason}` - If the snapshot could not be saved","ref":"AshCommanded.Commanded.SnapshotStore.html#save_snapshot/1-returns"},{"type":"callback","title":"AshCommanded.Commanded.SnapshotStore.save_snapshot/1","doc":"Saves a snapshot of an aggregate.","ref":"AshCommanded.Commanded.SnapshotStore.html#c:save_snapshot/1"},{"type":"callback","title":"Parameters - AshCommanded.Commanded.SnapshotStore.save_snapshot/1","doc":"* `snapshot` - The snapshot to save","ref":"AshCommanded.Commanded.SnapshotStore.html#c:save_snapshot/1-parameters"},{"type":"callback","title":"Returns - AshCommanded.Commanded.SnapshotStore.save_snapshot/1","doc":"* `:ok` - If the snapshot was saved successfully\n* `{:error, reason}` - If the snapshot could not be saved","ref":"AshCommanded.Commanded.SnapshotStore.html#c:save_snapshot/1-returns"},{"type":"module","title":"AshCommanded.Commanded.Transaction","doc":"Transaction support for command execution in AshCommanded.\n\nThis module provides functionality for wrapping command execution inside database\ntransactions, ensuring atomicity of operations across multiple resources and commands.\n\nTransactions can be used in two ways:\n1. Automatically based on command configuration in the DSL\n2. Manually by wrapping command execution in a transaction block","ref":"AshCommanded.Commanded.Transaction.html"},{"type":"module","title":"Features - AshCommanded.Commanded.Transaction","doc":"- Supports single command transactions\n- Supports multi-command transactions\n- Integrates with Ash Repository transactions\n- Handles rollback on error\n- Provides transaction options (timeout, isolation level)","ref":"AshCommanded.Commanded.Transaction.html#module-features"},{"type":"function","title":"AshCommanded.Commanded.Transaction.execute_commands/3","doc":"Executes multiple commands in a single transaction.","ref":"AshCommanded.Commanded.Transaction.html#execute_commands/3"},{"type":"function","title":"Parameters - AshCommanded.Commanded.Transaction.execute_commands/3","doc":"* `repo` - The repository to use for the transaction\n* `commands` - List of commands to execute, each with resource and action\n* `opts` - Transaction options","ref":"AshCommanded.Commanded.Transaction.html#execute_commands/3-parameters"},{"type":"function","title":"Command Structure - AshCommanded.Commanded.Transaction.execute_commands/3","doc":"Each command in the list should be a map with these keys:\n* `:command` - The command struct to execute\n* `:resource` - The resource module\n* `:action` - The action name (atom)\n* `:opts` - (Optional) Command-specific options","ref":"AshCommanded.Commanded.Transaction.html#execute_commands/3-command-structure"},{"type":"function","title":"Returns - AshCommanded.Commanded.Transaction.execute_commands/3","doc":"* `{:ok, result_map}` - Map of results from all commands\n* `{:error, failed_operation, failed_value, changes_so_far}` - Error information","ref":"AshCommanded.Commanded.Transaction.html#execute_commands/3-returns"},{"type":"function","title":"Examples - AshCommanded.Commanded.Transaction.execute_commands/3","doc":"iex> AshCommanded.Commanded.Transaction.execute_commands(MyApp.Repo, [\n    ...>   %{command: create_user_cmd, resource: MyApp.User, action: :create},\n    ...>   %{command: create_profile_cmd, resource: MyApp.Profile, action: :create}\n    ...> ])\n    {:ok, %{create_user: %MyApp.User{...}, create_profile: %MyApp.Profile{...}}}","ref":"AshCommanded.Commanded.Transaction.html#execute_commands/3-examples"},{"type":"function","title":"AshCommanded.Commanded.Transaction.run/3","doc":"Executes a function within a transaction.","ref":"AshCommanded.Commanded.Transaction.html#run/3"},{"type":"function","title":"Parameters - AshCommanded.Commanded.Transaction.run/3","doc":"* `repo` - The repository to use for the transaction\n* `fun` - The function to execute inside the transaction\n* `opts` - Transaction options","ref":"AshCommanded.Commanded.Transaction.html#run/3-parameters"},{"type":"function","title":"Options - AshCommanded.Commanded.Transaction.run/3","doc":"* `:timeout` - Transaction timeout in milliseconds\n* `:isolation_level` - Transaction isolation level (`:read_committed`, `:repeatable_read`, or `:serializable`)","ref":"AshCommanded.Commanded.Transaction.html#run/3-options"},{"type":"function","title":"Returns - AshCommanded.Commanded.Transaction.run/3","doc":"* `{:ok, result}` - The result of the function, if the transaction succeeded\n* `{:error, reason}` - The error that caused the transaction to roll back","ref":"AshCommanded.Commanded.Transaction.html#run/3-returns"},{"type":"function","title":"Examples - AshCommanded.Commanded.Transaction.run/3","doc":"iex> AshCommanded.Commanded.Transaction.run(MyApp.Repo, fn ->\n    ...>   # Execute commands or actions\n    ...>   AshCommanded.Commanded.CommandActionMapper.map_to_action(command, resource, action)\n    ...> end)\n    {:ok, %MyApp.User{id: \"123\", name: \"John\"}}","ref":"AshCommanded.Commanded.Transaction.html#run/3-examples"},{"type":"function","title":"AshCommanded.Commanded.Transaction.supports_transactions?/1","doc":"Checks whether a repository supports transactions.","ref":"AshCommanded.Commanded.Transaction.html#supports_transactions?/1"},{"type":"function","title":"Parameters - AshCommanded.Commanded.Transaction.supports_transactions?/1","doc":"* `repo` - The repository to check","ref":"AshCommanded.Commanded.Transaction.html#supports_transactions?/1-parameters"},{"type":"function","title":"Returns - AshCommanded.Commanded.Transaction.supports_transactions?/1","doc":"* `true` - If the repository supports transactions\n* `false` - If the repository does not support transactions","ref":"AshCommanded.Commanded.Transaction.html#supports_transactions?/1-returns"},{"type":"function","title":"Examples - AshCommanded.Commanded.Transaction.supports_transactions?/1","doc":"iex> AshCommanded.Commanded.Transaction.supports_transactions?(MyApp.Repo)\n    true\n\n    iex> AshCommanded.Commanded.Transaction.supports_transactions?(MyApp.InMemoryRepo)\n    false","ref":"AshCommanded.Commanded.Transaction.html#supports_transactions?/1-examples"},{"type":"type","title":"AshCommanded.Commanded.Transaction.transaction_function/0","doc":"Function that will be executed within a transaction","ref":"AshCommanded.Commanded.Transaction.html#t:transaction_function/0"},{"type":"type","title":"AshCommanded.Commanded.Transaction.transaction_options/0","doc":"Options for transaction execution","ref":"AshCommanded.Commanded.Transaction.html#t:transaction_options/0"},{"type":"module","title":"AshCommanded.Commanded.Dsl","doc":"The DSL extension for using Commanded with Ash resources.\n\nThis extension provides the ability to define commands, events, projections, and event handlers using Ash resources,\nand integrates with Commanded to provide CQRS and Event Sourcing patterns.","ref":"AshCommanded.Commanded.Dsl.html"},{"type":"module","title":"Usage - AshCommanded.Commanded.Dsl","doc":"```elixir\ndefmodule MyApp.User do\n  use Ash.Resource,\n    extensions: [AshCommanded.Commanded.Dsl]\n    \n  commanded do\n    commands do\n      command :register_user do\n        fields([:id, :email, :name])\n        identity_field(:id)\n      end\n    end\n    \n    events do\n      event :user_registered do\n        fields([:id, :email, :name])\n      end\n    end\n    \n    projections do\n      projection :user_registered do\n        changes(%{status: :active})\n      end\n    end\n    \n    event_handlers do\n      handler :user_registered_notification do\n        events [:user_registered]\n        action fn event, _metadata ->\n          MyApp.Notifications.send_welcome_email(event.email)\n          :ok\n        end\n      end\n    end\n  end\nend\n```","ref":"AshCommanded.Commanded.Dsl.html#module-usage"},{"type":"function","title":"AshCommanded.Commanded.Dsl.__sections__/0","doc":"Return the section definitions for use in custom DSL extensions.\n\nThis is used in tests to create custom DSL extensions without verifiers.","ref":"AshCommanded.Commanded.Dsl.html#__sections__/0"},{"type":"function","title":"AshCommanded.Commanded.Dsl.application/1","doc":"Get the application configuration from the DSL.","ref":"AshCommanded.Commanded.Dsl.html#application/1"},{"type":"function","title":"Parameters - AshCommanded.Commanded.Dsl.application/1","doc":"* `dsl` - The DSL state.","ref":"AshCommanded.Commanded.Dsl.html#application/1-parameters"},{"type":"function","title":"Returns - AshCommanded.Commanded.Dsl.application/1","doc":"A keyword list of application configuration options, or nil if no application configuration is defined.","ref":"AshCommanded.Commanded.Dsl.html#application/1-returns"},{"type":"function","title":"Examples - AshCommanded.Commanded.Dsl.application/1","doc":"iex> application(dsl)\n    [otp_app: :my_app, event_store: Commanded.EventStore.Adapters.InMemory]\n    \n    iex> application(dsl_without_app_config)\n    nil","ref":"AshCommanded.Commanded.Dsl.html#application/1-examples"},{"type":"macro","title":"AshCommanded.Commanded.Dsl.commanded/1","doc":"","ref":"AshCommanded.Commanded.Dsl.html#commanded/1"},{"type":"function","title":"AshCommanded.Commanded.Dsl.extension?/1","doc":"Determine if a resource uses the `commanded` extension.","ref":"AshCommanded.Commanded.Dsl.html#extension?/1"},{"type":"function","title":"Examples - AshCommanded.Commanded.Dsl.extension?/1","doc":"iex> AshCommanded.Commanded.Dsl.extension?(SomeResource)\n    true\n    \n    iex> AshCommanded.Commanded.Dsl.extension?(OtherResource)\n    false","ref":"AshCommanded.Commanded.Dsl.html#extension?/1-examples"},{"type":"module","title":"AshCommanded.Commanded.Sections.ApplicationSection","doc":"Defines the schema for the `application` section of the Commanded DSL.\n\nThe application section allows configuring a Commanded application for an Ash domain,\nspecifying settings such as the event store adapter, pubsub mechanism, and registry.","ref":"AshCommanded.Commanded.Sections.ApplicationSection.html"},{"type":"module","title":"Example - AshCommanded.Commanded.Sections.ApplicationSection","doc":"```elixir\ndefmodule MyApp.Domain do\n  use Ash.Domain\n  \n  commanded do\n    application do\n      otp_app :my_app\n      event_store Commanded.EventStore.Adapters.EventStore\n      pubsub :local\n      registry :local\n      include_supervisor? true\n      prefix \"MyApp\"\n    end\n  end\n  \n  # resources...\nend\n```","ref":"AshCommanded.Commanded.Sections.ApplicationSection.html#module-example"},{"type":"function","title":"AshCommanded.Commanded.Sections.ApplicationSection.entities/0","doc":"Returns the entities for the application section.\n\nThe application section doesn't define entities, only configuration options.","ref":"AshCommanded.Commanded.Sections.ApplicationSection.html#entities/0"},{"type":"function","title":"AshCommanded.Commanded.Sections.ApplicationSection.schema/0","doc":"Returns the schema for the application section.","ref":"AshCommanded.Commanded.Sections.ApplicationSection.html#schema/0"},{"type":"module","title":"AshCommanded.Commanded.Sections.CommandsSection","doc":"Defines the schema and entities for the `commands` section of the Commanded DSL.\n\nCommands define the actions that can be performed on a resource and correspond to commands\nin the Commanded CQRS/ES library.","ref":"AshCommanded.Commanded.Sections.CommandsSection.html"},{"type":"function","title":"AshCommanded.Commanded.Sections.CommandsSection.entities/0","doc":"","ref":"AshCommanded.Commanded.Sections.CommandsSection.html#entities/0"},{"type":"function","title":"AshCommanded.Commanded.Sections.CommandsSection.schema/0","doc":"","ref":"AshCommanded.Commanded.Sections.CommandsSection.html#schema/0"},{"type":"module","title":"AshCommanded.Commanded.Sections.EventHandlersSection","doc":"Defines the schema and entities for the `event_handlers` section of the Commanded DSL.\n\nEvent handlers are general purpose responders to events that don't necessarily update\nresource state but can perform side effects or other operations.","ref":"AshCommanded.Commanded.Sections.EventHandlersSection.html"},{"type":"function","title":"AshCommanded.Commanded.Sections.EventHandlersSection.entities/0","doc":"Returns the entities for the event_handlers section","ref":"AshCommanded.Commanded.Sections.EventHandlersSection.html#entities/0"},{"type":"function","title":"Examples - AshCommanded.Commanded.Sections.EventHandlersSection.entities/0","doc":"iex> AshCommanded.Commanded.Sections.EventHandlersSection.entities()\n    [%Spark.Dsl.Entity{name: :handler, ...}]","ref":"AshCommanded.Commanded.Sections.EventHandlersSection.html#entities/0-examples"},{"type":"function","title":"AshCommanded.Commanded.Sections.EventHandlersSection.schema/0","doc":"Returns the schema for the event_handlers section","ref":"AshCommanded.Commanded.Sections.EventHandlersSection.html#schema/0"},{"type":"function","title":"Examples - AshCommanded.Commanded.Sections.EventHandlersSection.schema/0","doc":"iex> AshCommanded.Commanded.Sections.EventHandlersSection.schema()\n    [\n      event_handlers: [\n        type: {:list, :any},\n        default: [],\n        doc: \"The handlers that respond to events without necessarily updating resource state\"\n      ]\n    ]","ref":"AshCommanded.Commanded.Sections.EventHandlersSection.html#schema/0-examples"},{"type":"module","title":"AshCommanded.Commanded.Sections.EventsSection","doc":"Defines the schema and entities for the `events` section of the Commanded DSL.\n\nEvents represent facts that have occurred in the system and are emitted by commands.","ref":"AshCommanded.Commanded.Sections.EventsSection.html"},{"type":"function","title":"AshCommanded.Commanded.Sections.EventsSection.entities/0","doc":"","ref":"AshCommanded.Commanded.Sections.EventsSection.html#entities/0"},{"type":"function","title":"AshCommanded.Commanded.Sections.EventsSection.schema/0","doc":"","ref":"AshCommanded.Commanded.Sections.EventsSection.html#schema/0"},{"type":"module","title":"AshCommanded.Commanded.Sections.ProjectionsSection","doc":"Defines the schema and entities for the `projections` section of the Commanded DSL.\n\nProjections define how events affect the resource state, transforming events into resource updates.","ref":"AshCommanded.Commanded.Sections.ProjectionsSection.html"},{"type":"function","title":"AshCommanded.Commanded.Sections.ProjectionsSection.entities/0","doc":"","ref":"AshCommanded.Commanded.Sections.ProjectionsSection.html#entities/0"},{"type":"function","title":"AshCommanded.Commanded.Sections.ProjectionsSection.schema/0","doc":"","ref":"AshCommanded.Commanded.Sections.ProjectionsSection.html#schema/0"},{"type":"module","title":"AshCommanded.Commanded.Transformers.BaseTransformer","doc":"A base module for Commanded DSL transformers.\n\nThis module provides common functionality for transformers that generate code\nbased on the Commanded DSL configuration.","ref":"AshCommanded.Commanded.Transformers.BaseTransformer.html"},{"type":"function","title":"AshCommanded.Commanded.Transformers.BaseTransformer.camelize_atom/1","doc":"Converts an atom to CamelCase string.","ref":"AshCommanded.Commanded.Transformers.BaseTransformer.html#camelize_atom/1"},{"type":"function","title":"Examples - AshCommanded.Commanded.Transformers.BaseTransformer.camelize_atom/1","doc":"iex> camelize_atom(:register_user)\n    \"RegisterUser\"","ref":"AshCommanded.Commanded.Transformers.BaseTransformer.html#camelize_atom/1-examples"},{"type":"function","title":"AshCommanded.Commanded.Transformers.BaseTransformer.create_module/3","doc":"Creates a module with the given name and contents.\n\nThis is a wrapper around Module.create/3 that handles the environment location.","ref":"AshCommanded.Commanded.Transformers.BaseTransformer.html#create_module/3"},{"type":"function","title":"Examples - AshCommanded.Commanded.Transformers.BaseTransformer.create_module/3","doc":"iex> create_module(MyApp.Commands.RegisterUser, quoted_ast, __ENV__)\n    :ok","ref":"AshCommanded.Commanded.Transformers.BaseTransformer.html#create_module/3-examples"},{"type":"function","title":"AshCommanded.Commanded.Transformers.BaseTransformer.get_module_prefix/1","doc":"Gets the module prefix for generated modules.\n\nThis extracts the base namespace from the resource module, which will be used\nas the prefix for all generated modules.","ref":"AshCommanded.Commanded.Transformers.BaseTransformer.html#get_module_prefix/1"},{"type":"function","title":"Examples - AshCommanded.Commanded.Transformers.BaseTransformer.get_module_prefix/1","doc":"iex> get_module_prefix(MyApp.Accounts.User)\n    MyApp.Accounts","ref":"AshCommanded.Commanded.Transformers.BaseTransformer.html#get_module_prefix/1-examples"},{"type":"function","title":"AshCommanded.Commanded.Transformers.BaseTransformer.get_resource_name/1","doc":"Gets the resource name from the resource module.","ref":"AshCommanded.Commanded.Transformers.BaseTransformer.html#get_resource_name/1"},{"type":"function","title":"Examples - AshCommanded.Commanded.Transformers.BaseTransformer.get_resource_name/1","doc":"iex> get_resource_name(MyApp.Accounts.User)\n    \"User\"","ref":"AshCommanded.Commanded.Transformers.BaseTransformer.html#get_resource_name/1-examples"},{"type":"module","title":"AshCommanded.Commanded.Transformers.CollectParameterTransforms","doc":"A transformer that collects parameter transformation specifications from the DSL.\n\nThis transformer runs through all commands in the DSL and collects the \ntransformation specifications from `transform_params` blocks, converting them\ninto a format that can be used during command execution.","ref":"AshCommanded.Commanded.Transformers.CollectParameterTransforms.html"},{"type":"function","title":"AshCommanded.Commanded.Transformers.CollectParameterTransforms.after?/1","doc":"Specifies that this transformer depends on some entities being extracted first.","ref":"AshCommanded.Commanded.Transformers.CollectParameterTransforms.html#after?/1"},{"type":"function","title":"AshCommanded.Commanded.Transformers.CollectParameterTransforms.after_compile?/0","doc":"","ref":"AshCommanded.Commanded.Transformers.CollectParameterTransforms.html#after_compile?/0"},{"type":"function","title":"AshCommanded.Commanded.Transformers.CollectParameterTransforms.before?/1","doc":"","ref":"AshCommanded.Commanded.Transformers.CollectParameterTransforms.html#before?/1"},{"type":"function","title":"AshCommanded.Commanded.Transformers.CollectParameterTransforms.transform/1","doc":"Transforms the DSL to collect parameter transformations.","ref":"AshCommanded.Commanded.Transformers.CollectParameterTransforms.html#transform/1"},{"type":"module","title":"AshCommanded.Commanded.Transformers.CollectParameterValidations","doc":"A transformer that collects parameter validation specifications from the DSL.\n\nThis transformer runs through all commands in the DSL and collects the \nvalidation specifications from `validate_params` blocks, converting them\ninto a format that can be used during command execution.","ref":"AshCommanded.Commanded.Transformers.CollectParameterValidations.html"},{"type":"function","title":"AshCommanded.Commanded.Transformers.CollectParameterValidations.after?/1","doc":"Specifies that this transformer depends on some entities being extracted first.","ref":"AshCommanded.Commanded.Transformers.CollectParameterValidations.html#after?/1"},{"type":"function","title":"AshCommanded.Commanded.Transformers.CollectParameterValidations.after_compile?/0","doc":"","ref":"AshCommanded.Commanded.Transformers.CollectParameterValidations.html#after_compile?/0"},{"type":"function","title":"AshCommanded.Commanded.Transformers.CollectParameterValidations.before?/1","doc":"","ref":"AshCommanded.Commanded.Transformers.CollectParameterValidations.html#before?/1"},{"type":"function","title":"AshCommanded.Commanded.Transformers.CollectParameterValidations.transform/1","doc":"Transforms the DSL to collect parameter validations.","ref":"AshCommanded.Commanded.Transformers.CollectParameterValidations.html#transform/1"},{"type":"module","title":"AshCommanded.Commanded.Transformers.CollectTransactionOptions","doc":"Collects transaction options from the DSL and applies them to commands.\n\nThis transformer processes both:\n1. Command-level transaction options within each command\n2. Resource-level default transaction options\n3. Transaction entity blocks within commands\n\nThe collected options are stored in each command's data structure for later use\nduring command execution.","ref":"AshCommanded.Commanded.Transformers.CollectTransactionOptions.html"},{"type":"function","title":"AshCommanded.Commanded.Transformers.CollectTransactionOptions.after?/1","doc":"","ref":"AshCommanded.Commanded.Transformers.CollectTransactionOptions.html#after?/1"},{"type":"function","title":"AshCommanded.Commanded.Transformers.CollectTransactionOptions.after_compile?/0","doc":"","ref":"AshCommanded.Commanded.Transformers.CollectTransactionOptions.html#after_compile?/0"},{"type":"function","title":"AshCommanded.Commanded.Transformers.CollectTransactionOptions.before?/1","doc":"","ref":"AshCommanded.Commanded.Transformers.CollectTransactionOptions.html#before?/1"},{"type":"function","title":"AshCommanded.Commanded.Transformers.CollectTransactionOptions.transform/1","doc":"Transforms the DSL state to collect transaction options.","ref":"AshCommanded.Commanded.Transformers.CollectTransactionOptions.html#transform/1"},{"type":"module","title":"AshCommanded.Commanded.Transformers.GenerateAggregateModule","doc":"Generates an aggregate module based on the commands and events defined in the DSL.\n\nFor each resource, this transformer will generate an aggregate module that:\n1. Defines a struct representing the aggregate state\n2. Implements the `execute/2` function for handling commands\n3. Implements the `apply/2` function for applying events to the state\n\nThis transformer should run after the command and event module transformers.","ref":"AshCommanded.Commanded.Transformers.GenerateAggregateModule.html"},{"type":"module","title":"Example - AshCommanded.Commanded.Transformers.GenerateAggregateModule","doc":"Given a resource with commands and events, this transformer will generate:\n\n```elixir\ndefmodule MyApp.UserAggregate do\n  @moduledoc \"Aggregate for User resource\"\n  \n  # Define the aggregate state struct\n  defstruct [:id, :email, :name, :status]\n  \n  # Command handlers\n  def execute(%__MODULE__{} = aggregate, %MyApp.Commands.RegisterUser{} = command) do\n    # Validate command - in this case, prevent duplicate registration\n    if aggregate.id != nil do\n      {:error, :user_already_registered}\n    else\n      # Return event(s) to be applied\n      {:ok, %MyApp.Events.UserRegistered{\n        id: command.id,\n        email: command.email,\n        name: command.name\n      }}\n    end\n  end\n  \n  def execute(%__MODULE__{} = aggregate, %MyApp.Commands.UpdateEmail{} = command) do\n    # Validate command - only allow updating existing user\n    if aggregate.id == nil do\n      {:error, :user_not_found}\n    else\n      # Return event(s) to be applied\n      {:ok, %MyApp.Events.EmailChanged{\n        id: command.id,\n        email: command.email\n      }}\n    end\n  end\n  \n  # Event handlers to update state\n  def apply(%__MODULE__{} = state, %MyApp.Events.UserRegistered{} = event) do\n    %__MODULE__{\n      state |\n      id: event.id,\n      email: event.email,\n      name: event.name,\n      status: :active\n    }\n  end\n  \n  def apply(%__MODULE__{} = state, %MyApp.Events.EmailChanged{} = event) do\n    %__MODULE__{\n      state |\n      email: event.email\n    }\n  end\nend\n```","ref":"AshCommanded.Commanded.Transformers.GenerateAggregateModule.html#module-example"},{"type":"function","title":"AshCommanded.Commanded.Transformers.GenerateAggregateModule.after?/1","doc":"Specifies that this transformer should run after the command and event module transformers.","ref":"AshCommanded.Commanded.Transformers.GenerateAggregateModule.html#after?/1"},{"type":"function","title":"AshCommanded.Commanded.Transformers.GenerateAggregateModule.after_compile?/0","doc":"","ref":"AshCommanded.Commanded.Transformers.GenerateAggregateModule.html#after_compile?/0"},{"type":"function","title":"AshCommanded.Commanded.Transformers.GenerateAggregateModule.before?/1","doc":"","ref":"AshCommanded.Commanded.Transformers.GenerateAggregateModule.html#before?/1"},{"type":"function","title":"AshCommanded.Commanded.Transformers.GenerateAggregateModule.build_aggregate_module/2","doc":"Builds the module name for an aggregate.","ref":"AshCommanded.Commanded.Transformers.GenerateAggregateModule.html#build_aggregate_module/2"},{"type":"function","title":"Examples - AshCommanded.Commanded.Transformers.GenerateAggregateModule.build_aggregate_module/2","doc":"iex> build_aggregate_module(\"User\", MyApp)\n    MyApp.UserAggregate","ref":"AshCommanded.Commanded.Transformers.GenerateAggregateModule.html#build_aggregate_module/2-examples"},{"type":"function","title":"AshCommanded.Commanded.Transformers.GenerateAggregateModule.build_aggregate_module_ast/7","doc":"Builds the AST (Abstract Syntax Tree) for an aggregate module.","ref":"AshCommanded.Commanded.Transformers.GenerateAggregateModule.html#build_aggregate_module_ast/7"},{"type":"function","title":"Examples - AshCommanded.Commanded.Transformers.GenerateAggregateModule.build_aggregate_module_ast/7","doc":"iex> build_aggregate_module_ast(\"User\", attribute_names, commands, events, command_modules, event_modules)\n    {:__block__, [], [{:@, [...], [{:moduledoc, [...], [...]}]}, ...]}","ref":"AshCommanded.Commanded.Transformers.GenerateAggregateModule.html#build_aggregate_module_ast/7-examples"},{"type":"function","title":"AshCommanded.Commanded.Transformers.GenerateAggregateModule.transform/1","doc":"Transforms the DSL state to generate an aggregate module.","ref":"AshCommanded.Commanded.Transformers.GenerateAggregateModule.html#transform/1"},{"type":"function","title":"Examples - AshCommanded.Commanded.Transformers.GenerateAggregateModule.transform/1","doc":"iex> transform(dsl_state)\n    {:ok, updated_dsl_state}","ref":"AshCommanded.Commanded.Transformers.GenerateAggregateModule.html#transform/1-examples"},{"type":"module","title":"AshCommanded.Commanded.Transformers.GenerateCommandModules","doc":"Generates command modules based on the commands defined in the DSL.\n\nFor each command defined in a resource, this transformer will generate a corresponding module\nwith the command name as a struct with the specified fields.\n\nThis transformer should run before any other code generation transformers.","ref":"AshCommanded.Commanded.Transformers.GenerateCommandModules.html"},{"type":"module","title":"Example - AshCommanded.Commanded.Transformers.GenerateCommandModules","doc":"Given a resource with the following command:\n\n```elixir\ncommand :register_user do\n  fields [:id, :email, :name]\n  identity_field :id\nend\n```\n\nThis transformer will generate a module like:\n\n```elixir\ndefmodule MyApp.Commands.RegisterUser do\n  @moduledoc \"Command for registering a user\"\n  \n  @type t :: %__MODULE__{\n    id: String.t(),\n    email: String.t(),\n    name: String.t()\n  }\n  \n  defstruct [:id, :email, :name]\nend\n```","ref":"AshCommanded.Commanded.Transformers.GenerateCommandModules.html#module-example"},{"type":"function","title":"AshCommanded.Commanded.Transformers.GenerateCommandModules.after?/1","doc":"","ref":"AshCommanded.Commanded.Transformers.GenerateCommandModules.html#after?/1"},{"type":"function","title":"AshCommanded.Commanded.Transformers.GenerateCommandModules.after_compile?/0","doc":"","ref":"AshCommanded.Commanded.Transformers.GenerateCommandModules.html#after_compile?/0"},{"type":"function","title":"AshCommanded.Commanded.Transformers.GenerateCommandModules.before?/1","doc":"Specifies that this transformer should run before the event module transformer.","ref":"AshCommanded.Commanded.Transformers.GenerateCommandModules.html#before?/1"},{"type":"function","title":"AshCommanded.Commanded.Transformers.GenerateCommandModules.transform/1","doc":"Transforms the DSL state to generate command modules.","ref":"AshCommanded.Commanded.Transformers.GenerateCommandModules.html#transform/1"},{"type":"function","title":"Examples - AshCommanded.Commanded.Transformers.GenerateCommandModules.transform/1","doc":"iex> transform(dsl_state)\n    updated_dsl_state","ref":"AshCommanded.Commanded.Transformers.GenerateCommandModules.html#transform/1-examples"},{"type":"module","title":"AshCommanded.Commanded.Transformers.GenerateCommandedApplication","doc":"Generates a Commanded.Application module based on the configuration in the application section.\n\nThis transformer:\n1. Extracts application configuration from the DSL\n2. Generates a module that uses Commanded.Application\n3. Configures it with the options from the application section\n4. Includes supervision tree integration if specified","ref":"AshCommanded.Commanded.Transformers.GenerateCommandedApplication.html"},{"type":"function","title":"AshCommanded.Commanded.Transformers.GenerateCommandedApplication.after_compile?/0","doc":"","ref":"AshCommanded.Commanded.Transformers.GenerateCommandedApplication.html#after_compile?/0"},{"type":"function","title":"AshCommanded.Commanded.Transformers.GenerateCommandedApplication.before?/1","doc":"","ref":"AshCommanded.Commanded.Transformers.GenerateCommandedApplication.html#before?/1"},{"type":"function","title":"AshCommanded.Commanded.Transformers.GenerateCommandedApplication.transform/1","doc":"Transforms the DSL state by adding the generated Commanded application module.","ref":"AshCommanded.Commanded.Transformers.GenerateCommandedApplication.html#transform/1"},{"type":"function","title":"Parameters - AshCommanded.Commanded.Transformers.GenerateCommandedApplication.transform/1","doc":"* `dsl` - The DSL state to transform.","ref":"AshCommanded.Commanded.Transformers.GenerateCommandedApplication.html#transform/1-parameters"},{"type":"function","title":"Returns - AshCommanded.Commanded.Transformers.GenerateCommandedApplication.transform/1","doc":"The transformed DSL state with the generated Commanded application module added.","ref":"AshCommanded.Commanded.Transformers.GenerateCommandedApplication.html#transform/1-returns"},{"type":"function","title":"Examples - AshCommanded.Commanded.Transformers.GenerateCommandedApplication.transform/1","doc":"iex> transform(dsl)\n    %{dsl | modules: dsl.modules ++ [generated_module_ast]}","ref":"AshCommanded.Commanded.Transformers.GenerateCommandedApplication.html#transform/1-examples"},{"type":"module","title":"AshCommanded.Commanded.Transformers.GenerateDomainRouterModule","doc":"Generates a domain-specific router module based on the commands defined in a domain's resources.\n\nFor each Ash.Domain containing resources with commands, this transformer will generate \na domain router module that:\n\n1. Defines dispatch functions for all commands in that domain\n2. Routes commands to the appropriate aggregate\n\nThis transformer should run after the command and aggregate module transformers.","ref":"AshCommanded.Commanded.Transformers.GenerateDomainRouterModule.html"},{"type":"module","title":"Example - AshCommanded.Commanded.Transformers.GenerateDomainRouterModule","doc":"Given a domain with resources that have commands, this transformer will generate:\n\n```elixir\ndefmodule MyApp.Accounts.Router do\n  @moduledoc \"Router for commands in the Accounts domain\"\n  \n  use Commanded.Commands.Router\n  \n  # Register User commands\n  dispatch [MyApp.Accounts.Commands.RegisterUser, MyApp.Accounts.Commands.UpdateEmail],\n    to: MyApp.Accounts.UserAggregate,\n    identity: :id\n    \n  # Register Account commands\n  dispatch [MyApp.Accounts.Commands.CreateAccount, MyApp.Accounts.Commands.CloseAccount],\n    to: MyApp.Accounts.AccountAggregate,\n    identity: :id\nend\n```","ref":"AshCommanded.Commanded.Transformers.GenerateDomainRouterModule.html#module-example"},{"type":"function","title":"AshCommanded.Commanded.Transformers.GenerateDomainRouterModule.after?/1","doc":"Specifies that this transformer should run after the command and aggregate module transformers.","ref":"AshCommanded.Commanded.Transformers.GenerateDomainRouterModule.html#after?/1"},{"type":"function","title":"AshCommanded.Commanded.Transformers.GenerateDomainRouterModule.after_compile?/0","doc":"","ref":"AshCommanded.Commanded.Transformers.GenerateDomainRouterModule.html#after_compile?/0"},{"type":"function","title":"AshCommanded.Commanded.Transformers.GenerateDomainRouterModule.before?/1","doc":"","ref":"AshCommanded.Commanded.Transformers.GenerateDomainRouterModule.html#before?/1"},{"type":"function","title":"AshCommanded.Commanded.Transformers.GenerateDomainRouterModule.build_domain_router_module/1","doc":"Builds the module name for a domain router.","ref":"AshCommanded.Commanded.Transformers.GenerateDomainRouterModule.html#build_domain_router_module/1"},{"type":"function","title":"Examples - AshCommanded.Commanded.Transformers.GenerateDomainRouterModule.build_domain_router_module/1","doc":"iex> build_domain_router_module(MyApp.Accounts)\n    MyApp.Accounts.Router","ref":"AshCommanded.Commanded.Transformers.GenerateDomainRouterModule.html#build_domain_router_module/1-examples"},{"type":"function","title":"AshCommanded.Commanded.Transformers.GenerateDomainRouterModule.transform/1","doc":"Transforms the DSL state to generate a domain router module.","ref":"AshCommanded.Commanded.Transformers.GenerateDomainRouterModule.html#transform/1"},{"type":"function","title":"Examples - AshCommanded.Commanded.Transformers.GenerateDomainRouterModule.transform/1","doc":"iex> transform(dsl_state)\n    {:ok, updated_dsl_state}","ref":"AshCommanded.Commanded.Transformers.GenerateDomainRouterModule.html#transform/1-examples"},{"type":"module","title":"AshCommanded.Commanded.Transformers.GenerateEventHandlerModules","doc":"Generates event handler modules based on the event_handlers defined in the DSL.\n\nFor each resource with event_handlers, this transformer will generate handler modules\nthat subscribe to the events and execute the specified actions or functions.\n\nThis transformer should run after the event module transformer.","ref":"AshCommanded.Commanded.Transformers.GenerateEventHandlerModules.html"},{"type":"module","title":"Example - AshCommanded.Commanded.Transformers.GenerateEventHandlerModules","doc":"Given a resource with several event_handlers, this transformer will generate:\n\n```elixir\ndefmodule MyApp.EventHandlers.UserNotificationHandler do\n  @moduledoc \"General purpose event handler for User events\"\n  \n  use Commanded.Event.Handler,\n    application: MyApp.CommandedApplication,\n    name: \"MyApp.EventHandlers.UserNotificationHandler\"\n    \n  def handle(%MyApp.Events.UserRegistered{} = event, _metadata) do\n    # Call the action or execute the function defined in the DSL\n    MyApp.Notifications.send_welcome_email(event.email)\n    :ok\n  end\n  \n  # More handlers for other events...\nend\n```","ref":"AshCommanded.Commanded.Transformers.GenerateEventHandlerModules.html#module-example"},{"type":"function","title":"AshCommanded.Commanded.Transformers.GenerateEventHandlerModules.after?/1","doc":"Specifies that this transformer should run after the event module transformer.","ref":"AshCommanded.Commanded.Transformers.GenerateEventHandlerModules.html#after?/1"},{"type":"function","title":"AshCommanded.Commanded.Transformers.GenerateEventHandlerModules.after_compile?/0","doc":"","ref":"AshCommanded.Commanded.Transformers.GenerateEventHandlerModules.html#after_compile?/0"},{"type":"function","title":"AshCommanded.Commanded.Transformers.GenerateEventHandlerModules.before?/1","doc":"","ref":"AshCommanded.Commanded.Transformers.GenerateEventHandlerModules.html#before?/1"},{"type":"function","title":"AshCommanded.Commanded.Transformers.GenerateEventHandlerModules.build_event_handlers_map/2","doc":"Builds a map of event names to their corresponding handlers.","ref":"AshCommanded.Commanded.Transformers.GenerateEventHandlerModules.html#build_event_handlers_map/2"},{"type":"function","title":"Examples - AshCommanded.Commanded.Transformers.GenerateEventHandlerModules.build_event_handlers_map/2","doc":"iex> build_event_handlers_map(handlers, events)\n    %{user_registered: [handler1, handler2], email_changed: [handler3]}","ref":"AshCommanded.Commanded.Transformers.GenerateEventHandlerModules.html#build_event_handlers_map/2-examples"},{"type":"function","title":"AshCommanded.Commanded.Transformers.GenerateEventHandlerModules.build_handler_module/3","doc":"Builds the module name for an event handler.","ref":"AshCommanded.Commanded.Transformers.GenerateEventHandlerModules.html#build_handler_module/3"},{"type":"function","title":"Examples - AshCommanded.Commanded.Transformers.GenerateEventHandlerModules.build_handler_module/3","doc":"iex> build_handler_module(:notification_handler, \"User\", MyApp)\n    MyApp.EventHandlers.UserNotificationHandler","ref":"AshCommanded.Commanded.Transformers.GenerateEventHandlerModules.html#build_handler_module/3-examples"},{"type":"function","title":"AshCommanded.Commanded.Transformers.GenerateEventHandlerModules.build_handler_module_ast/5","doc":"Builds the AST (Abstract Syntax Tree) for an event handler module.","ref":"AshCommanded.Commanded.Transformers.GenerateEventHandlerModules.html#build_handler_module_ast/5"},{"type":"function","title":"Examples - AshCommanded.Commanded.Transformers.GenerateEventHandlerModules.build_handler_module_ast/5","doc":"iex> build_handler_module_ast(MyApp.User, \"User\", handlers, event_map, event_modules)\n    {:__block__, [], [{:@, [...], [{:moduledoc, [...], [...]}]}, ...]}","ref":"AshCommanded.Commanded.Transformers.GenerateEventHandlerModules.html#build_handler_module_ast/5-examples"},{"type":"function","title":"AshCommanded.Commanded.Transformers.GenerateEventHandlerModules.group_handlers_by_name/1","doc":"Groups handlers by their name or autogenerated name based on resource.","ref":"AshCommanded.Commanded.Transformers.GenerateEventHandlerModules.html#group_handlers_by_name/1"},{"type":"function","title":"Examples - AshCommanded.Commanded.Transformers.GenerateEventHandlerModules.group_handlers_by_name/1","doc":"iex> group_handlers_by_name(handlers)\n    %{notification_handler: [handler1, handler2], integration_handler: [handler3]}","ref":"AshCommanded.Commanded.Transformers.GenerateEventHandlerModules.html#group_handlers_by_name/1-examples"},{"type":"function","title":"AshCommanded.Commanded.Transformers.GenerateEventHandlerModules.transform/1","doc":"Transforms the DSL state to generate event handler modules.","ref":"AshCommanded.Commanded.Transformers.GenerateEventHandlerModules.html#transform/1"},{"type":"function","title":"Examples - AshCommanded.Commanded.Transformers.GenerateEventHandlerModules.transform/1","doc":"iex> transform(dsl_state)\n    {:ok, updated_dsl_state}","ref":"AshCommanded.Commanded.Transformers.GenerateEventHandlerModules.html#transform/1-examples"},{"type":"module","title":"AshCommanded.Commanded.Transformers.GenerateEventModules","doc":"Generates event modules based on the events defined in the DSL.\n\nFor each event defined in a resource, this transformer will generate a corresponding module\nwith the event name as a struct with the specified fields.\n\nThis transformer should run after command modules have been generated.","ref":"AshCommanded.Commanded.Transformers.GenerateEventModules.html"},{"type":"module","title":"Example - AshCommanded.Commanded.Transformers.GenerateEventModules","doc":"Given a resource with the following event:\n\n```elixir\nevent :user_registered do\n  fields [:id, :email, :name]\nend\n```\n\nThis transformer will generate a module like:\n\n```elixir\ndefmodule MyApp.Events.UserRegistered do\n  @moduledoc \"Event representing when a user was registered\"\n  \n  @type t :: %__MODULE__{\n    id: String.t(),\n    email: String.t(),\n    name: String.t()\n  }\n  \n  defstruct [:id, :email, :name]\nend\n```","ref":"AshCommanded.Commanded.Transformers.GenerateEventModules.html#module-example"},{"type":"function","title":"AshCommanded.Commanded.Transformers.GenerateEventModules.after?/1","doc":"Specifies that this transformer should run after the command module transformer.","ref":"AshCommanded.Commanded.Transformers.GenerateEventModules.html#after?/1"},{"type":"function","title":"AshCommanded.Commanded.Transformers.GenerateEventModules.after_compile?/0","doc":"","ref":"AshCommanded.Commanded.Transformers.GenerateEventModules.html#after_compile?/0"},{"type":"function","title":"AshCommanded.Commanded.Transformers.GenerateEventModules.before?/1","doc":"","ref":"AshCommanded.Commanded.Transformers.GenerateEventModules.html#before?/1"},{"type":"function","title":"AshCommanded.Commanded.Transformers.GenerateEventModules.build_event_module/2","doc":"Builds the module name for an event.","ref":"AshCommanded.Commanded.Transformers.GenerateEventModules.html#build_event_module/2"},{"type":"function","title":"Examples - AshCommanded.Commanded.Transformers.GenerateEventModules.build_event_module/2","doc":"iex> build_event_module(%Event{name: :user_registered}, MyApp)\n    MyApp.Events.UserRegistered\n    \n    iex> build_event_module(%Event{name: :user_registered, event_name: :new_user}, MyApp)\n    MyApp.Events.NewUser","ref":"AshCommanded.Commanded.Transformers.GenerateEventModules.html#build_event_module/2-examples"},{"type":"function","title":"AshCommanded.Commanded.Transformers.GenerateEventModules.build_event_module_ast/2","doc":"Builds the AST (Abstract Syntax Tree) for an event module.","ref":"AshCommanded.Commanded.Transformers.GenerateEventModules.html#build_event_module_ast/2"},{"type":"function","title":"Examples - AshCommanded.Commanded.Transformers.GenerateEventModules.build_event_module_ast/2","doc":"iex> build_event_module_ast(%Event{name: :user_registered, fields: [:id]}, \"User\")\n    {:__block__, [], [{:@, [...], [{:moduledoc, [...], [...]}]}, ...]}","ref":"AshCommanded.Commanded.Transformers.GenerateEventModules.html#build_event_module_ast/2-examples"},{"type":"function","title":"AshCommanded.Commanded.Transformers.GenerateEventModules.transform/1","doc":"Transforms the DSL state to generate event modules.","ref":"AshCommanded.Commanded.Transformers.GenerateEventModules.html#transform/1"},{"type":"function","title":"Examples - AshCommanded.Commanded.Transformers.GenerateEventModules.transform/1","doc":"iex> transform(dsl_state)\n    {:ok, updated_dsl_state}","ref":"AshCommanded.Commanded.Transformers.GenerateEventModules.html#transform/1-examples"},{"type":"module","title":"AshCommanded.Commanded.Transformers.GenerateMainRouterModule","doc":"Generates the main router module that delegates to domain-specific routers.\n\nThis transformer collects all command-resource mappings from the DSL state \nand generates:\n\n1. Domain-specific router modules for each domain with commands\n2. A main router module that delegates to these domain routers\n\nThis transformer should run after the domain router transformer has processed all resources.","ref":"AshCommanded.Commanded.Transformers.GenerateMainRouterModule.html"},{"type":"module","title":"Example - AshCommanded.Commanded.Transformers.GenerateMainRouterModule","doc":"```elixir\n# Domain-specific routers\ndefmodule MyApp.Accounts.Router do\n  use Commanded.Commands.Router\n  \n  dispatch [MyApp.Accounts.Commands.RegisterUser],\n    to: MyApp.Accounts.UserAggregate,\n    identity: :id\nend\n\ndefmodule MyApp.Billing.Router do\n  use Commanded.Commands.Router\n  \n  dispatch [MyApp.Billing.Commands.CreateInvoice],\n    to: MyApp.Billing.InvoiceAggregate,\n    identity: :id\nend\n\n# Main router\ndefmodule MyApp.Router do\n  @moduledoc \"Main router for all commands in the application\"\n  \n  use Commanded.Commands.Router\n  \n  # Forward commands to domain-specific routers\n  identify MyApp.Accounts.Commands.RegisterUser, as: :accounts\n  forward :accounts, to: MyApp.Accounts.Router\n  \n  identify MyApp.Billing.Commands.CreateInvoice, as: :billing\n  forward :billing, to: MyApp.Billing.Router\n  \n  # Helper function to dispatch any command\n  def dispatch_command(command) do\n    __MODULE__.dispatch(command)\n  end\nend\n```\n\nFor simpler applications with a single domain, the main router may directly handle commands:\n\n```elixir\ndefmodule MyApp.Router do\n  use Commanded.Commands.Router\n  \n  dispatch [MyApp.Commands.RegisterUser, MyApp.Commands.CreateInvoice],\n    to: MyApp.UserAggregate,\n    identity: :id\nend\n```","ref":"AshCommanded.Commanded.Transformers.GenerateMainRouterModule.html#module-example"},{"type":"function","title":"AshCommanded.Commanded.Transformers.GenerateMainRouterModule.after?/1","doc":"Specifies that this transformer should run after the domain router transformer.","ref":"AshCommanded.Commanded.Transformers.GenerateMainRouterModule.html#after?/1"},{"type":"function","title":"AshCommanded.Commanded.Transformers.GenerateMainRouterModule.after_compile?/0","doc":"","ref":"AshCommanded.Commanded.Transformers.GenerateMainRouterModule.html#after_compile?/0"},{"type":"function","title":"AshCommanded.Commanded.Transformers.GenerateMainRouterModule.before?/1","doc":"","ref":"AshCommanded.Commanded.Transformers.GenerateMainRouterModule.html#before?/1"},{"type":"function","title":"AshCommanded.Commanded.Transformers.GenerateMainRouterModule.build_main_router_module/1","doc":"Builds the module name for the main router.","ref":"AshCommanded.Commanded.Transformers.GenerateMainRouterModule.html#build_main_router_module/1"},{"type":"function","title":"Examples - AshCommanded.Commanded.Transformers.GenerateMainRouterModule.build_main_router_module/1","doc":"iex> build_main_router_module(MyApp)\n    MyApp.Router","ref":"AshCommanded.Commanded.Transformers.GenerateMainRouterModule.html#build_main_router_module/1-examples"},{"type":"function","title":"AshCommanded.Commanded.Transformers.GenerateMainRouterModule.transform/1","doc":"Transforms the DSL state to generate router modules.\n\nThis transformer will:\n1. Collect command mappings from all resources\n2. Generate domain-specific router modules\n3. Generate the main router module","ref":"AshCommanded.Commanded.Transformers.GenerateMainRouterModule.html#transform/1"},{"type":"function","title":"Examples - AshCommanded.Commanded.Transformers.GenerateMainRouterModule.transform/1","doc":"iex> transform(dsl_state)\n    {:ok, updated_dsl_state}","ref":"AshCommanded.Commanded.Transformers.GenerateMainRouterModule.html#transform/1-examples"},{"type":"module","title":"AshCommanded.Commanded.Transformers.GenerateProjectionModules","doc":"Generates projection modules based on the projections defined in the DSL.\n\nFor each projection defined in a resource, this transformer will generate a corresponding module\nthat handles the transformation of events into resource updates.\n\nThis transformer should run after the event module transformer.","ref":"AshCommanded.Commanded.Transformers.GenerateProjectionModules.html"},{"type":"module","title":"Example - AshCommanded.Commanded.Transformers.GenerateProjectionModules","doc":"Given a resource with the following projection:\n\n```elixir\nprojection :user_registered do\n  action :create\n  changes(%{status: :active})\nend\n```\n\nThis transformer will generate a module like:\n\n```elixir\ndefmodule MyApp.Projections.UserRegistered do\n  @moduledoc \"Projection that handles user_registered events\"\n  \n  def apply(event, resource) do\n    # Apply changes to the resource based on the event\n    changes = %{status: :active}\n    Ash.Changeset.for_action(resource, event, :create, changes)\n  end\nend\n```","ref":"AshCommanded.Commanded.Transformers.GenerateProjectionModules.html#module-example"},{"type":"function","title":"AshCommanded.Commanded.Transformers.GenerateProjectionModules.after?/1","doc":"Specifies that this transformer should run after the event module transformer.","ref":"AshCommanded.Commanded.Transformers.GenerateProjectionModules.html#after?/1"},{"type":"function","title":"AshCommanded.Commanded.Transformers.GenerateProjectionModules.after_compile?/0","doc":"","ref":"AshCommanded.Commanded.Transformers.GenerateProjectionModules.html#after_compile?/0"},{"type":"function","title":"AshCommanded.Commanded.Transformers.GenerateProjectionModules.before?/1","doc":"","ref":"AshCommanded.Commanded.Transformers.GenerateProjectionModules.html#before?/1"},{"type":"function","title":"AshCommanded.Commanded.Transformers.GenerateProjectionModules.build_projection_module/2","doc":"Builds the module name for a projection.","ref":"AshCommanded.Commanded.Transformers.GenerateProjectionModules.html#build_projection_module/2"},{"type":"function","title":"Examples - AshCommanded.Commanded.Transformers.GenerateProjectionModules.build_projection_module/2","doc":"iex> build_projection_module(%Projection{name: :user_registered}, MyApp)\n    MyApp.Projections.UserRegistered","ref":"AshCommanded.Commanded.Transformers.GenerateProjectionModules.html#build_projection_module/2-examples"},{"type":"function","title":"AshCommanded.Commanded.Transformers.GenerateProjectionModules.build_projection_module_ast/2","doc":"Builds the AST (Abstract Syntax Tree) for a projection module.","ref":"AshCommanded.Commanded.Transformers.GenerateProjectionModules.html#build_projection_module_ast/2"},{"type":"function","title":"Examples - AshCommanded.Commanded.Transformers.GenerateProjectionModules.build_projection_module_ast/2","doc":"iex> build_projection_module_ast(%Projection{name: :user_registered, action: :create, changes: %{status: :active}}, \"User\")\n    {:__block__, [], [{:@, [...], [{:moduledoc, [...], [...]}]}, ...]}","ref":"AshCommanded.Commanded.Transformers.GenerateProjectionModules.html#build_projection_module_ast/2-examples"},{"type":"function","title":"AshCommanded.Commanded.Transformers.GenerateProjectionModules.transform/1","doc":"Transforms the DSL state to generate projection modules.","ref":"AshCommanded.Commanded.Transformers.GenerateProjectionModules.html#transform/1"},{"type":"function","title":"Examples - AshCommanded.Commanded.Transformers.GenerateProjectionModules.transform/1","doc":"iex> transform(dsl_state)\n    {:ok, updated_dsl_state}","ref":"AshCommanded.Commanded.Transformers.GenerateProjectionModules.html#transform/1-examples"},{"type":"module","title":"AshCommanded.Commanded.Transformers.GenerateProjectorModules","doc":"Generates projector modules based on the projections defined in the DSL.\n\nFor each resource with projections, this transformer will generate a projector module\nthat subscribes to the events and applies the corresponding projections to update\nthe resource state.\n\nThis transformer should run after the projection module transformer.","ref":"AshCommanded.Commanded.Transformers.GenerateProjectorModules.html"},{"type":"module","title":"Example - AshCommanded.Commanded.Transformers.GenerateProjectorModules","doc":"Given a resource with several projections, this transformer will generate:\n\n```elixir\ndefmodule MyApp.Projectors.UserProjector do\n  @moduledoc \"Event handler that projects events onto the User resource\"\n  \n  use Commanded.Event.Handler,\n    application: MyApp.CommandedApplication,\n    name: \"MyApp.Projectors.UserProjector\"\n    \n  alias MyApp.Projections\n  \n  def handle(%MyApp.Events.UserRegistered{} = event, _metadata) do\n    with {:ok, resource} <- Projections.UserRegistered.apply(event),\n         {:ok, _result} <- perform_action(resource, Projections.UserRegistered.action()) do\n      :ok\n    else\n      {:error, reason} -> {:error, reason}\n    end\n  end\n  \n  # More handlers for other events...\n  \n  defp perform_action(changeset, :create), do: Ash.create(changeset)\n  defp perform_action(changeset, :update), do: Ash.update(changeset)\n  defp perform_action(changeset, :destroy), do: Ash.destroy(changeset)\nend\n```","ref":"AshCommanded.Commanded.Transformers.GenerateProjectorModules.html#module-example"},{"type":"function","title":"AshCommanded.Commanded.Transformers.GenerateProjectorModules.after?/1","doc":"Specifies that this transformer should run after the projection module transformer.","ref":"AshCommanded.Commanded.Transformers.GenerateProjectorModules.html#after?/1"},{"type":"function","title":"AshCommanded.Commanded.Transformers.GenerateProjectorModules.after_compile?/0","doc":"","ref":"AshCommanded.Commanded.Transformers.GenerateProjectorModules.html#after_compile?/0"},{"type":"function","title":"AshCommanded.Commanded.Transformers.GenerateProjectorModules.before?/1","doc":"","ref":"AshCommanded.Commanded.Transformers.GenerateProjectorModules.html#before?/1"},{"type":"function","title":"AshCommanded.Commanded.Transformers.GenerateProjectorModules.build_event_projection_map/2","doc":"Builds a map of event names to their corresponding projections.","ref":"AshCommanded.Commanded.Transformers.GenerateProjectorModules.html#build_event_projection_map/2"},{"type":"function","title":"Examples - AshCommanded.Commanded.Transformers.GenerateProjectorModules.build_event_projection_map/2","doc":"iex> build_event_projection_map(projections, events)\n    %{user_registered: [projection1, projection2], email_changed: [projection3]}","ref":"AshCommanded.Commanded.Transformers.GenerateProjectorModules.html#build_event_projection_map/2-examples"},{"type":"function","title":"AshCommanded.Commanded.Transformers.GenerateProjectorModules.build_projector_module/2","doc":"Builds the module name for a projector.","ref":"AshCommanded.Commanded.Transformers.GenerateProjectorModules.html#build_projector_module/2"},{"type":"function","title":"Examples - AshCommanded.Commanded.Transformers.GenerateProjectorModules.build_projector_module/2","doc":"iex> build_projector_module(\"User\", MyApp)\n    MyApp.Projectors.UserProjector","ref":"AshCommanded.Commanded.Transformers.GenerateProjectorModules.html#build_projector_module/2-examples"},{"type":"function","title":"AshCommanded.Commanded.Transformers.GenerateProjectorModules.build_projector_module_ast/6","doc":"Builds the AST (Abstract Syntax Tree) for a projector module.","ref":"AshCommanded.Commanded.Transformers.GenerateProjectorModules.html#build_projector_module_ast/6"},{"type":"function","title":"Examples - AshCommanded.Commanded.Transformers.GenerateProjectorModules.build_projector_module_ast/6","doc":"iex> build_projector_module_ast(MyApp.User, \"User\", projections, event_map, event_modules, projection_modules)\n    {:__block__, [], [{:@, [...], [{:moduledoc, [...], [...]}]}, ...]}","ref":"AshCommanded.Commanded.Transformers.GenerateProjectorModules.html#build_projector_module_ast/6-examples"},{"type":"function","title":"AshCommanded.Commanded.Transformers.GenerateProjectorModules.transform/1","doc":"Transforms the DSL state to generate projector modules.","ref":"AshCommanded.Commanded.Transformers.GenerateProjectorModules.html#transform/1"},{"type":"function","title":"Examples - AshCommanded.Commanded.Transformers.GenerateProjectorModules.transform/1","doc":"iex> transform(dsl_state)\n    {:ok, updated_dsl_state}","ref":"AshCommanded.Commanded.Transformers.GenerateProjectorModules.html#transform/1-examples"},{"type":"module","title":"AshCommanded.Commanded.Verifiers.ValidateCommandFields","doc":"Verifies that all fields declared in commands exist as attributes in the resource.\n\nThis ensures that commands don't reference non-existent fields, which would lead to\nruntime errors.","ref":"AshCommanded.Commanded.Verifiers.ValidateCommandFields.html"},{"type":"module","title":"AshCommanded.Commanded.Verifiers.ValidateCommandNames","doc":"Verifies that each command name within a resource is unique.\n\nThis ensures that there are no duplicate command names, which would lead to\nconfusion and potentially overwriting modules.","ref":"AshCommanded.Commanded.Verifiers.ValidateCommandNames.html"},{"type":"module","title":"AshCommanded.Commanded.Verifiers.ValidateEventFields","doc":"Verifies that all fields declared in events exist as attributes in the resource.\n\nThis ensures that events don't reference non-existent fields, which would lead to\nruntime errors.","ref":"AshCommanded.Commanded.Verifiers.ValidateEventFields.html"},{"type":"module","title":"AshCommanded.Commanded.Verifiers.ValidateEventHandlerActions","doc":"Verifies that event handlers reference valid actions when using atom-based actions.\n\nThis ensures that when an event handler uses an action name (atom) rather than a function,\nthe action exists as a valid action in the resource.","ref":"AshCommanded.Commanded.Verifiers.ValidateEventHandlerActions.html"},{"type":"function","title":"AshCommanded.Commanded.Verifiers.ValidateEventHandlerActions.verify/1","doc":"Verifies that all action names referenced by event handlers exist or are common actions.","ref":"AshCommanded.Commanded.Verifiers.ValidateEventHandlerActions.html#verify/1"},{"type":"function","title":"Returns - AshCommanded.Commanded.Verifiers.ValidateEventHandlerActions.verify/1","doc":"* `:ok` - If all action references are valid or handlers use function definitions\n* `{:error, error}` - If any event handler references a potentially invalid action","ref":"AshCommanded.Commanded.Verifiers.ValidateEventHandlerActions.html#verify/1-returns"},{"type":"module","title":"AshCommanded.Commanded.Verifiers.ValidateEventHandlerEvents","doc":"Verifies that event handlers only reference valid events defined in the resource.\n\nThis ensures that each event handler is tied to valid events that were defined\nin the resource's events section, preventing references to non-existent events.","ref":"AshCommanded.Commanded.Verifiers.ValidateEventHandlerEvents.html"},{"type":"function","title":"AshCommanded.Commanded.Verifiers.ValidateEventHandlerEvents.verify/1","doc":"Verifies that all events referenced by event handlers exist in the resource.","ref":"AshCommanded.Commanded.Verifiers.ValidateEventHandlerEvents.html#verify/1"},{"type":"function","title":"Returns - AshCommanded.Commanded.Verifiers.ValidateEventHandlerEvents.verify/1","doc":"* `:ok` - If all events referenced by event handlers exist\n* `{:error, error}` - If any event handler references a non-existent event","ref":"AshCommanded.Commanded.Verifiers.ValidateEventHandlerEvents.html#verify/1-returns"},{"type":"module","title":"AshCommanded.Commanded.Verifiers.ValidateEventNames","doc":"Verifies that each event name within a resource is unique.\n\nThis ensures that there are no duplicate event names, which would lead to\nconfusion and potentially overwriting modules.","ref":"AshCommanded.Commanded.Verifiers.ValidateEventNames.html"},{"type":"module","title":"AshCommanded.Commanded.Verifiers.ValidateProjectionActions","doc":"Verifies that projections specify valid Ash actions.\n\nThis ensures that each projection specifies an action that is appropriate for\nresource operations, such as :create, :update, or :destroy.","ref":"AshCommanded.Commanded.Verifiers.ValidateProjectionActions.html"},{"type":"module","title":"AshCommanded.Commanded.Verifiers.ValidateProjectionChanges","doc":"Verifies that projection changes contain valid attribute references.\n\nThis ensures that when using a static map of changes, all keys in the map\ncorrespond to attributes that exist on the resource.","ref":"AshCommanded.Commanded.Verifiers.ValidateProjectionChanges.html"},{"type":"module","title":"AshCommanded.Commanded.Verifiers.ValidateProjectionEvents","doc":"Verifies that projections reference valid events defined in the resource.\n\nThis ensures that each projection is tied to a valid event that was defined\nin the resource's events section, preventing references to non-existent events.","ref":"AshCommanded.Commanded.Verifiers.ValidateProjectionEvents.html"},{"type":"extras","title":"README","doc":"","ref":"readme.html"},{"type":"extras","title":"Overview - README","doc":"AshCommanded is an Elixir library that provides [Command Query Responsibility Segregation (CQRS)](https://martinfowler.com/bliki/CQRS.html) and [Event-Sourcing (ES)](https://martinfowler.com/eaaDev/EventSourcing.html) patterns for the [Ash Framework](https://hexdocs.pm/ash/). It extends Ash resources with a Commanded DSL that enables defining commands, events, and projections. The extension relies on the excellent [Commanded](https://hexdocs.pm/commanded/Commanded.html) library. The [Commanded Guides](https://hexdocs.pm/commanded/commands.html) section explains the different concepts better than I could.\n\nSpecial thanks to [Ben Smith](https://github.com/slashdotdash) for the Commanded library and to [Barnabas J.] for letting me steal the library name.","ref":"readme.html#overview"},{"type":"extras","title":"Build and Test Commands - README","doc":"```bash\n# Install dependencies\nmix deps.get\n\n# Compile the project\nmix compile\n\n# Run all tests\nmix test\n\n# Run specific test file\nmix test path/to/test_file.exs:\n\n# Run specific test with line number\nmix test path/to/test_file.exs:42:\n\n# Run tests with coverage\nmix test --cover:\n```","ref":"readme.html#build-and-test-commands"},{"type":"extras","title":"Architecture - README","doc":"AshCommanded is built as a DSL extension for [Ash Framework](https://hexdocs.pm/ash/) resources using the [Spark DSL](https://hexdocs.pm/spark/) library for its extensible DSL capabilities. Its main components are:\n\n1. **DSL Extension**: The [`AshCommanded.Commanded.Dsl`](lib/commanded/dsl.ex) module defines five main sections:\n   - [`commands`](lib/commanded/sections/commands_section.ex): Define commands that trigger state changes\n   - [`events`](lib/commanded/sections/events_section.ex): Define events that are emitted by commands\n   - [`projections`](lib/commanded/sections/projections_section.ex): Define how events affect the resource state\n   - [`event_handlers`](lib/commanded/sections/event_handlers_section.ex): Define general purpose handlers for events\n   - [`application`](lib/commanded/sections/application_section.ex): Configure Commanded application settings\n\n2. **Code Generation**: The library dynamically generates Elixir modules:\n   - Command modules (structs with typespecs)\n   - Event modules (structs with typespecs)\n   - Projection modules (with event handlers)\n   - Projector modules (Commanded event handlers that apply projections)\n   - Event handler modules (general purpose event subscribers)\n   - Aggregate modules (for Commanded integration)\n   - Router modules (for command dispatching)\n   - Commanded application modules (with projector and handler supervision)\n\n3. **Transformers**: The DSL uses transformers to generate code:\n   - [`GenerateCommandModules`](lib/commanded/transformers/generate_command_modules.ex): Generates command structs\n   - [`GenerateEventModules`](lib/commanded/transformers/generate_event_modules.ex): Generates event structs\n   - [`GenerateProjectionModules`](lib/commanded/transformers/generate_projection_modules.ex): Generates projection modules\n   - [`GenerateProjectorModules`](lib/commanded/transformers/generate_projector_modules.ex): Generates Commanded event handlers that process events\n   - [`GenerateEventHandlerModules`](lib/commanded/transformers/generate_event_handler_modules.ex): Generates general purpose event handlers\n   - [`GenerateAggregateModule`](lib/commanded/transformers/generate_aggregate_module.ex): Generates aggregate module for Commanded\n   - [`GenerateDomainRouterModule`](lib/commanded/transformers/generate_domain_router_module.ex): Generates router module for each domain\n   - [`GenerateMainRouterModule`](lib/commanded/transformers/generate_main_router_module.ex): Generates main application router\n   - [`GenerateCommandedApplication`](lib/commanded/transformers/generate_commanded_application.ex): Generates Commanded application with projector and handler supervision\n\n4. **Advanced Features**:\n   - **Command Middleware**: Process commands through a pipeline of middleware functions\n   - **Parameter Transformation**: Transform command parameters before action execution\n   - **Parameter Validation**: Validate command parameters before action execution\n   - **Transactional Commands**: Execute commands within database transactions\n   - **Context Propagation**: Pass command, aggregate, and metadata context to actions\n   - **Error Standardization**: Normalized error handling across the extension\n\n5. **Verifiers**: Validate DSL usage:\n   - Command validation (names, fields, handlers, etc.)\n   - Event validation (names, fields, etc.)\n   - Projection validation (events, actions, changes, etc.)\n   - Event handler validation (events, actions, etc.)","ref":"readme.html#architecture"},{"type":"extras","title":"Usage Example - README","doc":"```elixir\ndefmodule ECommerce.Customer do\n  use Ash.Resource,\n    extensions: [AshCommanded.Commanded.Dsl]\n\n  attributes do\n    uuid_primary_key :id\n    attribute :name, :string\n    attribute :email, :string\n    attribute :status, :atom, constraints: [one_of: [:pending, :active]]\n  end\n\n  identities do\n    identity :unique_id, [:id]\n  end\n\n  actions do\n    defaults [:read]\n\n    create :register do\n      accept [:name, :email]\n      change {Ash.Changeset, :set_attribute, [:status, :pending]}\n    end\n\n    update :confirm_email do\n      accept []\n      change {Ash.Changeset, :set_attribute, [:status, :active]}\n    end\n  end\n\n  commanded do\n    commands do\n      command :register_customer do\n        fields([:id, :name, :email])\n        identity_field(:id)\n        action :register\n      end\n\n      command :confirm_email do\n        fields([:id])\n        identity_field(:id)\n        action :confirm_email\n      end\n    end\n\n    events do\n      event :customer_registered do\n        fields([:id, :name, :email])\n      end\n\n      event :email_confirmed do\n        fields([:id])\n      end\n    end\n\n    projections do\n      projection :customer_registered do\n        action(:create)\n        changes(%{\n          status: :pending\n        })\n      end\n\n      projection :email_confirmed do\n        action(:update_by_id)\n        changes(%{\n          status: :active\n        })\n      end\n    end\n    \n    event_handlers do\n      handler :notification_handler do\n        events [:customer_registered]\n        action fn event, _metadata ->\n          ECommerce.Notifications.send_welcome_email(event.email)\n          :ok\n        end\n      end\n      \n      handler :analytics_tracker do\n        events [:customer_registered, :email_confirmed]\n        action fn event, _metadata ->\n          ECommerce.Analytics.track(event)\n          :ok\n        end\n      end\n    end\n  end\nend\n```\n\nThis will generate:\n- `ECommerce.Commands.RegisterCustomer` - Command struct\n- `ECommerce.Events.CustomerRegistered` - Event struct\n- `ECommerce.Projections.CustomerRegistered` - Projection definition\n- `ECommerce.Projectors.CustomerProjector` - Commanded event handler for projections\n- `ECommerce.EventHandlers.CustomerNotificationHandler` - General purpose event handler\n- `ECommerce.EventHandlers.CustomerAnalyticsTrackerHandler` - General purpose event handler\n- `ECommerce.CustomerAggregate` - Aggregate module\n- `ECommerce.Store.Router` - Domain-specific router (if in an Ash.Domain)\n- `AshCommanded.Router` - Main application router","ref":"readme.html#usage-example"},{"type":"extras","title":"Documentation - README","doc":"AshCommanded provides comprehensive documentation that can be generated locally:\n\n```bash\n# Install dependencies\nmix deps.get\n\n# Generate cheatsheet and docs\nmix gen.docs\n```\n\nThe documentation includes:\n- Guides for commands, events, projections, event handlers, and routers\n- Guides for middleware, parameter handling, transactions, and context propagation\n- API reference for all modules\n- Cheatsheets for the DSL\n\nAdditional documentation files:\n- [Commands](commands.html)\n- [Events](events.html)\n- [Projections](projections.html)\n- [Event Handlers](event_handlers.html)\n- [Middleware](middleware.html)\n- [Parameter Handling](parameter_handling.html)\n- [Transactions](transactions.html)\n- [Context Propagation](context_propagation.html)\n- [Error Handling](error_handling.html)\n- [Application](application.html)\n- [Routers](routers.html)\n- [Snapshotting](snapshotting.html)","ref":"readme.html#documentation"},{"type":"extras","title":"Commands - README","doc":"Commands define the actions that can be performed on your resources. AshCommanded generates command modules as structs with typespecs.\n\n```elixir\ncommanded do\n  commands do\n    command :register_customer do\n      fields([:id, :name, :email])\n      identity_field(:id)\n      action :register\n    end\n\n    command :confirm_email do\n      fields([:id])\n      identity_field(:id)\n      action :confirm_email\n    end\n  end\nend\n```\n\nGenerated command modules include:\n\n- A struct with the specified fields\n- Typespecs for all fields\n- Standard module documentation\n\nExample generated command:\n```elixir\ndefmodule ECommerce.Commands.RegisterCustomer do\n  @moduledoc \"\"\"\n  Command for registering a new customer\n  \"\"\"\n  \n  @type t :: %__MODULE__{\n    id: String.t(),\n    email: String.t(),\n    name: String.t(),\n    status: atom()\n  }\n  \n  defstruct [:id, :email, :name, :status]\nend\n```","ref":"readme.html#commands"},{"type":"extras","title":"Command Handlers - README","doc":"Command handlers are modules that process commands and apply business logic. AshCommanded generates handler modules that invoke Ash actions.\n\n```elixir\ndefmodule AshCommanded.Commanded.CommandHandlers.CustomerHandler do\n  @behaviour Commanded.Commands.Handler\n  \n  def handle(%ECommerce.Commands.RegisterCustomer{} = cmd, _metadata) do\n    Ash.run_action(ECommerce.Customer, :register, Map.from_struct(cmd))\n  end\n  \n  def handle(%ECommerce.Commands.ConfirmEmail{} = cmd, _metadata) do\n    Ash.run_action(ECommerce.Customer, :confirm_email, Map.from_struct(cmd))\n  end\nend\n```\n\nHandler options:\n- `handler_name` - Custom function name for the handler clause\n- `action` - Specify a different Ash action to call (defaults to command name)\n- `autogenerate_handler?` - Set to false to disable handler generation","ref":"readme.html#command-handlers"},{"type":"extras","title":"Middleware, Parameter Handling, and Transactions - README","doc":"AshCommanded provides advanced features for command processing:","ref":"readme.html#middleware-parameter-handling-and-transactions"},{"type":"extras","title":"Middleware - README","doc":"Command middleware allows you to intercept and modify commands before they are executed:\n\n```elixir\ncommanded do\n  commands do\n    # Apply middleware to all commands in this resource\n    middleware AuditLogger\n    middleware {Authorization, roles: [:admin]}\n    \n    command :register_customer do\n      fields([:id, :name, :email])\n      # Command-specific middleware\n      middleware {RateLimiter, limit: 10}\n    end\n  end\nend\n```","ref":"readme.html#middleware"},{"type":"extras","title":"Parameter Transformation and Validation - README","doc":"You can transform and validate command parameters before action execution:\n\n```elixir\ncommand :create_order do\n  fields([:id, :items, :customer_id, :total])\n  \n  transform_params do\n    map item_ids: :items\n    compute :timestamp, &DateTime.utc_now/0\n    cast :total, :decimal\n  end\n  \n  validate_params do\n    validate :total, number: [greater_than: 0]\n    validate :items, present: true\n  end\nend\n```","ref":"readme.html#parameter-transformation-and-validation"},{"type":"extras","title":"Transaction Support - README","doc":"Execute commands within database transactions:\n\n```elixir\ncommand :place_order do\n  fields [:id, :customer_id, :items]\n  \n  # Use inline transaction options\n  in_transaction? true\n  repo MyApp.Repo\n  transaction_timeout 5000\n  transaction_isolation_level :serializable\n  \n  # Or use block syntax\n  transaction do\n    enabled? true\n    repo MyApp.Repo\n    timeout 5000\n    isolation_level :read_committed\n  end\nend\n```","ref":"readme.html#transaction-support"},{"type":"extras","title":"Context Propagation - README","doc":"Control how command context is passed to actions:\n\n```elixir\ncommand :register_customer do\n  fields [:id, :name, :email]\n  \n  # Context options\n  include_aggregate? true\n  include_command? true\n  include_metadata? true\n  context_prefix :cmd\n  static_context %{source: :registration_api}\nend\n```","ref":"readme.html#context-propagation"},{"type":"extras","title":"Events - README","doc":"Events represent facts that have occurred in your system. AshCommanded generates event modules as structs with typespecs.\n\n```elixir\ncommanded do\n  events do\n    event :customer_registered do\n      fields([:id, :name, :email])\n    end\n\n    event :email_confirmed do\n      fields([:id])\n    end\n  end\nend\n```\n\nGenerated event modules include:\n- A struct with the specified fields\n- Typespecs for all fields\n- Standard module documentation\n\nExample generated event:\n```elixir\ndefmodule ECommerce.Events.CustomerRegistered do\n  @moduledoc \"\"\"\n  Event emitted when a customer is registered\n  \"\"\"\n  \n  @type t :: %__MODULE__{\n    id: String.t(),\n    email: String.t(),\n    name: String.t(),\n    status: atom()\n  }\n  \n  defstruct [:id, :email, :name, :status]\nend\n```","ref":"readme.html#events"},{"type":"extras","title":"Aggregates and Events-Handlers - README","doc":"Aggregates process events and update state. AshCommanded generates aggregate modules for each resource.\nEach event that mutate state is handled by the Aggregate via an apply function that is automatically generated for you.\n```elixir\ndefmodule ECommerce.CustomerAggregate do\n  defstruct [:id, :email, :name, :status]\n  \n  # Apply event to update the aggregate state\n  def apply(%__MODULE__{} = state, %ECommerce.Events.CustomerRegistered{} = event) do\n    %__MODULE__{\n      state |\n      id: event.id,\n      email: event.email,\n      name: event.name\n    }\n  end\n  \n  def apply(%__MODULE__{} = state, %ECommerce.Events.EmailConfirmed{} = event) do\n    %__MODULE__{state | status: :active}\n  end\nend\n```\n\nThe aggregate maintains the current state by applying events in sequence. Each event handler updates specific fields based on the event data.","ref":"readme.html#aggregates-and-events-handlers"},{"type":"extras","title":"Projections - README","doc":"Projections define how events should update your read models. AshCommanded generates projection modules that handle specific event types.\n\n```elixir\ncommanded do\n  projections do\n    projection :customer_registered do\n      action(:create)\n      changes(%{\n        status: :pending\n      })\n    end\n\n    projection :email_confirmed do\n      action(:update_by_id)\n      changes(%{\n        status: :active\n      })\n    end\n  end\nend\n```\n\nProjection options:\n- `action` - The Ash action to perform (`:create`, `:update`, `:destroy`, etc.)\n- `changes` - Static map or function that returns the changes to apply\n- `autogenerate?` - Set to false to disable projection generation","ref":"readme.html#projections"},{"type":"extras","title":"Event Handlers - README","doc":"Event handlers define how to respond to domain events with side effects, integrations, notifications, or other operations. Unlike projections which focus on updating read models, event handlers are for operations that don't necessarily affect resource state.\n\n```elixir\ncommanded do\n  event_handlers do\n    # Function-based handler for sending notifications\n    handler :welcome_notification do\n      events [:customer_registered]\n      action fn event, _metadata ->\n        ECommerce.Notifications.send_welcome_email(event.email)\n        :ok\n      end\n    end\n    \n    # Handler with multiple events\n    handler :analytics_tracker do\n      events [:customer_registered, :email_confirmed]\n      action fn event, _metadata ->\n        ECommerce.Analytics.track(event)\n        :ok\n      end\n    end\n    \n    # Handler using an Ash action\n    handler :external_system_sync do\n      events [:customer_registered]\n      action :sync_to_crm\n      idempotent true\n    end\n    \n    # PubSub broadcasting handler\n    handler :event_broadcaster do\n      events [:customer_registered, :email_confirmed]\n      publish_to \"customer_events\"\n    end\n  end\nend\n```\n\nEvent handler options:\n- `events` - List of event names this handler will respond to\n- `action` - Action to perform when handling the event (atom or function)\n- `handler_name` - Override the auto-generated handler module name\n- `publish_to` - PubSub topic(s) to publish the event to\n- `idempotent` - Whether the handler is idempotent (default: false)\n- `autogenerate?` - Set to false to disable handler generation\n\nGenerated event handler modules handle the specified events and execute the defined actions or functions:\n\n```elixir\ndefmodule ECommerce.EventHandlers.CustomerWelcomeNotificationHandler do\n  use Commanded.Event.Handler,\n    application: ECommerce.CommandedApplication,\n    name: \"ECommerce.EventHandlers.CustomerWelcomeNotificationHandler\"\n  \n  def handle(%ECommerce.Events.CustomerRegistered{} = event, _metadata) do\n    ECommerce.Notifications.send_welcome_email(event.email)\n    :ok\n  end\nend\n```","ref":"readme.html#event-handlers"},{"type":"extras","title":"Projectors - README","doc":"Projectors are Commanded event handlers that listen for domain events and update read models. AshCommanded automatically generates projector modules using the `GenerateProjectorModules` transformer. These projectors:\n\n1. Subscribe to specific event types defined in your resource\n2. Process events using the Commanded event handling system\n3. Apply changes to your resources via Ash actions (create, update, destroy)\n\nFor example, a generated projector might look like:\n\n```elixir\ndefmodule ECommerce.Projectors.CustomerProjector do\n  use Commanded.Projections.Ecto, name: \"ECommerce.Projectors.CustomerProjector\"\n\n  project(%ECommerce.Events.CustomerRegistered{} = event, _metadata, fn _context ->\n    Ash.Changeset.new(ECommerce.Customer, event)\n    |> Ash.Changeset.for_action(:create, %{status: :pending})\n    |> Ash.create()\n  end)\n  \n  project(%ECommerce.Events.EmailConfirmed{} = event, _metadata, fn _context ->\n    Ash.Changeset.new(ECommerce.Customer, %{id: event.id})\n    |> Ash.Changeset.for_action(:update, %{status: :active})\n    |> Ash.update()\n  end)\n  \n  # Functions to apply different action types\n  defp apply_action_fn(:create), do: &Ash.create/1\n  defp apply_action_fn(:update), do: &Ash.update/1\n  defp apply_action_fn(:destroy), do: &Ash.destroy/1\nend\n```\n\nYou can customize the projector name with the `projector_name` option or disable automatic generation with `autogenerate?: false`.","ref":"readme.html#projectors"},{"type":"extras","title":"Router Usage - README","doc":"The generated routers allow dispatching commands to their appropriate handlers:\n\n```elixir\n# Dispatch a command through the main router\ncommand = %ECommerce.Commands.RegisterCustomer{id: \"123\", email: \"customer@example.com\", name: \"John Doe\"}\nAshCommanded.Router.dispatch(command)\n```","ref":"readme.html#router-usage"},{"type":"extras","title":"Commanded Application - README","doc":"The `application` section in the DSL allows configuring a Commanded application at the domain level:\n\n```elixir\ndefmodule ECommerce.Store do\n  use Ash.Domain, extensions: [AshCommanded.Commanded.Dsl]\n\n  resources do\n    resource ECommerce.Product\n    resource ECommerce.Customer\n    resource ECommerce.Order\n  end\n\n  commanded do\n    application do\n      otp_app :ecommerce\n      event_store Commanded.EventStore.Adapters.EventStore\n      include_supervisor? true\n    end\n  end\nend\n```\n\nThis generates a Commanded application module that:\n- Configures the event store and other Commanded settings\n- Includes the domain router\n- Provides a supervisor for all projectors\n- Can be added to your application's supervision tree","ref":"readme.html#commanded-application"},{"type":"extras","title":"Where are the Process Managers? - README","doc":"Process Managers in Commanded are responsible for coordinating one or more aggregates. They handle events and dispatch commands in response. This is very business logic specific and would be rather difficult to generate appropriately. It is suggested to write your Process Managers using [Reactor](https://hexdocs.pm/reactor/readme.html) instead, which is a library specifically designed for workflow orchestration in Elixir and works well with Commanded's event-driven architecture.","ref":"readme.html#where-are-the-process-managers"},{"type":"extras","title":"LICENSE","doc":"MIT License\n\nCopyright (c) 2025 Accountex\n\nPermission is hereby granted, free of charge, to any person obtaining a copy\nof this software and associated documentation files (the \"Software\"), to deal\nin the Software without restriction, including without limitation the rights\nto use, copy, modify, merge, publish, distribute, sublicense, and/or sell\ncopies of the Software, and to permit persons to whom the Software is\nfurnished to do so, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in all\ncopies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\nIMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\nFITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\nAUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\nLIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,\nOUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE\nSOFTWARE.","ref":"license.html"},{"type":"extras","title":"Commands","doc":"# Commands\n\nCommands in AshCommanded represent intentions to change the state of your application. They are the input side of the [CQRS pattern](https://martinfowler.com/bliki/CQRS.html). In Commanded, [commands](https://hexdocs.pm/commanded/commands.html) are the primary way to make changes to your domain.","ref":"commands.html"},{"type":"extras","title":"Defining Commands - Commands","doc":"Commands are defined in the `commanded` DSL extension for Ash resources:\n\n```elixir\ndefmodule ECommerce.Customer do\n  use Ash.Resource,\n    extensions: [AshCommanded.Commanded.Dsl]\n\n  attributes do\n    uuid_primary_key :id\n    attribute :email, :string\n    attribute :name, :string\n    attribute :status, :string\n  end\n\n  commanded do\n    commands do\n      command :register_customer do\n        fields([:id, :email, :name])\n        identity_field(:id)\n      end\n\n      command :update_status do\n        fields([:id, :status])\n        # Enable transaction support\n        in_transaction? true\n        repo ECommerce.Repo\n      end\n      \n      command :deactivate_customer do\n        fields([:id])\n        identity_field(:id)\n        \n        # Use block syntax for transaction options\n        transaction do\n          enabled? true\n          repo ECommerce.Repo\n          timeout 5000 \n          isolation_level :read_committed\n        end\n      end\n    end\n  end\nend\n```","ref":"commands.html#defining-commands"},{"type":"extras","title":"Command Options - Commands","doc":"Each command can have the following options:\n\n- `fields`: List of fields the command accepts\n- `identity_field`: Field used to identify the aggregate instance\n- `autogenerate_handler?`: Whether to generate a command handler (default: true)\n- `handler_name`: Custom name for the handler function (default: :handle)\n- `action`: Ash action to invoke when handling the command (defaults to command name)\n- `command_name`: Override the generated command module name\n- `in_transaction?`: Whether to execute the command in a transaction (boolean)\n- `repo`: The Ecto repository to use for transactions (atom)\n- `transaction_timeout`: The transaction timeout in milliseconds (number)\n- `transaction_isolation_level`: The transaction isolation level (atom)\n\nFor detailed information about transaction support, see the [Transactions](transactions.html) documentation.","ref":"commands.html#command-options"},{"type":"extras","title":"Generated Command Modules - Commands","doc":"For each command, AshCommanded generates a command module:\n\n```elixir\ndefmodule ECommerce.Commands.RegisterCustomer do\n  @moduledoc \"\"\"\n  Command for registering a new customer\n  \"\"\"\n\n  @type t :: %__MODULE__{\n    id: String.t(),\n    email: String.t(),\n    name: String.t()\n  }\n\n  defstruct [:id, :email, :name]\nend\n```","ref":"commands.html#generated-command-modules"},{"type":"extras","title":"Command Handlers - Commands","doc":"AshCommanded automatically generates [command handlers](https://hexdocs.pm/commanded/Commanded.Commands.Handler.html) that invoke the corresponding Ash actions:\n\n```elixir\ndefmodule AshCommanded.Commanded.CommandHandlers.CustomerHandler do\n  @behaviour Commanded.Commands.Handler\n\n  def handle(%ECommerce.Commands.RegisterCustomer{} = cmd, _metadata) do\n    Ash.run_action(ECommerce.Customer, :register_customer, Map.from_struct(cmd))\n  end\n\n  def handle(%ECommerce.Commands.UpdateStatus{} = cmd, _metadata) do\n    Ash.run_action(ECommerce.Customer, :update_status, Map.from_struct(cmd))\n  end\nend\n```\n\nCommand handlers in Commanded are responsible for validating, authorizing, and processing commands. See the [Commanded documentation on command handlers](https://hexdocs.pm/commanded/commands.html#handling-commands) for more details.","ref":"commands.html#command-handlers"},{"type":"extras","title":"Events","doc":"# Events\n\nEvents in AshCommanded represent facts that have occurred in your system. They are the source of truth for the state of your aggregates in the [event sourcing pattern](https://martinfowler.com/eaaDev/EventSourcing.html). In Commanded, [events](https://hexdocs.pm/commanded/events.html) are immutable facts that describe something that has happened in your domain.","ref":"events.html"},{"type":"extras","title":"Defining Events - Events","doc":"Events are defined in the `commanded` DSL extension for Ash resources:\n\n```elixir\ndefmodule ECommerce.Customer do\n  use Ash.Resource,\n    extensions: [AshCommanded.Commanded.Dsl]\n\n  attributes do\n    uuid_primary_key :id\n    attribute :email, :string\n    attribute :name, :string\n    attribute :status, :string\n  end\n\n  commanded do\n    events do\n      event :customer_registered do\n        fields([:id, :email, :name])\n      end\n\n      event :customer_status_updated do\n        fields([:id, :status])\n      end\n    end\n  end\nend\n```","ref":"events.html#defining-events"},{"type":"extras","title":"Event Options - Events","doc":"Each event can have the following options:\n\n- `fields`: List of fields the event contains\n- `event_name`: Override the generated event module name","ref":"events.html#event-options"},{"type":"extras","title":"Generated Event Modules - Events","doc":"For each event, AshCommanded generates an event module:\n\n```elixir\ndefmodule ECommerce.Events.CustomerRegistered do\n  @moduledoc \"\"\"\n  Event emitted when a customer is registered\n  \"\"\"\n\n  @type t :: %__MODULE__{\n    id: String.t(),\n    email: String.t(),\n    name: String.t()\n  }\n\n  defstruct [:id, :email, :name]\nend\n```","ref":"events.html#generated-event-modules"},{"type":"extras","title":"Event Handling - Events","doc":"Events are handled in several ways in AshCommanded:","ref":"events.html#event-handling"},{"type":"extras","title":"Aggregate State Updates - Events","doc":"In the [aggregate module](https://hexdocs.pm/commanded/aggregates.html), AshCommanded generates `apply/2` functions for each event to update the aggregate state. This follows the Commanded pattern for [event-sourced aggregates](https://hexdocs.pm/commanded/aggregates.html#event-sourced-aggregates):\n\n```elixir\ndefmodule ECommerce.CustomerAggregate do\n  defstruct [:id, :email, :name, :status]\n\n  def apply(%__MODULE__{} = state, %ECommerce.Events.CustomerRegistered{} = event) do\n    %__MODULE__{\n      state |\n      id: event.id,\n      email: event.email,\n      name: event.name\n    }\n  end\n\n  def apply(%__MODULE__{} = state, %ECommerce.Events.CustomerStatusUpdated{} = event) do\n    %__MODULE__{\n      state |\n      status: event.status\n    }\n  end\nend\n```","ref":"events.html#aggregate-state-updates"},{"type":"extras","title":"Projections - Events","doc":"Projections transform events into resource updates, allowing you to maintain read models. This aligns with the Commanded concept of [read model projections](https://hexdocs.pm/commanded/read-model-projections.html). See the [Projections](projections.html) documentation for details.","ref":"events.html#projections"},{"type":"extras","title":"Event Handlers - Events","doc":"Event handlers allow you to respond to events with side effects like sending notifications or integrating with external systems. These are implemented as [Commanded event handlers](https://hexdocs.pm/commanded/Commanded.Event.Handler.html). See the [Event Handlers](event_handlers.html) documentation for more information.","ref":"events.html#event-handlers"},{"type":"extras","title":"Choosing Between Projections and Event Handlers - Events","doc":"When deciding how to handle events, consider the following:\n\n- Use **projections** when you need to update your resource state or read models based on events\n- Use **event handlers** when you need to perform side effects like sending notifications, integrating with external systems, or publishing events to other systems\n\nYou can use both together to create a complete system that maintains consistent read models while also performing necessary side effects.","ref":"events.html#choosing-between-projections-and-event-handlers"},{"type":"extras","title":"Projections","doc":"# Projections\n\nProjections in AshCommanded define how events affect the state of your resources. They are the read model update mechanism in the [CQRS pattern](https://martinfowler.com/bliki/CQRS.html). In Commanded, [projections](https://hexdocs.pm/commanded/read-model-projections.html) transform domain events into a read-optimized format.","ref":"projections.html"},{"type":"extras","title":"Defining Projections - Projections","doc":"Projections are defined in the `commanded` DSL extension for Ash resources:\n\n```elixir\ndefmodule ECommerce.Customer do\n  use Ash.Resource,\n    extensions: [AshCommanded.Commanded.Dsl]\n\n  attributes do\n    uuid_primary_key :id\n    attribute :email, :string\n    attribute :name, :string\n    attribute :status, :string\n  end\n\n  commanded do\n    events do\n      event :customer_registered do\n        fields([:id, :email, :name])\n      end\n\n      event :customer_status_updated do\n        fields([:id, :status])\n      end\n    end\n\n    projections do\n      projection :customer_registered do\n        action(:create)\n        changes(%{\n          status: \"pending\"\n        })\n      end\n\n      projection :customer_status_updated do\n        action(:update_by_id)\n        changes(&Map.take(&1, [:status]))\n      end\n    end\n  end\nend\n```","ref":"projections.html#defining-projections"},{"type":"extras","title":"Projection Options - Projections","doc":"Each projection can have the following options:\n\n- `action`: The Ash action to use when handling the event (`:create`, `:update`, etc.)\n- `changes`: Static map or function to determine the changes to apply\n- `projector_name`: Override the generated projector module name","ref":"projections.html#projection-options"},{"type":"extras","title":"Generated Projector Modules - Projections","doc":"AshCommanded generates a projector module for each resource with projections. This projector is a [Commanded event handler](https://hexdocs.pm/commanded/Commanded.Event.Handler.html) that subscribes to events and updates the read model. It leverages [Commanded's projection support](https://hexdocs.pm/commanded/read-model-projections.html) to handle the events efficiently.\n\n```elixir\ndefmodule ECommerce.Projectors.CustomerProjector do\n  @moduledoc \"\"\"\n  Projector for Customer-related events\n  \"\"\"\n\n  use Commanded.Projections.Ecto, \n    name: \"ECommerce.Projectors.CustomerProjector\"\n  # See Commanded's Ecto projections: https://hexdocs.pm/commanded_ecto_projections/\n\n  # Each projection gets a project/3 function\n  project(%ECommerce.Events.CustomerRegistered{} = event, _metadata, fn _context ->\n    Ash.Changeset.new(ECommerce.Customer, event)\n    |> Ash.Changeset.for_action(:create, %{\n      id: event.id,\n      email: event.email, \n      name: event.name,\n      status: \"pending\"\n    })\n    |> Ash.create()\n  end)\n  \n  project(%ECommerce.Events.CustomerStatusUpdated{} = event, _metadata, fn _context ->\n    Ash.Changeset.new(ECommerce.Customer, event)\n    |> Ash.Changeset.for_action(:update_by_id, %{\n      status: event.status\n    })\n    |> Ash.update()\n  end)\n  \n  # Helper functions for applying different action types\n  defp apply_action_fn(:create), do: &Ash.create/1\n  defp apply_action_fn(:update), do: &Ash.update/1\n  defp apply_action_fn(:destroy), do: &Ash.destroy/1\nend\n```","ref":"projections.html#generated-projector-modules"},{"type":"extras","title":"Registering Projectors with Commanded - Projections","doc":"The generated projectors need to be registered with your Commanded application to start processing events. Add them to your application's supervisor tree:\n\n```elixir\ndefmodule ECommerce.Application do\n  use Application\n\n  def start(_type, _args) do\n    children = [\n      # ...other children\n      \n      # Start your projectors\n      ECommerce.Projectors.CustomerProjector\n    ]\n\n    opts = [strategy: :one_for_one, name: ECommerce.Supervisor]\n    Supervisor.start_link(children, opts)\n  end\nend\n```\n\nFor more complex projection needs, you can customize the `project/3` function directly in your own projector module:\n\n```elixir\ndefmodule ECommerce.CustomProjector do\n  use Commanded.Projections.Ecto, name: \"ECommerce.CustomProjector\"\n\n  project(%ECommerce.Events.CustomerRegistered{} = event, metadata, fn _context ->\n    # Access event and metadata\n    customer_id = event.id\n    timestamp = metadata.created_at\n    \n    # Custom projection logic\n    Ash.Changeset.new(ECommerce.Customer)\n    |> Ash.Changeset.for_action(:create, %{\n      id: customer_id,\n      email: event.email,\n      name: event.name,\n      status: \"pending\",\n      registered_at: timestamp\n    })\n    |> Ash.create()\n  end)\nend\n```\n\nEach resource typically gets one projector module containing handlers for all projections. The projector:\n\n1. Automatically registers with Commanded to receive events\n2. Processes events in real-time as they occur\n3. Updates the read models (resources) using Ash actions\n4. Maintains consistency between write and read models","ref":"projections.html#registering-projectors-with-commanded"},{"type":"extras","title":"Customizing Projector Generation - Projections","doc":"You can customize the projector generation with several options:\n\n```elixir\nprojection :user_registered do\n  # Specify the action to perform (create, update, destroy)\n  action(:create)\n  \n  # Changes to apply\n  changes(%{status: \"active\"})\n  \n  # Custom projector module name\n  projector_name(:CustomUserProjector)\n  \n  # Disable projector generation for this projection\n  autogenerate?(false)\nend\n```\n\nFor a resource with many projectors, you can also set the projector namespace:\n\n```elixir\ndefmodule ECommerce.Customer do\n  use Ash.Resource,\n    extensions: [AshCommanded.Commanded.Dsl]\n    \n  # Set custom namespace for all projectors\n  @projector_namespace ECommerce.CustomProjectors\n  \n  # Resource definition...\nend\n```","ref":"projections.html#customizing-projector-generation"},{"type":"extras","title":"Change Types - Projections","doc":"The `changes` option supports two formats:\n\n1. Static map - simple key-value changes:\n```elixir\nchanges(%{\n  status: \"active\"\n})\n```\n\n2. Function - dynamic changes based on the event:\n```elixir\nchanges(fn event ->\n  %{\n    status: event.status,\n    last_updated: DateTime.utc_now()\n  }\nend)\n```\n\nOr more concisely with capture syntax:\n```elixir\nchanges(&Map.take(&1, [:status]))\n```","ref":"projections.html#change-types"},{"type":"extras","title":"Event Handlers","doc":"# Event Handlers\n\nEvent handlers define how to respond to domain events with side effects, integrations, notifications, or other operations that don't necessarily update the resource state directly. Unlike projections which are focused on updating read models, event handlers allow you to execute arbitrary code in response to events. In Commanded, [event handlers](https://hexdocs.pm/commanded/Commanded.Event.Handler.html) provide a flexible way to react to domain events.","ref":"event_handlers.html"},{"type":"extras","title":"Introduction - Event Handlers","doc":"In [CQRS](https://martinfowler.com/bliki/CQRS.html) and [Event Sourcing](https://martinfowler.com/eaaDev/EventSourcing.html) systems, events represent facts that have occurred in your domain. While projections use these events to maintain read models, there are often other operations you want to perform when events occur. This follows Commanded's [event handling approach](https://hexdocs.pm/commanded/events.html#handling-events):\n\n- Sending notifications (emails, SMS, push notifications)\n- Integrating with external systems\n- Publishing events to message brokers\n- Logging or analytics tracking\n- Triggering downstream processes\n\nAshCommanded provides the `event_handlers` section to define these behaviors declaratively.","ref":"event_handlers.html#introduction"},{"type":"extras","title":"Usage - Event Handlers","doc":"In the e-commerce example, we can define event handlers to send notifications when orders are placed or shipments are created:\n\n```elixir\ndefmodule MyApp.Order do\n  use Ash.Resource,\n    extensions: [AshCommanded.Commanded.Dsl]\n\n  attributes do\n    uuid_primary_key :id\n    attribute :customer_id, :string\n    attribute :total_amount, :decimal\n    attribute :status, :atom, default: :pending\n  end\n\n  commanded do\n    events do\n      event :order_placed do\n        fields [:id, :customer_id, :total_amount]\n      end\n      \n      event :order_shipped do\n        fields [:id, :tracking_number]\n      end\n    end\n    \n    event_handlers do\n      # Send email notification when order is placed\n      handler :order_confirmation do\n        events [:order_placed]\n        action fn event, _metadata ->\n          MyApp.Notifications.send_order_confirmation(\n            event.customer_id,\n            event.id,\n            event.total_amount\n          )\n          :ok\n        end\n      end\n      \n      # Integrate with shipping provider when order is shipped\n      handler :shipping_notification do\n        events [:order_shipped]\n        action fn event, _metadata ->\n          MyApp.ShippingProvider.notify_shipment(event.id, event.tracking_number)\n          :ok\n        end\n      end\n      \n      # Publish events to a PubSub topic for other systems\n      handler :event_broadcaster do\n        events [:order_placed, :order_shipped]\n        publish_to \"order_events\"\n      end\n    end\n  end\nend\n```","ref":"event_handlers.html#usage"},{"type":"extras","title":"Handler Options - Event Handlers","doc":"Event handlers are highly configurable:\n\n| Option | Type | Description |\n|--------|------|-------------|\n| `events` | `list(atom)` | **Required**. List of event names this handler will respond to. |\n| `action` | `atom` or `quoted` | Action to perform when handling events. Can be an Ash action name or a quoted function. |\n| `handler_name` | `atom` | Override the auto-generated handler module name. |\n| `publish_to` | `atom` or `string` or `list` | Specify PubSub topic(s) to publish the event to. |\n| `idempotent` | `boolean` | Whether the handler is idempotent (safe to process the same event multiple times). Default: `false` |\n| `autogenerate?` | `boolean` | Whether to autogenerate a handler module. Default: `true` |","ref":"event_handlers.html#handler-options"},{"type":"extras","title":"Action Types - Event Handlers","doc":"There are three ways to specify the action a handler should take:","ref":"event_handlers.html#action-types"},{"type":"extras","title":"1. Function-Based Handlers - Event Handlers","doc":"Using an anonymous function gives you full control over event handling:\n\n```elixir\nhandler :notification_handler do\n  events [:order_placed]\n  action fn event, metadata ->\n    # Access event data\n    order_id = event.id\n    customer_id = event.customer_id\n    \n    # Access metadata\n    correlation_id = metadata.correlation_id\n    \n    # Call your notification service\n    MyApp.Notifications.send_email(customer_id, order_id, correlation_id)\n    \n    # Return :ok to indicate success\n    :ok\n  end\nend\n```","ref":"event_handlers.html#1-function-based-handlers"},{"type":"extras","title":"2. Ash Action Handlers - Event Handlers","doc":"You can reference an Ash action directly by its name:\n\n```elixir\nhandler :sync_to_crm do\n  events [:customer_registered]\n  action :create_crm_contact\nend\n```\n\nThis assumes your resource has a corresponding action defined.","ref":"event_handlers.html#2-ash-action-handlers"},{"type":"extras","title":"3. No-Op Handlers (Publishing Only) - Event Handlers","doc":"If you just want to publish events to a topic without additional processing:\n\n```elixir\nhandler :event_publisher do\n  events [:order_placed, :order_shipped, :order_cancelled]\n  publish_to \"order_events\"\nend\n```","ref":"event_handlers.html#3-no-op-handlers-publishing-only"},{"type":"extras","title":"Differences from Projections - Event Handlers","doc":"It's important to understand the distinction between projections and event handlers:\n\n| Projections | Event Handlers |\n|-------------|---------------|\n| Focus on updating read models | Focus on side effects and integrations |\n| Map events to resource changes | Execute arbitrary code in response to events |\n| Always tied to create/update/delete operations | Can perform any operation, not just data changes |\n| Primary purpose is maintaining consistent data views | Primary purpose is integration and notifications |\n| Limited to data operations within your application | Can interact with external systems and services |","ref":"event_handlers.html#differences-from-projections"},{"type":"extras","title":"Generated Modules - Event Handlers","doc":"AshCommanded automatically generates event handler modules based on your DSL configuration. These follow the [Commanded.Event.Handler](https://hexdocs.pm/commanded/Commanded.Event.Handler.html) pattern:\n\n```elixir\ndefmodule MyApp.EventHandlers.OrderOrderConfirmationHandler do\n  @moduledoc \"General purpose event handler for Order events (OrderConfirmation)\"\n\n  use Commanded.Event.Handler,\n    application: MyApp.CommandedApplication,\n    name: \"MyApp.EventHandlers.OrderOrderConfirmationHandler\"\n\n  def handle(%MyApp.Events.OrderPlaced{} = event, metadata) do\n    # Execute the function defined in the DSL\n    send_order_confirmation(event.customer_id, event.id, event.total_amount)\n    :ok\n  end\n  \n  # Helper functions for action execution and response handling\n  # ...\nend\n```","ref":"event_handlers.html#generated-modules"},{"type":"extras","title":"Error Handling - Event Handlers","doc":"Handler functions should return `:ok` on success or `{:error, reason}` on failure. When a handler fails:\n\n1. The error is logged\n2. The event might be retried, depending on your Commanded configuration\n3. The failure is propagated to your application's error handling mechanism\n\nFor critical handlers, you may want to implement retry logic or fallback mechanisms.","ref":"event_handlers.html#error-handling"},{"type":"extras","title":"Best Practices - Event Handlers","doc":"1. **Keep Handlers Focused**: Each handler should have a single responsibility.\n2. **Make Handlers Idempotent**: Since events might be processed multiple times, design handlers to be idempotent.\n3. **Separate Business Logic**: Put complex business logic in dedicated modules and have handlers call these functions.\n4. **Handle Errors Gracefully**: Include proper error handling to ensure your system remains resilient.\n5. **Consider Failure Domains**: Group handlers by failure domain - if one handler's failure shouldn't affect others, put them in different handler modules.\n6. **Be Mindful of Performance**: Long-running handlers can block event processing; consider using background jobs for heavy operations.","ref":"event_handlers.html#best-practices"},{"type":"extras","title":"Router Generation","doc":"# Router Generation\n\nAshCommanded automatically generates [Commanded Routers](https://hexdocs.pm/commanded/commands.html) for dispatching commands to the appropriate handlers and aggregates based on your domain structure.","ref":"routers.html"},{"type":"extras","title":"Domain-specific Routers - Router Generation","doc":"For each Ash.Domain containing resources with the AshCommanded.Commanded.Dsl extension, a domain-specific router is generated:\n\n```elixir\ndefmodule ECommerce.Store.Router do\n  use Commanded.Commands.Router\n\n  # For each resource in the domain\n  identify ECommerce.ProductAggregate, by: :id\n  dispatch [ECommerce.Commands.CreateProduct, ECommerce.Commands.UpdateProduct], to: ECommerce.ProductAggregate\n  \n  identify ECommerce.CustomerAggregate, by: :id\n  dispatch [ECommerce.Commands.RegisterCustomer, ECommerce.Commands.UpdateStatus], to: ECommerce.CustomerAggregate\nend\n```\n\nThese domain routers handle command routing within a specific domain context.","ref":"routers.html#domain-specific-routers"},{"type":"extras","title":"Main Application Router - Router Generation","doc":"AshCommanded also generates a main application router at `AshCommanded.Router`. The behavior of this router depends on the domain structure:","ref":"routers.html#main-application-router"},{"type":"extras","title":"Single Domain Setup - Router Generation","doc":"For applications with a single domain, the main router directly routes commands to aggregates:\n\n```elixir\ndefmodule AshCommanded.Router do\n  use Commanded.Commands.Router\n\n  # Direct command routing for all resources\n  identify ECommerce.ProductAggregate, by: :id\n  dispatch [ECommerce.Commands.CreateProduct, ECommerce.Commands.UpdateProduct], to: ECommerce.ProductAggregate\n  \n  identify ECommerce.CustomerAggregate, by: :id\n  dispatch [ECommerce.Commands.RegisterCustomer, ECommerce.Commands.UpdateStatus], to: ECommerce.CustomerAggregate\nend\n```","ref":"routers.html#single-domain-setup"},{"type":"extras","title":"Multiple Domain Setup - Router Generation","doc":"For applications with multiple domains, the main router forwards commands to the appropriate domain router:\n\n```elixir\ndefmodule AshCommanded.Router do\n  use Commanded.Commands.Router\n\n  # Forward to domain routers\n  forward ECommerce.Store.Router\n  forward ECommerce.Marketing.Router\n  forward ECommerce.Shipping.Router\nend\n```","ref":"routers.html#multiple-domain-setup"},{"type":"extras","title":"Command Dispatching - Router Generation","doc":"With these routers in place, you can dispatch commands using the [dispatch/1](https://hexdocs.pm/commanded/Commanded.Commands.Router.html#dispatch/1) or [dispatch/2](https://hexdocs.pm/commanded/Commanded.Commands.Router.html#dispatch/2) functions:\n\n```elixir\n# Basic command dispatch\ncommand = %ECommerce.Commands.CreateProduct{id: \"123\", name: \"Premium Widget\", price: 29.99}\nAshCommanded.Router.dispatch(command)\n\n# With options\ncommand = %ECommerce.Commands.CreateProduct{id: \"123\", name: \"Premium Widget\", price: 29.99}\nAshCommanded.Router.dispatch(command, correlation_id: UUID.uuid4(), metadata: %{user_id: \"user-123\"})\n```\n\nThe command will be routed to the appropriate aggregate and handler based on the command type and the resource's identity.","ref":"routers.html#command-dispatching"},{"type":"extras","title":"Identity Resolution - Router Generation","doc":"By default, the router uses the resource's primary identity key or the specified `identity_field` in the command definition.\n\n```elixir\ncommanded do\n  commands do\n    command :create_product do\n      fields([:id, :name, :price])\n      identity_field(:id)  # Explicitly specify the identity field\n    end\n  end\nend\n```\n\nIf no identity field is specified, the router will try to use:\n1. The first field from the resource's primary identity\n2. The `:id` field as a fallback","ref":"routers.html#identity-resolution"},{"type":"extras","title":"Commanded Application Configuration","doc":"# Commanded Application Configuration\n\nThe `application` section allows you to configure the [Commanded application](https://hexdocs.pm/commanded/application.html) that manages command dispatch, event publishing, and process management for your domain. The Commanded application is the core component that orchestrates the event-sourced domain.","ref":"application.html"},{"type":"extras","title":"Basic Usage - Commanded Application Configuration","doc":"The `application` section is defined at the domain level, allowing each domain to have its own Commanded application configuration:\n\n```elixir\ndefmodule ECommerce.Store do\n  use Ash.Domain, extensions: [AshCommanded.Commanded.Dsl]\n\n  resources do\n    resource ECommerce.Product\n    resource ECommerce.Customer\n    resource ECommerce.Order\n  end\n\n  commanded do\n    application do\n      otp_app :ecommerce\n      event_store Commanded.EventStore.Adapters.EventStore\n    end\n  end\nend\n```","ref":"application.html#basic-usage"},{"type":"extras","title":"Configuration Options - Commanded Application Configuration","doc":"The following options can be configured in the `application` section:\n\n| Option               | Type                 | Required | Default | Description                                                               |\n|----------------------|----------------------|----------|---------|---------------------------------------------------------------------------|\n| `otp_app`            | atom                 | Yes      | -       | The OTP application name to use for configuration                         |\n| `event_store`        | atom or keyword list | Yes      | -       | The event store adapter to use (e.g., `Commanded.EventStore.Adapters.EventStore`) |\n| `pubsub`             | atom                 | No       | `:local` | The pubsub adapter to use (`:local` or `:phoenix`)                      |\n| `registry`           | atom                 | No       | `:local` | The registry adapter to use (`:local` or `:global`)                     |\n| `snapshotting`       | keyword list         | No       | `[]`    | Configuration for aggregate snapshotting                                |\n| `include_supervisor?` | boolean             | No       | `false` | Whether to include a supervisor for the application                      |\n| `prefix`             | string               | No       | `nil`   | Application module prefix for generated code                             |","ref":"application.html#configuration-options"},{"type":"extras","title":"Events Stores - Commanded Application Configuration","doc":"AshCommanded supports all [Commanded event store adapters](https://hexdocs.pm/commanded/event-store.html):\n\n* [`Commanded.EventStore.Adapters.EventStore`](https://hexdocs.pm/commanded_eventstore_adapter/) - The default PostgreSQL event store.\n* [`Commanded.EventStore.Adapters.InMemory`](https://hexdocs.pm/commanded/Commanded.EventStore.Adapters.InMemory.html) - An in-memory event store useful for testing.\n\nExample using the default EventStore adapter:\n\n```elixir\ncommanded do\n  application do\n    otp_app :ecommerce\n    event_store Commanded.EventStore.Adapters.EventStore\n  end\nend\n```\n\nExample using the InMemory adapter for testing:\n\n```elixir\ncommanded do\n  application do\n    otp_app :ecommerce\n    event_store Commanded.EventStore.Adapters.InMemory\n  end\nend\n```","ref":"application.html#events-stores"},{"type":"extras","title":"Generated Application Module - Commanded Application Configuration","doc":"The DSL will generate an application module using the configuration. This module extends the [Commanded.Application](https://hexdocs.pm/commanded/Commanded.Application.html) behavior:\n\n```elixir\ndefmodule ECommerce.Store.Application do\n  use Commanded.Application,\n    otp_app: :ecommerce,\n    event_store: Commanded.EventStore.Adapters.EventStore,\n    pubsub: :local,\n    registry: :local,\n    router: ECommerce.Store.Router\n\n  # Supervision for projectors (only included if include_supervisor? is true)\n  def child_spec() do\n    Supervisor.child_spec(\n      {Supervisor, [\n        strategy: :one_for_one,\n        name: Module.concat(__MODULE__, Supervisor)\n      ]},\n      id: Module.concat(__MODULE__, Supervisor)\n    )\n  end\nend\n```","ref":"application.html#generated-application-module"},{"type":"extras","title":"Supervision - Commanded Application Configuration","doc":"By default, Commanded applications are not supervised. To include a supervisor for your application, set `include_supervisor?` to `true`:\n\n```elixir\ncommanded do\n  application do\n    otp_app :ecommerce\n    event_store Commanded.EventStore.Adapters.EventStore\n    include_supervisor? true\n  end\nend\n```\n\nThis will generate a supervisor for your application that you can add to your supervision tree:\n\n```elixir\ndefmodule ECommerce.Application do\n  use Application\n\n  def start(_type, _args) do\n    children = [\n      # ...other children\n      ECommerce.Store.Application.child_spec()\n    ]\n\n    opts = [strategy: :one_for_one, name: ECommerce.Supervisor]\n    Supervisor.start_link(children, opts)\n  end\nend\n```","ref":"application.html#supervision"},{"type":"extras","title":"Multiple Applications - Commanded Application Configuration","doc":"AshCommanded supports defining multiple Commanded applications across different domains. This is useful for separating different bounded contexts in your application:\n\n```elixir\ndefmodule ECommerce.Customers do\n  use Ash.Domain, extensions: [AshCommanded.Commanded.Dsl]\n\n  resources do\n    resource ECommerce.Customer\n  end\n\n  commanded do\n    application do\n      otp_app :ecommerce\n      event_store Commanded.EventStore.Adapters.EventStore, schema: \"customers\"\n    end\n  end\nend\n\ndefmodule ECommerce.Orders do\n  use Ash.Domain, extensions: [AshCommanded.Commanded.Dsl]\n\n  resources do\n    resource ECommerce.Order\n    resource ECommerce.OrderItem\n  end\n\n  commanded do\n    application do\n      otp_app :ecommerce\n      event_store Commanded.EventStore.Adapters.EventStore, schema: \"orders\"\n    end\n  end\nend\n```","ref":"application.html#multiple-applications"},{"type":"extras","title":"Usage - Commanded Application Configuration","doc":"Once the application is configured, you can dispatch commands through the domain's router:\n\n```elixir\ncommand = %ECommerce.Commands.RegisterCustomer{id: \"123\", email: \"customer@example.com\", name: \"John Doe\"}\nECommerce.Customers.Router.dispatch(command)\n```\n\nThe router will automatically route the command to the appropriate aggregate and handle it according to your command handlers.","ref":"application.html#usage"},{"type":"extras","title":"Aggregate Snapshotting","doc":"# Aggregate Snapshotting\n\nAshCommanded supports [aggregate snapshotting](https://hexdocs.pm/commanded/aggregate-snapshotting.html) as a performance optimization for aggregates with many events. Snapshotting allows the system to store the current state of an aggregate at specific points and then only replay events that occurred after the snapshot was taken. This implementation follows the [Commanded snapshotting architecture](https://hexdocs.pm/commanded/aggregate-snapshotting.html).","ref":"snapshotting.html"},{"type":"extras","title":"Why Use Snapshotting? - Aggregate Snapshotting","doc":"In event-sourced systems, aggregates rebuild their state by replaying all events from the beginning of time. This can become a performance issue for aggregates that have accumulated a large number of events over their lifetime.\n\nSnapshotting addresses this by:\n\n1. Capturing the aggregate state at a point in time\n2. Storing this snapshot for future use\n3. Only loading events that occurred after the snapshot was taken\n\nThis significantly improves performance for aggregates with many events.","ref":"snapshotting.html#why-use-snapshotting"},{"type":"extras","title":"Enabling Snapshotting - Aggregate Snapshotting","doc":"Snapshotting is configured at the application level in your domain:\n\n```elixir\ndefmodule MyApp.Domain do\n  use Ash.Domain\n  \n  resources do\n    resource MyApp.User\n  end\n  \n  commanded do\n    application do\n      otp_app :my_app\n      event_store Commanded.EventStore.Adapters.EventStore\n      \n      # Enable snapshotting\n      snapshotting true\n      \n      # Take a snapshot every 100 events (default)\n      snapshot_threshold 100\n      \n      # Snapshot schema version (for future schema evolution)\n      snapshot_version 1\n      \n      # Optional custom snapshot store module\n      # snapshot_store MyApp.CustomSnapshotStore\n    end\n  end\nend\n```","ref":"snapshotting.html#enabling-snapshotting"},{"type":"extras","title":"How Snapshotting Works - Aggregate Snapshotting","doc":"When snapshotting is enabled, the following happens (following Commanded's [snapshot loading process](https://hexdocs.pm/commanded/aggregate-snapshotting.html#how-it-works)):\n\n1. **Command Execution**: When a command is sent to an aggregate, the system first checks if a snapshot exists for that aggregate.\n\n2. **Snapshot Loading**: If a snapshot exists, the aggregate state is restored from the snapshot rather than rebuilding from scratch.\n\n3. **Event Loading**: Only events that occurred after the snapshot's version are loaded and applied to the aggregate.\n\n4. **Snapshot Creation**: After applying events, the system checks if a new snapshot should be taken based on the configured threshold.","ref":"snapshotting.html#how-snapshotting-works"},{"type":"extras","title":"Snapshot Storage - Aggregate Snapshotting","doc":"By default, snapshots are stored in memory using an ETS table, similar to Commanded's [in-memory snapshot store](https://hexdocs.pm/commanded/aggregate-snapshotting.html#snapshot-storage). This is suitable for development but not for production. \n\nIn a production environment, you should implement a custom snapshot store that persists snapshots to a database or other storage system, following Commanded's [snapshot adapter pattern](https://hexdocs.pm/commanded/aggregate-snapshotting.html#custom-snapshot-storage).","ref":"snapshotting.html#snapshot-storage"},{"type":"extras","title":"Custom Snapshot Store - Aggregate Snapshotting","doc":"You can implement a custom snapshot store by creating a module that implements the `AshCommanded.Commanded.SnapshotStore` behaviour, which is based on Commanded's [Snapshot adapter](https://hexdocs.pm/commanded/Commanded.Snapshotting.Adapter.html) interface:\n\n```elixir\ndefmodule MyApp.CustomSnapshotStore do\n  @behaviour AshCommanded.Commanded.SnapshotStore\n  \n  # Get a snapshot by aggregate ID and type\n  @impl true\n  def get_snapshot(source_uuid, source_type) do\n    # Implementation that retrieves from database\n  end\n  \n  # Save a snapshot\n  @impl true\n  def save_snapshot(snapshot) do\n    # Implementation that saves to database\n  end\n  \n  # Delete all snapshots for an aggregate\n  @impl true\n  def delete_snapshots(source_uuid, source_type) do\n    # Implementation that deletes from database\n  end\n  \n  # Initialize the store\n  @impl true\n  def init(config) do\n    # Implementation that initializes the store\n  end\nend\n```\n\nThen configure your domain to use it:\n\n```elixir\ncommanded do\n  application do\n    # ... other settings ...\n    snapshot_store MyApp.CustomSnapshotStore\n  end\nend\n```","ref":"snapshotting.html#custom-snapshot-store"},{"type":"extras","title":"Performance Considerations - Aggregate Snapshotting","doc":"- **Threshold**: The `snapshot_threshold` setting controls how frequently snapshots are taken. A lower value means more snapshots but potentially better performance for frequently accessed aggregates.\n\n- **State Size**: Snapshots store the entire aggregate state, which can be large. Consider the storage requirements when enabling snapshotting.\n\n- **Asynchronous Snapshots**: Snapshots are created asynchronously to avoid impacting command execution performance.","ref":"snapshotting.html#performance-considerations"},{"type":"extras","title":"Snapshot Format Evolution - Aggregate Snapshotting","doc":"The `snapshot_version` setting allows for future schema evolution. If you need to change the structure of your aggregates, you can increment this version number and add migration logic in your aggregate's `restore_from_snapshot/1` function.","ref":"snapshotting.html#snapshot-format-evolution"},{"type":"extras","title":"Example - Aggregate Snapshotting","doc":"```elixir\ndefmodule MyApp.User do\n  use Ash.Resource,\n    extensions: [AshCommanded.Commanded.Dsl]\n  \n  attributes do\n    uuid_primary_key :id\n    attribute :email, :string\n    attribute :name, :string\n    attribute :status, :string\n  end\n  \n  commanded do\n    commands do\n      command :register_user do\n        fields [:id, :email, :name]\n        identity_field :id\n      end\n      \n      command :change_email do\n        fields [:id, :email]\n        identity_field :id\n      end\n    end\n    \n    events do\n      event :user_registered do\n        fields [:id, :email, :name]\n      end\n      \n      event :email_changed do\n        fields [:id, :email]\n      end\n    end\n  end\nend\n```\n\nWith snapshotting enabled, after registering many users and changing emails multiple times, the system will automatically create snapshots when needed, improving command execution performance.","ref":"snapshotting.html#example"},{"type":"extras","title":"Command Middleware","doc":"# Command Middleware\n\nAshCommanded provides a middleware system that allows you to intercept and modify commands before they are dispatched, as well as process the results afterward. This concept is similar to [Commanded's middleware](https://hexdocs.pm/commanded/commands.html#command-handlers) functionality but adapted for the Ash framework context. This is useful for implementing cross-cutting concerns such as:\n\n- Validation\n- Logging\n- Authentication and authorization\n- Rate limiting\n- Metrics and monitoring\n- Auditing\n- Parameter transformation\n- Error handling and standardization","ref":"middleware.html"},{"type":"extras","title":"Middleware Concepts - Command Middleware","doc":"1. **Middleware Chain**: Middleware is applied in sequence, with each middleware having the opportunity to:\n   - Inspect and modify the command\n   - Provide additional context\n   - Short-circuit the chain by returning an error\n   - Process the result after command execution\n\n2. **Middleware Levels**: You can specify middleware at three different levels:\n   - Global (application-wide)\n   - Resource-level (applies to all commands in a resource)\n   - Command-level (applies to a specific command)\n\n3. **Middleware Context**: Middleware can pass information to each other using a context map.","ref":"middleware.html#middleware-concepts"},{"type":"extras","title":"Using Built-in Middleware - Command Middleware","doc":"AshCommanded includes several built-in middleware components that you can use immediately:","ref":"middleware.html#using-built-in-middleware"},{"type":"extras","title":"LoggingMiddleware - Command Middleware","doc":"Logs information about commands and their results:\n\n```elixir\ndefmodule MyApp.User do\n  use Ash.Resource,\n    extensions: [AshCommanded.Commanded.Dsl]\n\n  commanded do\n    commands do\n      # Resource-level middleware - applies to all commands\n      middleware [AshCommanded.Commanded.Middleware.LoggingMiddleware]\n      \n      # Command-specific middleware with options\n      command :register_user do\n        fields [:id, :name, :email]\n        middleware [\n          {AshCommanded.Commanded.Middleware.LoggingMiddleware, level: :debug}\n        ]\n      end\n    end\n  end\nend\n```\n\nConfiguration options:\n- `level`: The log level to use (`:debug`, `:info`, `:warn`, `:error`). Defaults to `:info`.\n- `error_level`: The log level to use for errors. Defaults to `:error`.","ref":"middleware.html#loggingmiddleware"},{"type":"extras","title":"ValidationMiddleware - Command Middleware","doc":"Validates commands before they are dispatched:\n\n```elixir\ndefmodule MyApp.User do\n  use Ash.Resource,\n    extensions: [AshCommanded.Commanded.Dsl]\n\n  commanded do\n    commands do\n      command :register_user do\n        fields [:id, :name, :email, :age]\n        \n        middleware [\n          {AshCommanded.Commanded.Middleware.ValidationMiddleware,\n            # Require specific fields\n            required: [:name, :email],\n            \n            # Validate field formats\n            format: [\n              email: ~r/@/,\n              name: ~r/^[a-zA-Z\\s]+$/\n            ],\n            \n            # Custom validation function\n            validate: fn command ->\n              if command.age && command.age < 18 do\n                {:error, \"User must be at least 18 years old\"}\n              else\n                :ok\n              end\n            end\n          }\n        ]\n      end\n    end\n  end\nend\n```\n\nConfiguration options:\n- `required`: List of fields that must be present and non-nil\n- `format`: Map of field names to regular expressions for format validation\n- `validate`: Custom validation function that takes a command and returns `:ok` or `{:error, reason}`\n- `validations`: List of validations to apply (combines the above options)","ref":"middleware.html#validationmiddleware"},{"type":"extras","title":"Creating Custom Middleware - Command Middleware","doc":"You can create your own middleware by implementing the `AshCommanded.Commanded.Middleware.CommandMiddleware` behaviour or by using `AshCommanded.Commanded.Middleware.BaseMiddleware` as a starting point. This is conceptually similar to how you would [implement middleware in Commanded](https://hexdocs.pm/commanded/middleware.html):\n\n```elixir\ndefmodule MyApp.AuthorizationMiddleware do\n  use AshCommanded.Commanded.Middleware.BaseMiddleware\n  \n  @impl true\n  def before_dispatch(command, context, next) do\n    # Extract user from context\n    user = Map.get(context, :current_user)\n    \n    # Check authorization\n    if authorized?(command, user) do\n      # Continue with the command\n      next.(command, context)\n    else\n      # Deny the command\n      {:error, :unauthorized}\n    end\n  end\n  \n  @impl true\n  def after_dispatch({:ok, result} = success, _command, _context) do\n    # Process successful result\n    success\n  end\n  \n  def after_dispatch({:error, reason} = error, _command, _context) do\n    # Process error result\n    error\n  end\n  \n  # Helper function for authorization check\n  defp authorized?(command, user) do\n    # Your authorization logic here...\n    true\n  end\nend\n```","ref":"middleware.html#creating-custom-middleware"},{"type":"extras","title":"Global Middleware Configuration - Command Middleware","doc":"You can specify global middleware that applies to all commands in your application using the application configuration:\n\n```elixir\n# In config/config.exs\nconfig :ash_commanded, :global_middleware, [\n  AshCommanded.Commanded.Middleware.LoggingMiddleware,\n  {MyApp.AuthorizationMiddleware, role_check: true}\n]\n```","ref":"middleware.html#global-middleware-configuration"},{"type":"extras","title":"Middleware Context - Command Middleware","doc":"Middleware can use the context to share information:\n\n```elixir\ndefmodule MyApp.TimingMiddleware do\n  use AshCommanded.Commanded.Middleware.BaseMiddleware\n  \n  @impl true\n  def before_dispatch(command, context, next) do\n    # Add timestamp to context\n    context_with_time = Map.put(context, :start_time, System.monotonic_time())\n    \n    # Call next middleware with updated context\n    next.(command, context_with_time)\n  end\n  \n  @impl true\n  def after_dispatch(result, _command, context) do\n    # Calculate elapsed time\n    start_time = Map.get(context, :start_time)\n    elapsed = System.convert_time_unit(\n      System.monotonic_time() - start_time,\n      :native,\n      :millisecond\n    )\n    \n    # Log timing information\n    Logger.info(\"Command processed in #{elapsed}ms\")\n    \n    result\n  end\nend\n```","ref":"middleware.html#middleware-context"},{"type":"extras","title":"Middleware Order - Command Middleware","doc":"Middleware is applied in the following order:\n\n1. Global middleware (from application config)\n2. Resource-level middleware\n3. Command-specific middleware\n\nWithin each level, middleware is applied in the order it's defined.","ref":"middleware.html#middleware-order"},{"type":"extras","title":"Best Practices - Command Middleware","doc":"1. **Keep Middleware Focused**: Each middleware should have a single responsibility\n2. **Use Context for Communication**: Share data between middleware using the context map\n3. **Handle Errors Gracefully**: Make sure to handle errors appropriately in `after_dispatch`\n4. **Log Debugging Information**: Include helpful log messages for debugging\n5. **Test Middleware in Isolation**: Write unit tests for your middleware\n6. **Consider Performance**: Keep middleware lightweight to avoid impacting command processing time","ref":"middleware.html#best-practices"},{"type":"extras","title":"Parameter Transformation and Validation","doc":"# Parameter Transformation and Validation\n\nAshCommanded provides powerful parameter transformation and validation capabilities for command handling, allowing you to:\n\n1. Transform command fields before they're sent to actions\n2. Validate command parameters to ensure they meet specific criteria\n3. Add computed fields based on input parameters\n4. Apply complex, multi-step transformations to command data\n\nThese features enhance the integration between commands and Ash actions, making it easier to map command fields to action parameters and to ensure that commands meet your business rules.","ref":"parameter_handling.html"},{"type":"extras","title":"Parameter Transformation - Parameter Transformation and Validation","doc":"Parameter transformation allows you to modify command parameters before they're sent to Ash actions. This is useful for:\n\n- Renaming fields to match action expectations\n- Type conversion (e.g., string to integer)\n- Computing new fields\n- Setting default values\n- Applying complex transformations","ref":"parameter_handling.html#parameter-transformation"},{"type":"extras","title":"Basic Usage - Parameter Transformation and Validation","doc":"```elixir\ndefmodule MyApp.User do\n  use Ash.Resource,\n    extensions: [AshCommanded.Commanded.Dsl]\n\n  commanded do\n    commands do\n      command :register_user do\n        fields [:id, :name, :email, :birthdate]\n        \n        transform_params do\n          # Map a field from one name to another\n          map :name, to: :full_name\n          \n          # Cast a field to a specific type\n          cast :birthdate, :date\n          \n          # Compute a new field\n          compute :display_name, fn params ->\n            \"#{params.full_name} <#{params.email}>\"\n          end\n          \n          # Transform a field value\n          transform :email, &String.downcase/1\n          \n          # Set default values\n          default :status, \"active\"\n          default :created_at, &DateTime.utc_now/0\n          \n          # Custom transformation for all params\n          custom fn params ->\n            # Additional custom transformations...\n            params\n          end\n        end\n      end\n    end\n  end\nend\n```","ref":"parameter_handling.html#basic-usage"},{"type":"extras","title":"Transformation Types - Parameter Transformation and Validation","doc":"| Transformation | Description | Example |\n|----------------|-------------|---------|\n| `map` | Renames a field | `map :name, to: :full_name` |\n| `cast` | Converts a field to a specific type | `cast :age, :integer` |\n| `compute` | Creates a new field using a function | `compute :display_name, fn p -> \"#{p.name}\" end` |\n| `transform` | Applies a function to a field | `transform :email, &String.downcase/1` |\n| `default` | Sets a default value if the field is nil | `default :status, \"active\"` |\n| `custom` | Applies a function to the entire params map | `custom fn p -> Map.put(p, :key, \"value\") end` |","ref":"parameter_handling.html#transformation-types"},{"type":"extras","title":"Type Casting - Parameter Transformation and Validation","doc":"AshCommanded supports casting fields to the following types:\n\n- `:string`\n- `:integer`\n- `:float`\n- `:boolean`\n- `:date`\n- `:datetime`\n- `:atom`\n- `:list`\n- `:map`\n\nFor example:\n\n```elixir\ntransform_params do\n  # Convert string to integer\n  cast :age, :integer\n  \n  # Convert string to datetime\n  cast :created_at, :datetime\n  \n  # Convert string to boolean\n  cast :active, :boolean\nend\n```","ref":"parameter_handling.html#type-casting"},{"type":"extras","title":"Parameter Validation - Parameter Transformation and Validation","doc":"Parameter validation ensures that command data meets specific criteria before processing. This allows you to:\n\n- Ensure required fields are present\n- Validate field formats (e.g., email, date)\n- Check value ranges (e.g., minimum/maximum)\n- Apply domain-specific validation rules\n- Prevent invalid commands from being processed","ref":"parameter_handling.html#parameter-validation"},{"type":"extras","title":"Basic Usage - Parameter Transformation and Validation","doc":"```elixir\ndefmodule MyApp.User do\n  use Ash.Resource,\n    extensions: [AshCommanded.Commanded.Dsl]\n\n  commanded do\n    commands do\n      command :register_user do\n        fields [:id, :name, :email, :age, :password]\n        \n        validate_params do\n          # Type validation\n          validate :name, type: :string\n          validate :email, type: :string\n          validate :age, type: :integer\n          \n          # Format validation\n          validate :email, format: ~r/^[^\\s]+@[^\\s]+\\.[^\\s]+$/\n          \n          # Range validation\n          validate :age, min: 18, max: 120\n          \n          # Length validation\n          validate :name, min_length: 2, max_length: 100\n          \n          # Custom validation\n          validate :password, fn value ->\n            cond do\n              String.length(value) < 8 ->\n                {:error, \"Password must be at least 8 characters\"}\n              not String.match?(value, ~r/[A-Z]/) ->\n                {:error, \"Password must contain an uppercase letter\"}\n              not String.match?(value, ~r/[0-9]/) ->\n                {:error, \"Password must contain a number\"}\n              true ->\n                :ok\n            end\n          end\n        end\n      end\n    end\n  end\nend\n```","ref":"parameter_handling.html#basic-usage"},{"type":"extras","title":"Validation Types - Parameter Transformation and Validation","doc":"| Validation | Description | Example |\n|------------|-------------|---------|\n| `type` | Checks the field's type | `validate :age, type: :integer` |\n| `format` | Validates against a regular expression | `validate :email, format: ~r/@/` |\n| `min`/`max` | Checks numeric range | `validate :age, min: 18, max: 120` |\n| `min_length`/`max_length` | Checks string length | `validate :name, min_length: 2` |\n| `min_items`/`max_items` | Checks list item count | `validate :tags, min_items: 1` |\n| `one_of` | Checks value against a list | `validate :role, one_of: [:user, :admin]` |\n| `subset_of` | For lists, checks all values | `validate :roles, subset_of: [:user, :admin]` |\n| Custom function | Applies arbitrary validation logic | `validate :field, fn value -> ... end` |","ref":"parameter_handling.html#validation-types"},{"type":"extras","title":"Combining Multiple Validations - Parameter Transformation and Validation","doc":"You can apply multiple validations to a single field:\n\n```elixir\nvalidate_params do\n  validate :password do\n    min_length 8\n    format ~r/[A-Z]/  # Contains uppercase letter\n    format ~r/[a-z]/  # Contains lowercase letter\n    format ~r/[0-9]/  # Contains number\n    format ~r/[^A-Za-z0-9]/  # Contains special character\n  end\nend\n```","ref":"parameter_handling.html#combining-multiple-validations"},{"type":"extras","title":"Integration with Command Middleware - Parameter Transformation and Validation","doc":"Both parameter transformation and validation can also be implemented as middleware. The built-in `ValidationMiddleware` provides similar functionality to the `validate_params` DSL:\n\n```elixir\ndefmodule MyApp.User do\n  use Ash.Resource,\n    extensions: [AshCommanded.Commanded.Dsl]\n\n  commanded do\n    commands do\n      command :register_user do\n        fields [:id, :name, :email, :age]\n        \n        middleware [\n          {AshCommanded.Commanded.Middleware.ValidationMiddleware,\n            required: [:name, :email, :age],\n            format: [email: ~r/@/],\n            validate: fn command ->\n              if command.age < 18 do\n                {:error, \"User must be at least 18 years old\"}\n              else\n                :ok\n              end\n            end\n          }\n        ]\n      end\n    end\n  end\nend\n```","ref":"parameter_handling.html#integration-with-command-middleware"},{"type":"extras","title":"Benefits Over Basic Mapping - Parameter Transformation and Validation","doc":"The parameter transformation and validation system provides several advantages over basic field mapping:\n\n1. **Type Safety**: Automatic type conversion ensures values match expected types\n2. **Data Normalization**: Fields can be consistently transformed (e.g., lowercase emails)\n3. **Computation**: Derived fields can be calculated based on input parameters\n4. **Validation**: Problems are caught early in the command lifecycle\n5. **Security**: Input can be sanitized and validated before processing\n6. **Default Values**: Missing or nil values can be automatically populated\n7. **Contextual Processing**: Command parameters can be transformed based on other parameters\n\nBy using these features, you can create a more robust integration between your commands and Ash actions, with clear validation rules and transformation logic.","ref":"parameter_handling.html#benefits-over-basic-mapping"},{"type":"extras","title":"Transactions","doc":"# Transactions\n\nAshCommanded provides transaction support for commands, allowing you to execute commands atomically and ensure consistency across multiple commands or actions.","ref":"transactions.html"},{"type":"extras","title":"Configuring Transactions for Commands - Transactions","doc":"Transactions can be configured at the command level in the `commanded` DSL:\n\n```elixir\ndefmodule MyApp.Order do\n  use Ash.Resource,\n    extensions: [AshCommanded.Commanded.Dsl]\n\n  commanded do\n    commands do\n      # Enable transactions with inline options\n      command :create_order do\n        fields [:id, :customer_id, :total]\n        identity_field :id\n        in_transaction? true\n        repo MyApp.Repo\n        transaction_timeout 5000\n        transaction_isolation_level :read_committed\n      end\n\n      # Enable transactions with block syntax\n      command :update_order do\n        fields [:id, :total]\n        identity_field :id\n        \n        transaction do\n          enabled? true\n          repo MyApp.Repo\n          timeout 5000\n          isolation_level :read_committed\n        end\n      end\n    end\n  end\nend\n```","ref":"transactions.html#configuring-transactions-for-commands"},{"type":"extras","title":"Transaction Options - Transactions","doc":"You can configure the following transaction options:","ref":"transactions.html#transaction-options"},{"type":"extras","title":"Command-level Options - Transactions","doc":"- `in_transaction?`: Whether to execute the command in a transaction (boolean, default: false)\n- `repo`: The Ecto repository to use for transactions (atom, required if `in_transaction?` is true)\n- `transaction_timeout`: The transaction timeout in milliseconds (number, optional)\n- `transaction_isolation_level`: The transaction isolation level (atom, optional)","ref":"transactions.html#command-level-options"},{"type":"extras","title":"Block Syntax Options - Transactions","doc":"Using the `transaction` block, you can configure:\n\n```elixir\ntransaction do\n  enabled? true             # Enable/disable transactions for this command\n  repo MyApp.Repo           # The Ecto repository to use\n  timeout 5000              # Timeout in milliseconds\n  isolation_level :serializable  # Isolation level (:read_committed, :repeatable_read, etc.)\nend\n```","ref":"transactions.html#block-syntax-options"},{"type":"extras","title":"Resource-level Default Options - Transactions","doc":"You can also set default transaction options at the resource level:\n\n```elixir\ndefmodule MyApp.Domain do\n  use Ash.Domain\n\n  commanded do\n    commands do\n      # Default transaction options for all commands in this domain\n      default_repo MyApp.Repo\n      default_transaction_timeout 10000\n      default_transaction_isolation_level :read_committed\n    end\n  end\nend\n```","ref":"transactions.html#resource-level-default-options"},{"type":"extras","title":"Multi-Command Transactions - Transactions","doc":"For complex operations that require multiple commands to be executed atomically, AshCommanded provides support for multi-command transactions:\n\n```elixir\n# Execute multiple commands in a single transaction\n{:ok, results} = AshCommanded.Commanded.Transaction.execute_commands(\n  MyApp.Repo,\n  [\n    %MyApp.Commands.CreateOrder{id: \"order-123\", customer_id: \"cust-456\", total: 100.00},\n    %MyApp.Commands.CreateOrderItem{order_id: \"order-123\", product_id: \"prod-789\", quantity: 2}\n  ],\n  timeout: 5000,\n  isolation_level: :read_committed\n)\n```\n\nThe `execute_commands/3` function:\n1. Executes all commands within a single database transaction\n2. Returns all results if successful, or rolls back the entire transaction on any error\n3. Takes the same transaction options as individual commands","ref":"transactions.html#multi-command-transactions"},{"type":"extras","title":"Error Handling - Transactions","doc":"When a command executed in a transaction encounters an error, the entire transaction is rolled back and an error is returned:\n\n```elixir\n{:error, %AshCommanded.Commanded.Error{\n  type: :command_error,\n  message: \"Transaction failed: some error message\",\n  context: %{error: \"...\"}\n}}\n```\n\nThe error will include details about which command failed and why, allowing for proper error handling and reporting.","ref":"transactions.html#error-handling"},{"type":"extras","title":"Transaction Validation - Transactions","doc":"Before executing a command in a transaction, AshCommanded validates:\n\n1. That the specified repository exists and is configured\n2. That the repository supports transactions\n3. That any required transaction options are properly set\n\nThis validation helps prevent runtime errors and ensures that transactions are properly configured.","ref":"transactions.html#transaction-validation"},{"type":"extras","title":"Usage Recommendations - Transactions","doc":"Consider using transactions when:\n\n1. A command needs to make changes to multiple resources atomically\n2. Multiple commands need to be executed as a single unit of work\n3. You need to ensure that either all changes succeed or none do\n\nTransactions are particularly valuable in domains like e-commerce, banking, and inventory management where consistency is critical.","ref":"transactions.html#usage-recommendations"},{"type":"extras","title":"Context Propagation","doc":"# Context Propagation\n\nAshCommanded provides comprehensive context propagation between commands and actions, allowing for rich information sharing throughout the command execution pipeline.","ref":"context_propagation.html"},{"type":"extras","title":"Overview - Context Propagation","doc":"Context propagation enables:\n\n1. Access to the aggregate state within action execution\n2. Access to the original command in actions\n3. Metadata sharing between commands and actions\n4. Custom static context values\n5. Flexible context key prefixing to prevent collisions\n\nThis mechanism ensures that actions have full access to the command execution context, improving traceability and enabling more sophisticated behavior.","ref":"context_propagation.html#overview"},{"type":"extras","title":"Configuring Context Propagation - Context Propagation","doc":"Context propagation can be configured at the command level in the `commanded` DSL:\n\n```elixir\ndefmodule MyApp.User do\n  use Ash.Resource,\n    extensions: [AshCommanded.Commanded.Dsl]\n\n  commanded do\n    commands do\n      command :register_user do\n        fields [:id, :email, :name]\n        identity_field :id\n        \n        # Context propagation options\n        include_aggregate? true \n        include_command? true\n        include_metadata? true\n        context_prefix :cmd\n        static_context %{source: :web_api}\n      end\n    end\n  end\nend\n```","ref":"context_propagation.html#configuring-context-propagation"},{"type":"extras","title":"Context Options - Context Propagation","doc":"The following options control context propagation:\n\n- `include_aggregate?` - Whether to include the aggregate state in the action context (default: `true`)\n- `include_command?` - Whether to include the command struct in the action context (default: `true`)\n- `include_metadata?` - Whether to include command metadata in the action context (default: `true`)\n- `context_prefix` - An optional prefix for context keys to prevent collisions (default: `nil`)\n- `static_context` - Static context values to include in every action execution (default: `%{}`)","ref":"context_propagation.html#context-options"},{"type":"extras","title":"Context Structure - Context Propagation","doc":"By default, the following context keys are included:\n\n- `:aggregate` - The current state of the aggregate\n- `:command` - The command being executed\n- `:metadata` - Command metadata (if present)\n\nWhen a `context_prefix` is specified, the keys become:\n\n- `:\"prefix.aggregate\"` - The current state of the aggregate\n- `:\"prefix.command\"` - The command being executed\n- `:\"prefix.metadata\"` - Command metadata (if present)\n\nAny values in `static_context` are merged directly into the context map.","ref":"context_propagation.html#context-structure"},{"type":"extras","title":"Accessing Context in Actions - Context Propagation","doc":"Within your Ash actions, you can access the context as follows:\n\n```elixir\ndefmodule MyApp.UserActions do\n  def create_user(changeset) do\n    # Get context from the changeset\n    context = Ash.Changeset.get_context(changeset)\n    \n    # Access aggregate state (if included)\n    aggregate = Map.get(context, :aggregate)\n    \n    # Access command (if included)\n    command = Map.get(context, :command)\n    \n    # Access metadata (if included)\n    metadata = Map.get(context, :metadata)\n    \n    # Access static context values\n    source = Map.get(context, :source)\n    \n    # Use context data in your action logic\n    # ...\n    \n    changeset\n  end\nend\n```\n\nWith a `context_prefix`, you would access the keys with the prefix:\n\n```elixir\n# With context_prefix: :cmd\naggregate = Map.get(context, :\"cmd.aggregate\")\ncommand = Map.get(context, :\"cmd.command\")\nmetadata = Map.get(context, :\"cmd.metadata\")\n```","ref":"context_propagation.html#accessing-context-in-actions"},{"type":"extras","title":"Context and Event Generation - Context Propagation","doc":"When commands are processed, the action result can influence the generated event:\n\n```elixir\ndefmodule MyApp.UserActions do\n  def create_user(changeset) do\n    # Return additional data to include in the event\n    # This will be merged with the command fields\n    {:ok, %{registered_at: DateTime.utc_now()}}\n  end\nend\n```\n\nThe additional map returned by the action will be merged with the command fields when generating the event, allowing for dynamic event enrichment.","ref":"context_propagation.html#context-and-event-generation"},{"type":"extras","title":"Context in Middleware - Context Propagation","doc":"Command middleware also has access to the context, enabling powerful pattern like:\n\n- Audit logging using command, metadata, and aggregate state\n- Authorization based on command and metadata\n- Dynamic command transformation based on context\n- Context-aware validation","ref":"context_propagation.html#context-in-middleware"},{"type":"extras","title":"Example: Using Context in Practice - Context Propagation","doc":"```elixir\ndefmodule MyApp.User do\n  use Ash.Resource,\n    extensions: [AshCommanded.Commanded.Dsl]\n\n  attributes do\n    uuid_primary_key :id\n    attribute :email, :string\n    attribute :name, :string\n    attribute :status, :string\n    attribute :created_at, :utc_datetime\n  end\n\n  actions do\n    create :create do\n      accept [:id, :email, :name]\n      change create_timestamp()\n    end\n    \n    update :update_email do\n      accept [:email]\n      change update_timestamp()\n    end\n  end\n  \n  # Action helpers using context\n  defp create_timestamp(changeset) do\n    context = Ash.Changeset.get_context(changeset)\n    source = Map.get(context, :source, :unknown)\n    \n    changeset\n    |> Ash.Changeset.change_attribute(:created_at, DateTime.utc_now())\n    |> Ash.Changeset.put_context(:creation_source, source)\n  end\n  \n  defp update_timestamp(changeset) do\n    context = Ash.Changeset.get_context(changeset)\n    command = Map.get(context, :command)\n    \n    timestamp = DateTime.utc_now()\n    \n    changeset\n    |> Ash.Changeset.change_attribute(:updated_at, timestamp)\n    |> Ash.Changeset.put_metadata(:last_change, %{\n      field: :email,\n      from: Map.get(context.aggregate || %{}, :email),\n      to: command.email,\n      timestamp: timestamp\n    })\n  end\n  \n  commanded do\n    commands do\n      command :register_user do\n        fields [:id, :email, :name]\n        identity_field :id\n        action :create\n        static_context %{source: :registration_form}\n      end\n      \n      command :update_email do\n        fields [:id, :email]\n        identity_field :id\n        action :update_email\n        context_prefix :user\n      end\n    end\n    \n    events do\n      event :user_registered do\n        fields [:id, :email, :name, :created_at]\n      end\n      \n      event :email_updated do\n        fields [:id, :email, :updated_at]\n      end\n    end\n  end\nend\n```","ref":"context_propagation.html#example-using-context-in-practice"},{"type":"extras","title":"Benefits of Context Propagation - Context Propagation","doc":"1. **Richer Domain Logic** - Actions can incorporate command and aggregate information\n2. **Improved Auditing** - Full command context for better traceability\n3. **Better Event Generation** - Events can include action-specific data\n4. **Reduced Duplication** - No need to repeat data already in commands or aggregates\n5. **Context-Aware Behavior** - Actions can adapt based on context information","ref":"context_propagation.html#benefits-of-context-propagation"},{"type":"extras","title":"Best Practices - Context Propagation","doc":"1. Use `context_prefix` when integrating with other extensions to avoid key collisions\n2. Keep the `static_context` map small and focused on truly static values\n3. Use `include_aggregate?` and `include_command?` selectively for large objects\n4. Document which context keys your actions depend on\n5. Consider using middleware for cross-cutting concerns rather than action-specific context handling","ref":"context_propagation.html#best-practices"},{"type":"extras","title":"Error Handling in AshCommanded","doc":"# Error Handling in AshCommanded\n\nThis document explains the error handling system in AshCommanded, which provides standardized, structured error reports and consistent error handling across the entire framework.","ref":"error_handling.html"},{"type":"extras","title":"Overview - Error Handling in AshCommanded","doc":"AshCommanded provides a standardized approach to error handling through the `AshCommanded.Commanded.Error` module. This system ensures that:\n\n1. Errors have a consistent structure across the framework\n2. Error messages are user-friendly and informative\n3. Developers can easily identify the source and context of errors\n4. Errors can be properly formatted for display to users\n5. Errors from different sources (Ash, Commanded) are normalized to a common format","ref":"error_handling.html#overview"},{"type":"extras","title":"Error Structure - Error Handling in AshCommanded","doc":"All errors in AshCommanded are represented using the `AshCommanded.Commanded.Error` struct, which has the following fields:\n\n| Field | Type | Description |\n|-------|------|-------------|\n| `type` | atom | The category of error (e.g., `:validation_error`, `:command_error`) |\n| `message` | string | A human-readable error message |\n| `path` | list | Path to the error in a data structure (for nested errors) |\n| `field` | atom | The specific field that caused the error (if applicable) |\n| `value` | any | The value that caused the error (if applicable) |\n| `context` | map | Additional contextual information about the error |","ref":"error_handling.html#error-structure"},{"type":"extras","title":"Error Types - Error Handling in AshCommanded","doc":"The following error types are used throughout AshCommanded:\n\n| Type | Description | Common Uses |\n|------|-------------|-------------|\n| `:validation_error` | Error validating command parameters | Parameter validation failures |\n| `:transformation_error` | Error transforming command parameters | Type casting failures, computation errors |\n| `:command_error` | Error related to command structure or processing | Missing required fields, invalid command structure |\n| `:aggregate_error` | Error in aggregate processing | Command execution failures in aggregates |\n| `:dispatch_error` | Error dispatching a command | Router errors, event store errors |\n| `:action_error` | Error executing an Ash action | Resource action failures |\n| `:projection_error` | Error applying a projection | Event handling errors in projectors |","ref":"error_handling.html#error-types"},{"type":"extras","title":"Creating Errors - Error Handling in AshCommanded","doc":"You can create errors using the constructor functions in the `AshCommanded.Commanded.Error` module:\n\n```elixir\n# General constructor\nError.new(:validation_error, \"Value must be positive\", field: :age, value: -5)\n\n# Type-specific constructors\nError.validation_error(\"Value must be positive\", field: :age, value: -5)\nError.transformation_error(\"Failed to cast to integer\", field: :age, value: \"abc\")\nError.command_error(\"Missing required field\", field: :id)\nError.aggregate_error(\"Aggregate not found\", context: %{aggregate_id: \"123\"})\nError.action_error(\"Failed to execute action\", context: %{action: :create_user})\n```","ref":"error_handling.html#creating-errors"},{"type":"extras","title":"Error Normalization - Error Handling in AshCommanded","doc":"AshCommanded automatically normalizes errors from different sources into the standard format. This is handled through the `normalize_error/1` and `normalize_errors/1` functions:\n\n```elixir\n# Convert an Ash error to AshCommanded format\nash_error = %Ash.Error.Invalid{errors: [%Ash.Error.Changes.InvalidAttribute{field: :name, message: \"can't be blank\"}]}\nnormalized_error = Error.normalize_error(ash_error)\n\n# Convert a Commanded error to AshCommanded format\ncommanded_error = %Commanded.Aggregates.ExecutionError{message: \"Failed to execute command\"}\nnormalized_error = Error.normalize_error(commanded_error)\n\n# Normalize a list of mixed errors\nerrors = [ash_error, commanded_error, \"Simple string error\"]\nnormalized_errors = Error.normalize_errors(errors)\n```","ref":"error_handling.html#error-normalization"},{"type":"extras","title":"Error Formatting - Error Handling in AshCommanded","doc":"To display errors in a human-readable format, use the `format/1` function:\n\n```elixir\nerror = Error.validation_error(\"Value must be positive\", field: :age, value: -5)\nformatted = Error.format(error)\n# => \"Validation error: Value must be positive (field: age, value: -5)\"\n```","ref":"error_handling.html#error-formatting"},{"type":"extras","title":"Error Handling in Components - Error Handling in AshCommanded","doc":"","ref":"error_handling.html#error-handling-in-components"},{"type":"extras","title":"Parameter Validation - Error Handling in AshCommanded","doc":"The parameter validator uses standardized error handling:\n\n```elixir\nparams = %{name: \"John\", email: \"invalid-email\", age: 15}\nvalidations = [\n  {:validate, :name, [min_length: 5]},\n  {:validate, :email, [format: ~r/@.*\\./]},\n  {:validate, :age, [min: 18]}\n]\n\ncase AshCommanded.Commanded.ParameterValidator.validate_params(params, validations) do\n  :ok ->\n    # Proceed with valid parameters\n    \n  {:error, errors} ->\n    # Handle validation errors\n    formatted_errors = Enum.map(errors, &Error.format/1)\n    IO.puts(\"Validation failed: #{Enum.join(formatted_errors, \", \")}\")\nend\n```","ref":"error_handling.html#parameter-validation"},{"type":"extras","title":"Command Action Mapper - Error Handling in AshCommanded","doc":"The command action mapper handles errors throughout the command execution process:\n\n```elixir\nresult = AshCommanded.Commanded.CommandActionMapper.map_to_action(\n  command,\n  MyApp.User,\n  :create_user,\n  transforms: transforms,\n  validations: validations\n)\n\ncase result do\n  {:ok, record} ->\n    # Command executed successfully\n    \n  {:error, errors} when is_list(errors) ->\n    # Multiple errors occurred\n    formatted_errors = Enum.map(errors, &Error.format/1)\n    IO.puts(\"Command failed: #{Enum.join(formatted_errors, \", \")}\")\n    \n  {:error, error} ->\n    # A single error occurred\n    IO.puts(\"Command failed: #{Error.format(error)}\")\nend\n```","ref":"error_handling.html#command-action-mapper"},{"type":"extras","title":"Aggregate Error Handling - Error Handling in AshCommanded","doc":"The generated aggregate modules include standardized error handling:\n\n```elixir\ndef execute(%__MODULE__{} = aggregate, %MyApp.Commands.RegisterUser{} = command) do\n  try do\n    # Command handling logic...\n  rescue\n    e in _ ->\n      {:error, Error.aggregate_error(\"Error executing command: #{Exception.message(e)}\", \n        context: %{command: command.__struct__, error: inspect(e)})}\n  end\nend\n```","ref":"error_handling.html#aggregate-error-handling"},{"type":"extras","title":"Extracting Errors from Results - Error Handling in AshCommanded","doc":"To extract errors from command results, use the `errors_from_result/1` function:\n\n```elixir\nresult = AshCommanded.Commands.RegisterUser.execute(command)\n\nerrors = Error.errors_from_result(result)\nif Enum.empty?(errors) do\n  IO.puts(\"Command succeeded!\")\nelse\n  formatted_errors = Enum.map(errors, &Error.format/1)\n  IO.puts(\"Command failed: #{Enum.join(formatted_errors, \", \")}\")\nend\n```","ref":"error_handling.html#extracting-errors-from-results"},{"type":"extras","title":"Best Practices - Error Handling in AshCommanded","doc":"1. **Always use standardized errors**: Use the `Error` module for all error creation rather than returning raw atoms or strings.\n\n2. **Include contextual information**: When creating errors, include relevant context such as field names, values, and additional contextual data.\n\n3. **Handle errors gracefully**: Design your code to handle errors properly, using pattern matching to extract error information.\n\n4. **Use proper error types**: Choose the appropriate error type for each situation to help with debugging and error handling.\n\n5. **Format errors for display**: When displaying errors to users, use the `format/1` function to create readable error messages.","ref":"error_handling.html#best-practices"},{"type":"extras","title":"Integration with Ash Framework - Error Handling in AshCommanded","doc":"AshCommanded automatically converts Ash errors to the standardized format, making it easier to integrate with existing Ash resources:\n\n```elixir\ncase AshCommanded.Commanded.CommandActionMapper.map_to_action(command, resource, action_name) do\n  {:ok, result} ->\n    # Success\n    \n  {:error, errors} ->\n    # Both Ash errors and AshCommanded errors will be properly formatted\n    formatted_errors = Enum.map(Error.normalize_errors(errors), &Error.format/1)\nend\n```","ref":"error_handling.html#integration-with-ash-framework"},{"type":"extras","title":"AshCommanded DSL Cheatsheet","doc":"# AshCommanded DSL Cheatsheet","ref":"ashcommanded-commanded-dsl.html"},{"type":"extras","title":"Resource Configuration - AshCommanded DSL Cheatsheet","doc":"```elixir\ndefmodule MyApp.User do\n  use Ash.Resource,\n    extensions: [AshCommanded.Commanded.Dsl]\n    \n  # Resource configuration...\n    \n  commanded do\n    # Commanded DSL configuration...\n  end\nend\n```","ref":"ashcommanded-commanded-dsl.html#resource-configuration"},{"type":"extras","title":"Command Definition - AshCommanded DSL Cheatsheet","doc":"```elixir\ncommanded do\n  commands do\n    command :register_user do\n      fields [:id, :name, :email]\n      identity_field :id\n      action :register\n      \n      # Optional settings\n      command_name :RegisterUser\n      autogenerate_handler? true\n      handler_name :handle_register\n      in_transaction? true\n      repo MyApp.Repo\n      \n      # Advanced options\n      middleware [MyApp.LoggingMiddleware]\n      \n      # Parameter transformations\n      transform_params do\n        map email: :normalized_email\n        compute :timestamp, &DateTime.utc_now/0\n      end\n      \n      # Parameter validations\n      validate_params do\n        validate :email, format: ~r/@/\n        validate :name, present: true\n      end\n      \n      # Alternative transaction syntax\n      transaction do\n        enabled? true\n        repo MyApp.Repo\n        timeout 5000\n        isolation_level :read_committed\n      end\n      \n      # Context options\n      include_aggregate? true\n      include_command? true\n      include_metadata? true\n      context_prefix :cmd\n    end\n  end\nend\n```","ref":"ashcommanded-commanded-dsl.html#command-definition"},{"type":"extras","title":"Event Definition - AshCommanded DSL Cheatsheet","doc":"```elixir\ncommanded do\n  events do\n    event :user_registered do\n      fields [:id, :name, :email]\n      \n      # Optional settings\n      event_name :UserRegistered\n    end\n  end\nend\n```","ref":"ashcommanded-commanded-dsl.html#event-definition"},{"type":"extras","title":"Projection Definition - AshCommanded DSL Cheatsheet","doc":"```elixir\ncommanded do\n  projections do\n    projection :user_registered do\n      # Define the Ash action to invoke\n      action :create\n      \n      # Static changes\n      changes(%{\n        status: \"active\"\n      })\n      \n      # Or dynamic changes via function\n      changes(fn event ->\n        %{\n          id: event.id,\n          name: event.name,\n          email: event.email,\n          registered_at: DateTime.utc_now()\n        }\n      end)\n      \n      # Optional settings\n      autogenerate? true\n      projector_name :CustomUserProjector\n    end\n  end\nend\n```","ref":"ashcommanded-commanded-dsl.html#projection-definition"},{"type":"extras","title":"Event Handler Definition - AshCommanded DSL Cheatsheet","doc":"```elixir\ncommanded do\n  event_handlers do\n    # Function-based handler\n    handler :notification_handler do\n      events [:user_registered]\n      action fn event, _metadata ->\n        MyApp.Notifications.send_welcome_email(event.email)\n        :ok\n      end\n    end\n    \n    # Ash action handler\n    handler :external_sync do\n      events [:user_registered]\n      action :sync_to_crm\n      idempotent true\n    end\n    \n    # PubSub publishing handler\n    handler :event_broadcaster do\n      events [:user_registered, :email_changed]\n      publish_to \"user_events\"\n    end\n    \n    # Optional settings\n    handler :custom_name_handler do\n      events [:user_registered]\n      action fn _, _ -> :ok end\n      handler_name :CustomHandler\n      autogenerate? true\n    end\n  end\nend\n```","ref":"ashcommanded-commanded-dsl.html#event-handler-definition"},{"type":"extras","title":"Application Configuration - AshCommanded DSL Cheatsheet","doc":"```elixir\ndefmodule MyApp.Domain do\n  use Ash.Domain, extensions: [AshCommanded.Commanded.Dsl]\n  \n  resources do\n    resource MyApp.User\n  end\n  \n  commanded do\n    application do\n      otp_app :my_app\n      event_store Commanded.EventStore.Adapters.EventStore\n      \n      # Optional settings\n      pubsub :local\n      registry :local\n      include_supervisor? true\n      \n      # Snapshotting\n      snapshotting true\n      snapshot_threshold 100\n      snapshot_version 1\n      snapshot_store MyApp.CustomSnapshotStore\n    end\n  end\nend\n```","ref":"ashcommanded-commanded-dsl.html#application-configuration"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.3"}}